@@ -2,24 +2,24 @@
  *
  *  $Id: commands.c,v 1.39 2008/07/03 15:08:34 smorlat Exp $
  *
- *  Copyright (C) 2006  Sandro Santilli <strk@keybit.net>
+ *  Copyright (C) 2006-2009  Sandro Santilli <strk@keybit.net>
  *  Copyright (C) 2004  Simon MORLAT <simon.morlat@linphone.org>
  *
- ****************************************************************************
+****************************************************************************
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  ****************************************************************************/
 
@@ -46,6 +46,7 @@ static int lpc_cmd_help(LinphoneCore *, char *);
 static int lpc_cmd_proxy(LinphoneCore *, char *);
 static int lpc_cmd_call(LinphoneCore *, char *);
 static int lpc_cmd_answer(LinphoneCore *, char *);
+static int lpc_cmd_autoanswer(LinphoneCore *, char *);
 static int lpc_cmd_terminate(LinphoneCore *, char *);
 static int lpc_cmd_call_logs(LinphoneCore *, char *);
 static int lpc_cmd_ipv6(LinphoneCore *, char *);
@@ -58,12 +59,18 @@ static int lpc_cmd_friend(LinphoneCore *, char*);
 static int lpc_cmd_soundcard(LinphoneCore *, char *);
 static int lpc_cmd_play(LinphoneCore *, char *);
 static int lpc_cmd_record(LinphoneCore *, char *);
+static int lpc_cmd_register(LinphoneCore *, char *);
+static int lpc_cmd_unregister(LinphoneCore *, char *);
+static int lpc_cmd_duration(LinphoneCore *lc, char *args);
+static int lpc_cmd_status(LinphoneCore *lc, char *args);
+
 /* Command handler helpers */
 static void linphonec_proxy_add(LinphoneCore *lc);
 static void linphonec_proxy_display(LinphoneProxyConfig *lc);
 static void linphonec_proxy_list(LinphoneCore *lc);
 static void linphonec_proxy_remove(LinphoneCore *lc, int index);
 static  int linphonec_proxy_use(LinphoneCore *lc, int index);
+static void linphonec_proxy_show(LinphoneCore *lc,int index);
 static void linphonec_friend_display(LinphoneFriend *fr);
 static int linphonec_friend_list(LinphoneCore *lc, char *arg);
 static void linphonec_display_command_help(LPC_COMMAND *cmd);
@@ -71,9 +78,15 @@ static int linphonec_friend_call(LinphoneCore *lc, unsigned int num);
 static int linphonec_friend_add(LinphoneCore *lc, const char *name, const char *addr);
 static int linphonec_friend_delete(LinphoneCore *lc, int num);
 
+
+
 /* Command table management */
 static LPC_COMMAND *lpc_find_command(const char *name);
 
+void linphonec_out(const char *fmt,...);
+
+
+
 /***************************************************************************
  *
  *  Global variables
@@ -94,15 +107,22 @@ LPC_COMMAND commands[] = {
 	{ "answer", lpc_cmd_answer, "Answer a call",
 		"Accept an incoming call."
 	},
+	{ "autoanswer", lpc_cmd_autoanswer, "Show/set auto-answer mode",
+		"'autoanswer'       \t: show current autoanswer mode\n"
+		"'autoanswer enable'\t: enable autoanswer mode\n"
+		"'autoanswer disable'\t: disable autoanswer mode \n"},
 	{ "proxy", lpc_cmd_proxy, "Manage proxies",
 		"'proxy list' : list all proxy setups.\n"
 		"'proxy add' : add a new proxy setup.\n"
 		"'proxy remove <index>' : remove proxy setup with number index.\n"
 		"'proxy use <index>' : use proxy with number index as default proxy.\n"
-		"'proxy unuse' : don't use a default proxy."
+		"'proxy unuse' : don't use a default proxy.\n"
+		"'proxy show <index>' : show configuration and status of the proxy numbered by index.\n"
+		"'proxy show default' : show configuration and status of the default proxy.\n"
 	},
 	{ "soundcard", lpc_cmd_soundcard, "Manage soundcards",
 		"'soundcard list' : list all sound devices.\n"
+		"'soundcard show' : show current sound devices configuration.\n"
 		"'soundcard use <index>' : select a sound device.\n"
 		"'soundcard use files' : use .wav files instead of soundcard\n"
 	},
@@ -122,13 +142,13 @@ LPC_COMMAND commands[] = {
 	},
 	{ "stun", lpc_cmd_stun, "Set stun server address",
 		"'stun'        : show stun settings.\n"
-		"'stun <addr>' : set stun address.\n"
+		"'stun <addr>' : set stun server address.\n"
 	},
-	{ "firewall", lpc_cmd_firewall, "Set ",
+	{ "firewall", lpc_cmd_firewall, "Set firewall policy",
 		"'firewall'        : show current firewall policy.\n"
 		"'firewall none'   : use direct connection.\n"
 		"'firewall nat'    : use nat address given with the 'nat' command.\n"
-		"'firewall stun'   : use stun server given with the 'server' command.\n"
+		"'firewall stun'   : use stun server given with the 'stun' command.\n"
 	},
         { "call-logs", lpc_cmd_call_logs, "Calls history",
                 NULL },
@@ -149,6 +169,14 @@ LPC_COMMAND commands[] = {
 		"'record <wav file>'    : record into wav file."
 	},
 	{ "quit", lpc_cmd_quit, "Exit linphonec", NULL },
+	{ "register", lpc_cmd_register, "Register in one line to a proxy" , "register <sip identity> <sip proxy> <password>"},
+	{ "unregister", lpc_cmd_unregister, "Unregister from default proxy", NULL	},
+	{ "duration", lpc_cmd_duration, "Print duration in seconds of the last call.", NULL },
+	{ "status", lpc_cmd_status, "Print various status information", 
+			"'status register'  \t: print status concerning registration\n"
+			"'status autoanswer'\t: tell whether autoanswer mode is enabled\n"
+			"'status hook'      \t: print hook status\n" },
+					
 	{ (char *)NULL, (lpc_cmd_handler)NULL, (char *)NULL, (char *)NULL }
 };
 
@@ -187,7 +215,7 @@ linphonec_parse_command_line(LinphoneCore *lc, char *cl)
 		while ( isdigit(*cl) || *cl == '#' || *cl == '*' )
 		{
 			linphone_core_send_dtmf(lc, *cl);
-			sleep(1); // be nice
+			ms_sleep(1); // be nice
 			++cl;
 		}
 
@@ -199,13 +227,13 @@ linphonec_parse_command_line(LinphoneCore *lc, char *cl)
 	cmd=lpc_find_command(cl);
 	if ( !cmd )
 	{
-		printf("'%s': Cannot understand this.\n", cl);
+		linphonec_out("'%s': Cannot understand this.\n", cl);
 		return 1;
 	}
 
 	if ( ! cmd->func(lc, args) )
 	{
-		printf("Syntax error.\n");
+		linphonec_out("Syntax error.\n");
 		linphonec_display_command_help(cmd);
 	}
 
@@ -262,18 +290,18 @@ lpc_cmd_help(LinphoneCore *lc, char *arg)
 
 	if (!arg || !*arg)
 	{
-		printf("Commands are:\n");
-		printf("---------------------------\n");
+		linphonec_out("Commands are:\n");
+		linphonec_out("---------------------------\n");
 
 		while (commands[i].help)
 		{
-			printf("%10.10s\t%s\n", commands[i].name,
+			linphonec_out("%10.10s\t%s\n", commands[i].name,
 				commands[i].help);
 			i++;
 		}
 		
-		printf("---------------------------\n");
-		printf("Type 'help <command>' for more details.\n");
+		linphonec_out("---------------------------\n");
+		linphonec_out("Type 'help <command>' for more details.\n");
 
 		return 1;
 	}
@@ -281,7 +309,7 @@ lpc_cmd_help(LinphoneCore *lc, char *arg)
 	cmd=lpc_find_command(arg);
 	if ( !cmd )
 	{
-		printf("No such command.\n");
+		linphonec_out("No such command.\n");
 		return 1;
 	}
 
@@ -290,6 +318,8 @@ lpc_cmd_help(LinphoneCore *lc, char *arg)
 
 }
 
+static char callee_name[256]={0};
+
 static int
 lpc_cmd_call(LinphoneCore *lc, char *args)
 {
@@ -300,29 +330,33 @@ lpc_cmd_call(LinphoneCore *lc, char *args)
 
 	if ( lc->call != NULL )
 	{
-		printf("Terminate current call first.\n");
+		linphonec_out("Terminate current call first.\n");
 	}
 	else
 	{
 		if ( -1 == linphone_core_invite(lc, args) )
 		{
-			printf("Error from linphone_core_invite.\n");
+			linphonec_out("Error from linphone_core_invite.\n");
 		}
 		else
 		{
-			/* current_call=args; */
+			snprintf(callee_name,sizeof(callee_name),"%s",args);
 		}
 	}
 	return 1;
 }
 
+static const char *linphonec_get_callee(){
+	return callee_name;
+}
+
 static int
 lpc_cmd_refer(LinphoneCore *lc, char *args)
 {
 	if (args)
 		linphone_core_refer(lc, args);
 	else{
-		printf("refer needs an argument\n");
+		linphonec_out("refer needs an argument\n");
 	}
 	return 1;
 }
@@ -332,7 +366,7 @@ lpc_cmd_terminate(LinphoneCore *lc, char *args)
 {
 	if ( -1 == linphone_core_terminate_call(lc, NULL) )
 	{
-		printf("No active call.\n");
+		linphonec_out("No active call.\n");
 	}
 	return 1;
 }
@@ -342,11 +376,34 @@ lpc_cmd_answer(LinphoneCore *lc, char *args)
 {
 	if ( -1 == linphone_core_accept_call(lc, NULL) )
 	{
-		printf("No incoming call.\n");
+		linphonec_out("No incoming call.\n");
 	}
 	return 1;
 }
 
+static int
+lpc_cmd_autoanswer(LinphoneCore *lc, char *args)
+{
+	if ( ! args )
+	{
+		if ( linphonec_get_autoanswer() ) {
+			linphonec_out("Auto answer is enabled. Use 'autoanswer disable' to disable.\n");
+		} else {
+			linphonec_out("Auto answer is disabled. Use 'autoanswer enable' to enable.\n");
+		}
+		return 1;
+	}
+
+	if (strstr(args,"enable")){
+		linphonec_set_autoanswer(TRUE);
+		linphonec_out("Auto answer disabled.\n");
+	}else if (strstr(args,"disable")){
+		linphonec_set_autoanswer(FALSE);
+		linphonec_out("Auto answer enabled.\n");
+	}else return 0;
+	return 1;
+}
+
 static int
 lpc_cmd_quit(LinphoneCore *lc, char *args)
 {
@@ -370,7 +427,7 @@ lpc_cmd_nat(LinphoneCore *lc, char *args)
 
 	nat = linphone_core_get_nat_address(lc);
 	use = linphone_core_get_firewall_policy(lc)==LINPHONE_POLICY_USE_NAT_ADDRESS;
-	printf("Nat address: %s%s\n", nat ? nat : "unspecified" , use ? "" : " (disabled - use 'firewall nat' to enable)");
+	linphonec_out("Nat address: %s%s\n", nat ? nat : "unspecified" , use ? "" : " (disabled - use 'firewall nat' to enable)");
 
 	return 1;
 }
@@ -391,7 +448,7 @@ lpc_cmd_stun(LinphoneCore *lc, char *args)
 
 	stun = linphone_core_get_stun_server(lc);
 	use = linphone_core_get_firewall_policy(lc)==LINPHONE_POLICY_USE_STUN;
-	printf("Stun server: %s%s\n", stun ? stun : "unspecified" , use? "" : " (disabled - use 'firewall stun' to enable)");
+	linphonec_out("Stun server: %s%s\n", stun ? stun : "unspecified" , use? "" : " (disabled - use 'firewall stun' to enable)");
 
 	return 1;
 }
@@ -414,7 +471,7 @@ lpc_cmd_firewall(LinphoneCore *lc, char *args)
 			setting = linphone_core_get_stun_server(lc);
 			if ( ! setting )
 			{
-				printf("No stun server address is defined, use 'stun <address>' first");
+				linphonec_out("No stun server address is defined, use 'stun <address>' first\n");
 				return 1;
 			}
 			linphone_core_set_firewall_policy(lc,LINPHONE_POLICY_USE_STUN);
@@ -424,7 +481,7 @@ lpc_cmd_firewall(LinphoneCore *lc, char *args)
 			setting = linphone_core_get_nat_address(lc);
 			if ( ! setting )
 			{
-				printf("No nat address is defined, use 'nat <address>' first");
+				linphonec_out("No nat address is defined, use 'nat <address>' first");
 				return 1;
 			}
 			linphone_core_set_firewall_policy(lc,LINPHONE_POLICY_USE_NAT_ADDRESS);
@@ -434,18 +491,19 @@ lpc_cmd_firewall(LinphoneCore *lc, char *args)
 	switch(linphone_core_get_firewall_policy(lc))
 	{
 		case LINPHONE_POLICY_NO_FIREWALL:
-			printf("No firewall\n");
+			linphonec_out("No firewall\n");
 			break;
 		case LINPHONE_POLICY_USE_STUN:
-			printf("Using stun server %s to discover firewall address\n", setting ? setting : linphone_core_get_stun_server(lc));
+			linphonec_out("Using stun server %s to discover firewall address\n", setting ? setting : linphone_core_get_stun_server(lc));
 			break;
 		case LINPHONE_POLICY_USE_NAT_ADDRESS:
-			printf("Using supplied nat address %s.\n", setting ? setting : linphone_core_get_nat_address(lc));
+			linphonec_out("Using supplied nat address %s.\n", setting ? setting : linphone_core_get_nat_address(lc));
 			break;
 	}
 	return 1;
 }
 
+#ifndef WIN32
 /* Helper function for processing freind names */
 static int
 lpc_friend_name(char **args, char **name)
@@ -475,6 +533,7 @@ lpc_friend_name(char **args, char **name)
 	}
 	return 1;
 }
+#endif
 
 static int
 lpc_cmd_friend(LinphoneCore *lc, char *args)
@@ -496,7 +555,7 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		if ( ! *args ) return 0;
 		friend_num = strtol(args, NULL, 10);
 		if ( errno == ERANGE ) {
-			printf("Invalid friend number\n");
+			linphonec_out("Invalid friend number\n");
 			return 0;
 		}
 		linphonec_friend_call(lc, friend_num);
@@ -516,7 +575,7 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		{
 			friend_num = strtol(args, NULL, 10);
 			if ( errno == ERANGE ) {
-				printf("Invalid friend number\n");
+				linphonec_out("Invalid friend number\n");
 				return 0;
 			}
 		}
@@ -525,6 +584,7 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 	}
 	else if ( !strncmp(args, "add", 3) )
 	{
+#ifndef WIN32
 		char  *name;
 		char  addr[80];
 		char *addr_p = addr;
@@ -534,6 +594,7 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		if ( ! *args ) return 0;
 		while (*args == ' ') args++;
 		if ( ! *args ) return 0;
+
 		if (!lpc_friend_name(&args,  &name)) return 0;
 
 		while (*args == ' ') args++;
@@ -549,6 +610,11 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		}
 		strcpy(addr_p, addr_orig);
 		linphonec_friend_add(lc, name, addr);
+#else
+		LinphoneFriend *new_friend;
+		new_friend = linphone_friend_new_with_addr(args);
+		linphone_core_add_friend(lc, new_friend);
+#endif
 		return 1;
 	}
 	return 0;
@@ -592,9 +658,13 @@ lpc_cmd_proxy(LinphoneCore *lc, char *args)
 
 	if (strcmp(arg1,"add")==0)
 	{
+#ifdef HAVE_READLINE
 		rl_inhibit_completion=1;
+#endif
 		linphonec_proxy_add(lc);
+#ifdef HAVE_READLINE
 		rl_inhibit_completion=0;
+#endif
 	}
 	else if (strcmp(arg1,"list")==0)
 	{
@@ -610,23 +680,43 @@ lpc_cmd_proxy(LinphoneCore *lc, char *args)
 		{
 			proxynum=atoi(arg2);
 			if ( linphonec_proxy_use(lc, proxynum) )
-				printf("Default proxy set to %d.\n", proxynum);
+				linphonec_out("Default proxy set to %d.\n", proxynum);
 		}
 		else
 		{
 			proxynum=linphone_core_get_default_proxy(lc, NULL);
-			if ( proxynum == -1 ) printf("No default proxy.\n");
-			else printf("Current default proxy is %d.\n", proxynum);
+			if ( proxynum == -1 ) linphonec_out("No default proxy.\n");
+			else linphonec_out("Current default proxy is %d.\n", proxynum);
 		}
-	}
-	else if (strcmp(arg1, "unuse")==0)
-	{
+	}else if (strcmp(arg1, "unuse")==0){
 		linphone_core_set_default_proxy(lc, NULL);
-		printf("Use no proxy.\n");
+		linphonec_out("Use no proxy.\n");
 	}
+
+	else if (strcmp(arg1, "show")==0)
+	{
+		if (arg2 && *arg2)
+		{
+			if (strstr(arg2,"default"))
+			{
+		proxynum=linphone_core_get_default_proxy(lc, NULL);
+		if ( proxynum < 0 ) {
+			linphonec_out("No default proxy defined\n");
+			return 1;
+		}
+		linphonec_proxy_show(lc,proxynum);
+			}
+			else
+			{
+		linphonec_proxy_show(lc, atoi(arg2));
+			}
+		}
+		else return 0; /* syntax error */
+	}
+
 	else
 	{
-		printf("Syntax error - see 'help proxy'\n");
+		return 0; /* syntax error */
 	}
 
 	return 1;
@@ -640,7 +730,7 @@ lpc_cmd_call_logs(LinphoneCore *lc, char *args)
 	{
 		LinphoneCallLog *cl=(LinphoneCallLog*)elem->data;
 		char *str=linphone_call_log_to_str(cl);
-		printf("%s\n",str);
+		linphonec_out("%s\n",str);
 		ms_free(str);
 	}
 	return 1;
@@ -651,72 +741,95 @@ lpc_cmd_ipv6(LinphoneCore *lc, char *arg1)
 {
 	if ( ! arg1 )
 	{
-		printf("Syntax error - see 'help ipv6'\n");
-		return 1;
+		return 0; /* syntax error */
 	}
 
 	if (strcmp(arg1,"status")==0)
 	{
-		printf("ipv6 use enabled: %s\n",linphone_core_ipv6_enabled(lc) ? "true":"false");
+		linphonec_out("ipv6 use enabled: %s\n",linphone_core_ipv6_enabled(lc) ? "true":"false");
 	}
 	else if (strcmp(arg1,"enable")==0)
 	{
 		linphone_core_enable_ipv6(lc,TRUE);
-		printf("ipv6 use enabled.\n");
+		linphonec_out("ipv6 use enabled.\n");
 	}
 	else if (strcmp(arg1,"disable")==0)
 	{
 		linphone_core_enable_ipv6(lc,FALSE);
-		printf("ipv6 use disabled.\n");
+		linphonec_out("ipv6 use disabled.\n");
 	}
 	else
 	{
-		printf("Syntax error - see 'help ipv6'\n");
+		return 0; /* syntax error */
 	}
 	return 1;
 }
 
-static int lpc_cmd_soundcard(LinphoneCore *lc, char *cmd){
-	int i;
-	if (cmd==NULL){
-		printf("Syntax error - see 'help soundcard'\n");
-		return 1;
+static int lpc_cmd_soundcard(LinphoneCore *lc, char *args)
+{
+	int i, index;
+	const char **dev;
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0; /* syntax error */
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
 	}
-	if (strcmp(cmd,"list")==0){
-		const char **dev=linphone_core_get_sound_devices(lc);
-		for(i=0;dev[i]!=NULL;i++){
-			printf("%i: %s\n",i,dev[i]);
+
+	if (strcmp(arg1, "list")==0)
+	{
+		dev=linphone_core_get_sound_devices(lc);
+		for(i=0; dev[i]!=NULL; ++i){
+			linphonec_out("%i: %s\n",i,dev[i]);
 		}
 		return 1;
-	}else{
-		char *tmp=alloca(strlen(cmd)+1);
-		char *card=alloca(strlen(cmd)+1);
-		int index;
-		int n=sscanf(cmd,"%s %s",tmp,card);
-		if (n==2 && strcmp(tmp,"use")==0){
-			if (strcmp(card,"files")==0) {
-				printf("Using wav files instead of soundcard.\n");
-				linphone_core_use_files(lc,TRUE);
-				return 1;
-			}else{
-				const char **dev=linphone_core_get_sound_devices(lc);
-				index=atoi(card);
-				for(i=0;dev[i]!=NULL;i++){
-					if (i==index){
-						linphone_core_set_ringer_device(lc,dev[i]);
-						linphone_core_set_playback_device(lc,dev[i]);
-						linphone_core_set_capture_device(lc,dev[i]);
-						printf("Using sound device %s\n",dev[i]);
-						return 1;
-					}
-				}
-				printf("no such sound device\n");
-				return 1;
-			}
-		}
-		printf("Syntax error - see 'help soundcard'\n");
 	}
-	return 1;
+
+	if (strcmp(arg1, "show")==0)
+	{
+		linphonec_out("Ringer device: %s\n",
+			linphone_core_get_ringer_device(lc));
+		linphonec_out("Playback device: %s\n",
+			linphone_core_get_playback_device(lc));
+		linphonec_out("Capture device: %s\n",
+			linphone_core_get_capture_device(lc));
+		return 1;
+	}
+
+	if (strcmp(arg1, "use")==0 && arg2)
+	{
+		if (strcmp(arg2, "files")==0)
+		{
+			linphonec_out("Using wav files instead of soundcard.\n");
+			linphone_core_use_files(lc,TRUE);
+			return 1;
+		}
+
+		dev=linphone_core_get_sound_devices(lc);
+		index=atoi(arg2); /* FIXME: handle not-a-number */
+		for(i=0;dev[i]!=NULL;i++)
+		{
+			if (i!=index) continue;
+
+			linphone_core_set_ringer_device(lc,dev[i]);
+			linphone_core_set_playback_device(lc,dev[i]);
+			linphone_core_set_capture_device(lc,dev[i]);
+			linphonec_out("Using sound device %s\n",dev[i]);
+			return 1;
+		}
+		linphonec_out("No such sound device\n");
+		return 1;
+	}
+
+	return 0; /* syntax error */
 }
 
 /***************************************************************************
@@ -732,18 +845,18 @@ linphonec_proxy_add(LinphoneCore *lc)
 	bool_t enable_register=FALSE;
 	LinphoneProxyConfig *cfg;
 
-	printf("Adding new proxy setup. Hit ^D to abort.\n");
+	linphonec_out("Adding new proxy setup. Hit ^D to abort.\n");
 
 	/*
 	 * SIP Proxy address
 	 */
 	while (1)
 	{
-		char *input=readline("Enter proxy sip address: ");
+		char *input=linphonec_readline("Enter proxy sip address: ");
 		char *clean;
 
 		if ( ! input ) {
-			printf("Aborted.\n");
+			linphonec_out("Aborted.\n");
 			return;
 		}
 
@@ -757,7 +870,7 @@ linphonec_proxy_add(LinphoneCore *lc)
 		cfg=linphone_proxy_config_new();
 		if (linphone_proxy_config_set_server_addr(cfg,clean)<0)
 		{
-			printf("Invalid sip address (sip:sip.domain.tld).\n");
+			linphonec_out("Invalid sip address (sip:sip.domain.tld).\n");
 			free(input);
 			linphone_proxy_config_destroy(cfg);
 			continue;
@@ -771,11 +884,11 @@ linphonec_proxy_add(LinphoneCore *lc)
 	 */
 	while (1)
 	{
-		char *input=readline("Your identity for this proxy: ");
+		char *input=linphonec_readline("Your identity for this proxy: ");
 		char *clean;
 
 		if ( ! input ) {
-			printf("Aborted.\n");
+			linphonec_out("Aborted.\n");
 			linphone_proxy_config_destroy(cfg);
 			return;
 		}
@@ -790,7 +903,7 @@ linphonec_proxy_add(LinphoneCore *lc)
 		linphone_proxy_config_set_identity(cfg, clean);
 		if ( ! cfg->reg_identity )
 		{
-			printf("Invalid identity (sip:name@sip.domain.tld).\n");
+			linphonec_out("Invalid identity (sip:name@sip.domain.tld).\n");
 			free(input);
 			continue;
 		}
@@ -803,11 +916,11 @@ linphonec_proxy_add(LinphoneCore *lc)
 	 */
 	while (1)
 	{
-		char *input=readline("Do you want to register on this proxy (yes/no): ");
+		char *input=linphonec_readline("Do you want to register on this proxy (yes/no): ");
 		char *clean;
 
 		if ( ! input ) {
-			printf("Aborted.\n");
+			linphonec_out("Aborted.\n");
 			linphone_proxy_config_destroy(cfg);
 			return;
 		}
@@ -822,7 +935,7 @@ linphonec_proxy_add(LinphoneCore *lc)
 		if ( ! strcmp(clean, "yes") ) enable_register=TRUE;
 		else if ( ! strcmp(clean, "no") ) enable_register=FALSE;
 		else {
-			printf("Please answer with 'yes' or 'no'\n");
+			linphonec_out("Please answer with 'yes' or 'no'\n");
 			free(input);
 			continue;
 		}
@@ -839,11 +952,11 @@ linphonec_proxy_add(LinphoneCore *lc)
 		long int expires=0;
 		while (1)
 		{
-			char *input=readline("Specify register expiration time"
+			char *input=linphonec_readline("Specify register expiration time"
 				" in seconds (default is 600): ");
 
 			if ( ! input ) {
-				printf("Aborted.\n");
+				linphonec_out("Aborted.\n");
 				linphone_proxy_config_destroy(cfg);
 				return;
 			}
@@ -851,13 +964,13 @@ linphonec_proxy_add(LinphoneCore *lc)
 			expires=strtol(input, (char **)NULL, 10);
 			if ( expires == LONG_MIN || expires == LONG_MAX )
 			{
-				printf("Invalid value: %s\n", strerror(errno));
+				linphonec_out("Invalid value: %s\n", strerror(errno));
 				free(input);
 				continue;
 			}
 
 			linphone_proxy_config_expires(cfg, expires);
-			printf("Expiration: %d seconds\n", cfg->expires);
+			linphonec_out("Expiration: %d seconds\n", cfg->expires);
 
 			free(input);
 			break;
@@ -869,11 +982,11 @@ linphonec_proxy_add(LinphoneCore *lc)
 	 */
 	while (1)
 	{
-		char *input=readline("Specify route if needed: ");
+		char *input=linphonec_readline("Specify route if needed: ");
 		char *clean;
 
 		if ( ! input ) {
-			printf("Aborted.\n");
+			linphonec_out("Aborted.\n");
 			linphone_proxy_config_destroy(cfg);
 			return;
 		}
@@ -882,14 +995,14 @@ linphonec_proxy_add(LinphoneCore *lc)
 		clean=lpc_strip_blanks(input);
 		if ( ! *clean ) {
 			free(input);
-			printf("No route specified.\n");
+			linphonec_out("No route specified.\n");
 			break;
 		}
 
 		linphone_proxy_config_set_route(cfg, clean);
 		if ( ! cfg->reg_route )
 		{
-			printf("Invalid route.\n");
+			linphonec_out("Invalid route.\n");
 			free(input);
 			continue;
 		}
@@ -906,14 +1019,14 @@ linphonec_proxy_add(LinphoneCore *lc)
 		char *input;
 		char *clean;
 
-		printf("--------------------------------------------\n");
+		linphonec_out("--------------------------------------------\n");
 		linphonec_proxy_display(cfg);
-		printf("--------------------------------------------\n");
-		input=readline("Accept the above proxy configuration (yes/no) ?: ");
+		linphonec_out("--------------------------------------------\n");
+		input=linphonec_readline("Accept the above proxy configuration (yes/no) ?: ");
 
 
 		if ( ! input ) {
-			printf("Aborted.\n");
+			linphonec_out("Aborted.\n");
 			linphone_proxy_config_destroy(cfg);
 			return;
 		}
@@ -928,13 +1041,13 @@ linphonec_proxy_add(LinphoneCore *lc)
 		if ( ! strcmp(clean, "yes") ) break;
 		else if ( ! strcmp(clean, "no") )
 		{
-			printf("Declined.\n");
+			linphonec_out("Declined.\n");
 			linphone_proxy_config_destroy(cfg);
 			free(input);
 			return;
 		}
 
-		printf("Please answer with 'yes' or 'no'\n");
+		linphonec_out("Please answer with 'yes' or 'no'\n");
 		free(input);
 		continue;
 	}
@@ -945,18 +1058,33 @@ linphonec_proxy_add(LinphoneCore *lc)
 	/* automatically set the last entered proxy as the default one */
 	linphone_core_set_default_proxy(lc,cfg);
 
-	printf("Proxy added.\n");
+	linphonec_out("Proxy added.\n");
 }
 
 static void
 linphonec_proxy_display(LinphoneProxyConfig *cfg)
 {
-	printf("sip address: %s\nroute: %s\nidentity: %s\nregister: %s\nexpires: %i\n",
+	linphonec_out("sip address: %s\nroute: %s\nidentity: %s\nregister: %s\nexpires: %i\nregistered: %s\n",
 			cfg->reg_proxy,
 			(cfg->reg_route!=NULL)?cfg->reg_route:"",
 			(cfg->reg_identity!=NULL)?cfg->reg_identity:"",
 			(cfg->reg_sendregister)?"yes":"no",
-			cfg->expires);
+			cfg->expires,
+			linphone_proxy_config_is_registered(cfg) ? "yes" : "no");
+}
+
+static void linphonec_proxy_show(LinphoneCore *lc, int index)
+{
+	const MSList *elem;
+	int i;
+	for(elem=linphone_core_get_proxy_config_list(lc),i=0;elem!=NULL;elem=elem->next,++i){
+		if (index==i){
+			LinphoneProxyConfig *cfg=(LinphoneProxyConfig *)elem->data;
+			linphonec_proxy_display(cfg);
+			return;
+		}
+	}
+	linphonec_out("No proxy with index %i\n", index);
 }
 
 static void
@@ -969,11 +1097,12 @@ linphonec_proxy_list(LinphoneCore *lc)
 	proxies=linphone_core_get_proxy_config_list(lc);
 	for(n=0;proxies!=NULL;proxies=ms_list_next(proxies),n++){
 		if (n==def)
-			printf("****** Proxy %i - this is the default one - *******\n",n);
+			linphonec_out("****** Proxy %i - this is the default one - *******\n",n);
 		else 
-			printf("****** Proxy %i *******\n",n);
+			linphonec_out("****** Proxy %i *******\n",n);
 		linphonec_proxy_display((LinphoneProxyConfig*)proxies->data);
 	}
+	if ( ! n ) linphonec_out("No proxies defined\n");
 }
 
 static void
@@ -984,11 +1113,11 @@ linphonec_proxy_remove(LinphoneCore *lc, int index)
 	proxies=linphone_core_get_proxy_config_list(lc);
 	cfg=(LinphoneProxyConfig*)ms_list_nth_data(proxies,index);
 	if (cfg==NULL){
-		printf("No such proxy.\n");
+		linphonec_out("No such proxy.\n");
 		return;
 	}
 	linphone_core_remove_proxy_config(lc,cfg);
-	printf("Proxy %s removed.\n", cfg->reg_proxy);
+	linphonec_out("Proxy %s removed.\n", cfg->reg_proxy);
 	linphone_proxy_config_destroy(cfg);
 }
 
@@ -1000,7 +1129,7 @@ linphonec_proxy_use(LinphoneCore *lc, int index)
 	proxies=linphone_core_get_proxy_config_list(lc);
 	cfg=(LinphoneProxyConfig*)ms_list_nth_data(proxies,index);
 	if (cfg==NULL){
-		printf("No such proxy (try 'proxy list').");
+		linphonec_out("No such proxy (try 'proxy list').");
 		return 0;
 	}
 	linphone_core_set_default_proxy(lc,cfg);
@@ -1014,8 +1143,8 @@ linphonec_friend_display(LinphoneFriend *fr)
 	char *addr = linphone_friend_get_addr(fr);
 	//char *url = linphone_friend_get_url(fr);
 
-	printf("name: %s\n", name);
-	printf("address: %s\n", addr);
+	linphonec_out("name: %s\n", name);
+	linphonec_out("address: %s\n", addr);
 }
 
 static int
@@ -1036,7 +1165,7 @@ linphonec_friend_list(LinphoneCore *lc, char *pat)
 			char *name = linphone_friend_get_name(friend->data);
 			if ( ! strstr(name, pat) ) continue;
 		}
-		printf("****** Friend %i *******\n",n);
+		linphonec_out("****** Friend %i *******\n",n);
 		linphonec_friend_display((LinphoneFriend*)friend->data);
 	}
 
@@ -1058,7 +1187,7 @@ linphonec_friend_call(LinphoneCore *lc, unsigned int num)
 			return lpc_cmd_call(lc, addr);
 		}
 	}
-	printf("No such friend %u\n", num);
+	linphonec_out("No such friend %u\n", num);
 	return 1;
 }
 
@@ -1098,15 +1227,137 @@ linphonec_friend_delete(LinphoneCore *lc, int num)
 		return 0;
 	}
 
-	printf("No such friend %u\n", num);
+	linphonec_out("No such friend %u\n", num);
 	return 1;
 }
 
 static void
 linphonec_display_command_help(LPC_COMMAND *cmd)
 {
-	if ( cmd->doc ) printf ("%s\n", cmd->doc);
-	else printf("%s\n", cmd->help);
+	if ( cmd->doc ) linphonec_out ("%s\n", cmd->doc);
+	else linphonec_out("%s\n", cmd->help);
+}
+
+
+static int lpc_cmd_register(LinphoneCore *lc, char *args){
+	char identity[512];
+	char proxy[512];
+	char passwd[512];
+	LinphoneProxyConfig *cfg;
+	const MSList *elem;
+	passwd[0]=proxy[0]=identity[0]='\0';
+	sscanf(args,"%s %s %s",identity,proxy,passwd);
+	if (proxy[0]=='\0' || identity[0]=='\0'){
+		linphonec_out("Missing parameters, see help register\n");
+		return 1;
+	}
+	if (passwd[0]!='\0'){
+		osip_from_t *from;
+		LinphoneAuthInfo *info;
+		osip_from_init(&from);
+		if (osip_from_parse(from,identity)==0){
+			char realm[128];
+			snprintf(realm,sizeof(realm)-1,"\"%s\"",from->url->host);
+			info=linphone_auth_info_new(from->url->username,NULL,passwd,NULL,realm);
+			linphone_core_add_auth_info(lc,info);
+		}
+		osip_from_free(from);
+	}
+	elem=linphone_core_get_proxy_config_list(lc);
+	if (elem) {
+		cfg=(LinphoneProxyConfig*)elem->data;
+		linphone_proxy_config_edit(cfg);
+	}
+	else cfg=linphone_proxy_config_new();
+	linphone_proxy_config_set_identity(cfg,identity);
+	linphone_proxy_config_set_server_addr(cfg,proxy);
+	linphone_proxy_config_enable_register(cfg,TRUE);
+	if (elem) linphone_proxy_config_done(cfg);
+	else linphone_core_add_proxy_config(lc,cfg);
+	linphone_core_set_default_proxy(lc,cfg);
+	return 1;
+}
+
+static int lpc_cmd_unregister(LinphoneCore *lc, char *args){
+	LinphoneProxyConfig *cfg=NULL;
+	linphone_core_get_default_proxy(lc,&cfg);
+	if (cfg && linphone_proxy_config_is_registered(cfg)) {
+		linphone_proxy_config_edit(cfg);
+		linphone_proxy_config_enable_register(cfg,FALSE);
+		linphone_proxy_config_done(cfg);
+	}else{
+		linphonec_out("unregistered\n");
+	}
+	return 1;
+}
+
+static int lpc_cmd_duration(LinphoneCore *lc, char *args){
+	LinphoneCallLog *cl;
+	const MSList *elem=linphone_core_get_call_logs(lc);
+	for(;elem!=NULL;elem=elem->next){
+		if (elem->next==NULL){
+			cl=(LinphoneCallLog*)elem->data;
+			linphonec_out("%i seconds\n",cl->duration);
+		}
+	}
+	return 1;
+}
+
+static int lpc_cmd_status(LinphoneCore *lc, char *args)
+{
+	if ( ! args ) return 0;
+
+	LinphoneProxyConfig *cfg;
+	linphone_core_get_default_proxy(lc,&cfg);
+	if (strstr(args,"register"))
+	{
+		if (cfg)
+		{
+			if (linphone_proxy_config_is_registered(cfg)){
+				linphonec_out("registered, identity=%s duration=%i\n",
+					linphone_proxy_config_get_identity(cfg),
+					linphone_proxy_config_get_expires(cfg));
+			}else if (linphone_proxy_config_register_enabled(cfg)){
+				linphonec_out("registered=-1\n");
+			}else linphonec_out("registered=0\n");
+		}
+		else linphonec_out("registered=0\n");
+	}
+	else if (strstr(args,"autoanswer"))
+	{
+		if (cfg && linphone_proxy_config_is_registered(cfg))
+			linphonec_out("autoanswer=%i\n",linphonec_get_autoanswer());
+		else linphonec_out("unregistered\n");
+	}
+	else if (strstr(args,"hook"))
+	{
+		gstate_t call_state=linphone_core_get_state(lc,GSTATE_GROUP_CALL);
+		if (!cfg || !linphone_proxy_config_is_registered(cfg)){
+			linphonec_out("unregistered\n");
+			return 1;
+		}
+		switch(call_state){
+			case GSTATE_CALL_OUT_INVITE:
+				linphonec_out("hook=dialing\n");
+			break;
+			case GSTATE_CALL_IDLE:
+				linphonec_out("hook=offhook\n");
+			break;
+			case GSTATE_CALL_OUT_CONNECTED:
+				linphonec_out("Call out, hook=%s duration=%i\n", linphonec_get_callee(),
+					linphone_core_get_current_call_duration(lc));
+			break;
+			case GSTATE_CALL_IN_CONNECTED:
+				linphonec_out("hook=answered duration=%i\n" ,
+					linphone_core_get_current_call_duration(lc));
+			default:
+				break;
+		}
+		
+	}
+	else return 0;
+
+	return 1;
 }
 
 /***************************************************************************
@@ -1214,7 +1465,7 @@ lpc_find_command(const char *name)
  *
  * Revision 1.16  2006/01/18 09:25:32  strk
  * Command completion inhibited in proxy addition and auth request prompts.
- * Avoided use of readline's internal filename completion.
+ * Avoided use of linphonec_readline's internal filename completion.
  *
  * Revision 1.15  2006/01/14 13:29:32  strk
  * Reworked commands interface to use a table structure,
@@ -1229,7 +1480,7 @@ lpc_find_command(const char *name)
  * Added linphonec.h. Code layout change (added comments, forward decl,
  * globals on top, copyright notices and Logs). Handled out-of-memory
  * condition on history management. Removed assumption on sizeof(char).
- * Fixed bug in authentication prompt (introduced by readline).
+ * Fixed bug in authentication prompt (introduced by linphonec_readline).
  * Added support for multiple authentication requests (up to MAX_PENDING_AUTH).
  *
  *
