@@ -6,21 +6,21 @@
  *  Copyright (C) 2002  Florian Winterstein <flox@gmx.net>
  *  Copyright (C) 2000  Simon MORLAT <simon.morlat@free.fr>
  *
- ****************************************************************************
+****************************************************************************
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  ****************************************************************************/
 
@@ -31,13 +31,24 @@
 #include <string.h>
 #include <signal.h>
 #include <limits.h>
+#include <ctype.h>
+
 #include <linphonecore.h>
+#include "private.h" /*coreapi/private.h, needed for LINPHONE_VERSION */
 #include "linphonec.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef WIN32
+#include <ws2tcpip.h>
+#include <ctype.h>
+#include <conio.h>
+#else
+#include <sys/socket.h>
+#include <netdb.h>
+#include <sys/un.h>
+#include <sys/stat.h>
 #endif
 
+
 #ifdef HAVE_GETTEXT
 #include <libintl.h>
 #ifndef _
@@ -47,6 +58,8 @@
 #define _(something)	(something)
 #endif
 
+#define DEFAULT_TCP_PORT 32333
+
 /***************************************************************************
  *
  *  Types
@@ -69,13 +82,15 @@ char *lpc_strip_blanks(char *input);
 static int handle_configfile_migration(void);
 static int copy_file(const char *from, const char *to);
 static int linphonec_parse_cmdline(int argc, char **argv);
-static int linphonec_initialize_readline(void);
-static int linphonec_finish_readline();
 static int linphonec_init(int argc, char **argv);
 static int linphonec_main_loop (LinphoneCore * opm, char * sipAddr);
 static int linphonec_idle_call (void);
+#ifdef HAVE_READLINE
+static int linphonec_initialize_readline(void);
+static int linphonec_finish_readline();
 static char **linephonec_readline_completion(const char *text,
 	int start, int end);
+#endif
 
 /* These are callback for linphone core */
 static void linphonec_call_received(LinphoneCore *lc, const char *from);
@@ -95,7 +110,6 @@ static void linphonec_text_received(LinphoneCore *lc, LinphoneChatRoom *cr,
 static void linphonec_display_status (LinphoneCore * lc, const char *something);
 static void linphonec_general_state (LinphoneCore * lc, LinphoneGeneralState *gstate);
 static void print_prompt(LinphoneCore *opm);
-
 /***************************************************************************
  *
  * Global variables 
@@ -104,20 +118,32 @@ static void print_prompt(LinphoneCore *opm);
 
 LinphoneCore linphonec;
 FILE *mylogfile;
+#ifdef HAVE_READLINE
 static char *histfile_name=NULL;
 static char last_in_history[256];
+#endif
 //auto answer (-a) option
 static bool_t auto_answer=FALSE;
 static bool_t answer_call=FALSE;
 static bool_t vcap_enabled=FALSE;
 static bool_t display_enabled=FALSE;
+static bool_t preview_enabled=FALSE;
 static bool_t show_general_state=FALSE;
+static bool_t unix_socket=FALSE;
+static int tcp_port=0; /* see --tcp: tcp port to listen for commands */
 LPC_AUTH_STACK auth_stack;
 static int trace_level = 0;
 static char *logfile_name = NULL;
 static char configfile_name[PATH_MAX];
 static char *sipAddr = NULL; /* for autocall */
+static ortp_socket_t client_sock=-1;
 char prompt[PROMPT_MAX_LEN];
+char sock_unix_path[128]={0};
+
+static ortp_thread_t net_reader_th;
+static bool_t net_reader_run=FALSE;
+static ortp_socket_t server_sock;
+
 
 LinphoneCoreVTable linphonec_vtable = {
 	show:(ShowInterfaceCb) stub,
@@ -202,19 +228,23 @@ linphonec_call_received(LinphoneCore *lc, const char *from)
 static void
 linphonec_prompt_for_auth(LinphoneCore *lc, const char *realm, const char *username)
 {
-	LinphoneAuthInfo *pending_auth;
-
-	if ( auth_stack.nitems+1 > MAX_PENDING_AUTH )
-	{
-		fprintf(stderr,
-			"Can't accept another authentication request.\n"
-			"Consider incrementing MAX_PENDING_AUTH macro.\n");
-		return;
-	} 
-
-	pending_auth=linphone_auth_info_new(username,NULL,NULL,NULL,realm);
-	auth_stack.elem[auth_stack.nitems++]=pending_auth;
+	/* no prompt possible when using pipes or tcp mode*/
+	if (unix_socket || tcp_port){
+		linphone_core_abort_authentication(lc,NULL);
+	}else{
+		LinphoneAuthInfo *pending_auth;
 	
+		if ( auth_stack.nitems+1 > MAX_PENDING_AUTH )
+		{
+			fprintf(stderr,
+				"Can't accept another authentication request.\n"
+				"Consider incrementing MAX_PENDING_AUTH macro.\n");
+			return;
+		} 
+	
+		pending_auth=linphone_auth_info_new(username,NULL,NULL,NULL,realm);
+		auth_stack.elem[auth_stack.nitems++]=pending_auth;
+	}
 }
 
 /*
@@ -322,6 +352,210 @@ linphonec_general_state (LinphoneCore * lc, LinphoneGeneralState *gstate)
         }  
 }
 
+static char received_prompt[PROMPT_MAX_LEN];
+static ms_mutex_t prompt_mutex;
+static bool_t have_prompt=FALSE;
+
+static void *prompt_reader_thread(void *arg){
+	char *ret;
+	char tmp[PROMPT_MAX_LEN];
+	while ((ret=fgets(tmp,sizeof(tmp),stdin))!=NULL){
+		ms_mutex_lock(&prompt_mutex);
+		strcpy(received_prompt,ret);
+		have_prompt=TRUE;
+		ms_mutex_unlock(&prompt_mutex);
+	}
+	return NULL;
+}
+
+static void start_prompt_reader(void){
+	ortp_thread_t th;
+	ms_mutex_init(&prompt_mutex,NULL);
+	ortp_thread_create(&th,NULL,prompt_reader_thread,NULL);
+}
+
+static ortp_socket_t create_server_socket(int port){
+	ortp_socket_t sock;
+	if (!unix_socket){
+		char service[12];
+		/*setup the server socket */
+		struct addrinfo *ai=NULL;
+		struct addrinfo hints;
+		memset(&hints,0,sizeof(hints));
+		hints.ai_family=AF_INET;
+		snprintf(service,sizeof(service),"%i",port);
+	
+		getaddrinfo("127.0.0.1",service,&hints,&ai);
+		if (ai==NULL){
+			fprintf(stderr,"getaddrinfo failed on port %s",service);
+			exit(-1);
+		}
+		sock=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
+		if (bind(sock,ai->ai_addr,ai->ai_addrlen)!=0){
+			fprintf(stderr,"Failed to bind command socket.\n");
+			exit(-1);
+		}
+		listen(sock,1);
+	}else{
+#ifndef WIN32
+		struct sockaddr_un sa;
+		sock=socket(AF_UNIX,SOCK_STREAM,0);
+		sa.sun_family=AF_UNIX;
+		snprintf(sock_unix_path,sizeof(sock_unix_path)-1,"/tmp/linphonec-%i",getuid());
+		strncpy(sa.sun_path,sock_unix_path,sizeof(sa.sun_path)-1);
+		unlink(sock_unix_path);/*in case we didn't finished properly previous time */
+		fchmod(sock,S_IRUSR|S_IWUSR);
+		if (bind(sock,(struct sockaddr*)&sa,sizeof(sa))!=0){
+			fprintf(stderr,"Failed to bind command unix socket.\n");
+			exit(-1);
+		}
+		listen(sock,1);
+		printf("Listening from unix socket %s\n",sock_unix_path);
+#else
+		fprintf(stderr,"Window pipe implementation not written yet.\n");
+#endif
+	}
+	return sock;
+}
+
+static void *net_thread(void*p){
+	char tmp[250];
+	struct sockaddr_storage ss;
+#ifndef WIN32
+	struct sockaddr_un su;
+#endif
+	socklen_t ssize;
+	
+	server_sock=create_server_socket(tcp_port);
+	if (server_sock==-1) return NULL;
+	while(net_reader_run){
+		while(client_sock!=-1){ /*sleep until the last command is finished*/
+#ifndef WIN32
+			usleep(20000);
+#else
+			Sleep(20);
+#endif
+		}
+		if (!unix_socket){
+			ssize=sizeof(ss);
+			client_sock=accept(server_sock,(struct sockaddr*)&ss,&ssize);
+		}else{
+#ifndef WIN32
+			ssize=sizeof(su);
+			client_sock=accept(server_sock,(struct sockaddr*)&su,&ssize);
+#endif
+		}
+		if (client_sock!=-1){
+			int len;
+			/*now read from the client */
+			if ((len=recv(client_sock,tmp,sizeof(tmp)-1,0))>0){
+				ortp_mutex_lock(&prompt_mutex);
+				tmp[len]='\0';
+				strcpy(received_prompt,tmp);
+				printf("Receiving command '%s'\n",received_prompt);fflush(stdout);
+				have_prompt=TRUE;
+				ortp_mutex_unlock(&prompt_mutex);
+			}else{
+				printf("read nothing\n");fflush(stdout);
+				close_socket(client_sock);
+				client_sock=-1;
+			}
+			
+		}else{
+			if (net_reader_run) fprintf(stderr,"accept() failed: %s\n",getSocketError());
+		}
+	}
+	
+	return NULL;
+}
+
+static void start_net_reader(void){
+	ms_mutex_init(&prompt_mutex,NULL);
+	net_reader_run=TRUE;
+	ortp_thread_create(&net_reader_th,NULL,net_thread,NULL);
+}
+
+static void stop_net_reader(void){
+	net_reader_run=FALSE;
+	close(server_sock);
+	if (sock_unix_path[0]!=0){
+		unlink(sock_unix_path);
+	}
+	/*ortp_thread_join(net_reader_th,NULL);*/
+}
+
+#ifdef HAVE_READLINE
+#define BOOL_HAVE_READLINE 1
+#else
+#define BOOL_HAVE_READLINE 0
+#endif
+
+char *linphonec_readline(char *prompt){
+	if (tcp_port ||unix_socket || !BOOL_HAVE_READLINE ){
+		static bool_t prompt_reader_started=FALSE;
+		static bool_t net_reader_started=FALSE;
+		if (!prompt_reader_started){
+			start_prompt_reader();
+			prompt_reader_started=TRUE;
+		}
+		if ((tcp_port>0 || unix_socket) && !net_reader_started){
+			start_net_reader();
+			net_reader_started=TRUE;
+		}
+		fprintf(stdout,"%s",prompt);
+		fflush(stdout);
+		while(1){
+			ms_mutex_lock(&prompt_mutex);
+			if (have_prompt){
+				char *ret=strdup(received_prompt);
+				have_prompt=FALSE;
+				ms_mutex_unlock(&prompt_mutex);
+				return ret;
+			}
+			ms_mutex_unlock(&prompt_mutex);
+			linphonec_idle_call();
+#ifdef WIN32
+			Sleep(20);
+#else
+			usleep(20000);
+#endif
+		}
+	}else{
+#ifdef HAVE_READLINE
+		return readline(prompt);
+#endif
+	}
+}
+
+void linphonec_out(const char *fmt,...){
+	char *res;
+	va_list args;
+	va_start (args, fmt);
+	res=ortp_strdup_vprintf(fmt,args);
+	va_end (args);
+	printf("%s",res);fflush(stdout);
+	if (client_sock!=-1){
+		if (send(client_sock,res,strlen(res),0)==-1){
+			fprintf(stderr,"Fail to send output via socket: %s",getSocketError());
+		}
+	}
+	ortp_free(res);
+}
+
+void linphonec_command_finished(void){
+	if (client_sock!=-1){
+		close_socket(client_sock);
+		client_sock=-1;
+	}
+}
+
+void linphonec_set_autoanswer(bool_t enabled){
+	auto_answer=enabled;
+}
+
+bool_t linphonec_get_autoanswer(){
+	return auto_answer;
+}
 
 /***************************************************************************/
 /*
@@ -417,12 +651,14 @@ linphonec_init(int argc, char **argv)
 	linphone_core_init (&linphonec, &linphonec_vtable, configfile_name,
 			    NULL);
 	linphone_core_enable_video(&linphonec,vcap_enabled,display_enabled);
-	if (!(vcap_enabled || display_enabled)) printf("Warning: video is disabled in linphonec.\n");
+	linphone_core_enable_video_preview(&linphonec,preview_enabled);
+	if (!(vcap_enabled || display_enabled)) printf("Warning: video is disabled in linphonec, use -V or -C or -D to enable.\n");
+#ifdef HAVE_READLINE
 	/*
 	 * Initialize readline
 	 */
 	linphonec_initialize_readline();
-
+#endif
 	/*
 	 * Initialize signal handlers
 	 */
@@ -444,8 +680,12 @@ linphonec_finish(int exit_status)
 
 	/* Terminate any pending call */
    	linphonec_parse_command_line(&linphonec, "terminate");
-
+#ifdef HAVE_READLINE
 	linphonec_finish_readline();
+#endif
+	if (net_reader_run)
+		stop_net_reader();
+
 
 	linphone_core_uninit (&linphonec);
 
@@ -474,15 +714,16 @@ linphonec_prompt_for_auth_final(LinphoneCore *lc)
 {
 	char *input, *iptr;
 	char auth_prompt[256];
+#ifdef HAVE_READLINE
 	rl_hook_func_t *old_event_hook;
-
+#endif
 	LinphoneAuthInfo *pending_auth=auth_stack.elem[auth_stack.nitems-1];
 
 	snprintf(auth_prompt, 256, "Password for %s on %s: ",
 		pending_auth->username, pending_auth->realm);
 
 	printf("\n");
-
+#ifdef HAVE_READLINE
 	/*
 	 * Disable event hook to avoid entering an
 	 * infinite loop. This would prevent idle_call
@@ -491,10 +732,11 @@ linphonec_prompt_for_auth_final(LinphoneCore *lc)
 	 */
 	old_event_hook=rl_event_hook;
 	rl_event_hook=NULL;
+#endif
 
 	while (1)
 	{
-		input=readline(auth_prompt);
+		input=linphonec_readline(auth_prompt);
 
 		/*
 		 * If EOF (^D) is sent you probably don't want
@@ -531,15 +773,14 @@ linphonec_prompt_for_auth_final(LinphoneCore *lc)
 	linphone_auth_info_set_passwd(pending_auth, input);
 	linphone_core_add_auth_info(lc, pending_auth);
 	--(auth_stack.nitems);
-
+#ifdef HAVE_READLINE
 	/*
 	 * Reset line_buffer, to avoid the password
 	 * to be used again from outer readline
 	 */
 	rl_line_buffer[0]='\0';
-
 	rl_event_hook=old_event_hook;
-
+#endif
 	return 1;
 }
 
@@ -590,28 +831,25 @@ linphonec_idle_call ()
 		answer_call=FALSE;
 	}
 
-#if 0 /* Automatic exit should be requested with a command line switch */
-	/* Quit if autocall mode was on and no call is in progress */
-	if ( sipAddr != NULL && opm->call == NULL )
-	{
-   		linphonec_parse_command_line(&linphonec, "quit");
-	}
-#endif
-
 	if ( auth_stack.nitems )
 	{
 		/*
 		 * Inhibit command completion
 		 * during password prompts
 		 */
+#ifdef HAVE_READLINE
 		rl_inhibit_completion=1;
+#endif
 		linphonec_prompt_for_auth_final(opm);
+#ifdef HAVE_READLINE
 		rl_inhibit_completion=0;
+#endif
 	}
 
 	return 0;
 }
 
+#ifdef HAVE_READLINE
 /*
  * Use globals:
  *
@@ -664,6 +902,8 @@ linphonec_finish_readline()
 	return 0;
 }
 
+#endif
+
 static void print_prompt(LinphoneCore *opm){
 #ifdef IDENTITY_AS_PROMPT
 	snprintf(prompt, PROMPT_MAX_LEN, "%s> ",
@@ -690,7 +930,7 @@ linphonec_main_loop (LinphoneCore * opm, char * sipAddr)
 		run=linphonec_parse_command_line(&linphonec, buf);
 	}
 
-	while ((input=readline(prompt)))
+	while ((input=linphonec_readline(prompt)))
 	{
 		char *iptr; /* input and input pointer */
 		size_t input_len;
@@ -710,19 +950,23 @@ linphonec_main_loop (LinphoneCore * opm, char * sipAddr)
 			continue;
 		}
 
-
+#ifdef HAVE_READLINE
 		/*
 		 * Only add to history if not already
-		 * last item in it
+		 * last item in it, and only if the command
+		 * doesn't start with a space (to allow for
+		 * hiding passwords)
 		 */
-		if ( strcmp(last_in_history, iptr) )
+		if ( iptr == input && strcmp(last_in_history, iptr) )
 		{
 			strncpy(last_in_history,iptr,sizeof(last_in_history));
 			last_in_history[sizeof(last_in_history)-1]='\0';
 			add_history(iptr);
 		}
+#endif
 
 		linphonec_parse_command_line(&linphonec, iptr);
+		linphonec_command_finished();
 		free(input);
 	}
 
@@ -772,7 +1016,7 @@ linphonec_parse_cmdline(int argc, char **argv)
 					 argv[arg_num]);
 				exit(EXIT_FAILURE);
 			}
-			snprintf(configfile_name, PATH_MAX, argv[arg_num]);
+			snprintf(configfile_name, PATH_MAX, "%s", argv[arg_num]);
 		}
 		else if (strncmp ("-s", argv[arg_num], 2) == 0)
 		{
@@ -796,6 +1040,7 @@ linphonec_parse_cmdline(int argc, char **argv)
                 {
                         display_enabled = TRUE;
 			vcap_enabled = TRUE;
+			preview_enabled=TRUE;
                 }
 		else if ((strncmp ("-v", argv[arg_num], 2) == 0)
 			 ||
@@ -810,6 +1055,18 @@ linphonec_parse_cmdline(int argc, char **argv)
 		{
 			show_general_state = TRUE;
 		}
+		else if (strncmp ("--tcp", argv[arg_num], 5) == 0)
+		{
+			if (arg_num+1 < argc){
+				tcp_port = atoi(argv[arg_num+1]);
+				if (tcp_port!=0) arg_num++;
+			}
+			if (tcp_port==0) tcp_port=DEFAULT_TCP_PORT;
+		}
+		else if (strncmp ("--pipe", argv[arg_num], 6) == 0)
+		{
+			unix_socket=1;
+		}
 		else if (old_arg_num == arg_num)
 		{
 			fprintf (stderr, "ERROR: bad arguments\n");
@@ -924,7 +1181,7 @@ copy_file(const char *from, const char *to)
 	{
 		snprintf(message, 255, "Can't open %s for reading: %s\n",
 			from, strerror(errno));
-		fprintf(stderr, message);
+		fprintf(stderr, "%s", message);
 		return 0;
 	}
 
@@ -934,7 +1191,7 @@ copy_file(const char *from, const char *to)
 	{
 		snprintf(message, 255, "Can't open %s for writing: %s\n",
 			to, strerror(errno));
-		fprintf(stderr, message);
+		fprintf(stderr, "%s", message);
 		return 0;
 	}
 
@@ -953,6 +1210,7 @@ copy_file(const char *from, const char *to)
 	return 1;
 }
 
+#ifdef HAVE_READLINE
 static char **
 linephonec_readline_completion(const char *text, int start, int end)
 {
@@ -982,6 +1240,8 @@ linephonec_readline_completion(const char *text, int start, int end)
 	return matches;
 }
 
+#endif
+
 /*
  * Strip blanks from a string.
  * Return a pointer into the provided string.
