@@ -25,11 +25,17 @@
 #ifndef LINPHONEC_H
 #define LINPHONEC_H 1
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef HAVE_READLINE_H
 #include <readline.h>
+#define HAVE_READLINE
 #else
 #ifdef HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
+#define HAVE_READLINE
 #endif
 #endif
 #ifdef HAVE_HISTORY_H
@@ -99,6 +105,10 @@ typedef struct {
 extern int linphonec_parse_command_line(LinphoneCore *lc, char *cl);
 extern char *linphonec_command_generator(const char *text, int state);
 extern void linphonec_finish(int exit_status);
+extern char *linphonec_readline(char *prompt);
+void linphonec_set_autoanswer(bool_t enabled);
+bool_t linphonec_get_autoanswer();
+
 
 #endif /* def LINPHONEC_H */
 
