@@ -23,12 +23,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/mediastream.h"
 #include <eXosip2/eXosip.h>
 #include <osipparser2/osip_message.h>
+#include <osipparser2/osip_parser.h>
 
 static int linphone_answer_sdp(LinphoneCore *lc, eXosip_event_t *ev, sdp_message_t *sdp);
 
+static bool_t linphone_call_matches_event(LinphoneCall *call, eXosip_event_t *ev){
+	return call->cid==ev->cid;
+}
+
 static void linphone_call_proceeding(LinphoneCore *lc, eXosip_event_t *ev){
-	if (lc->call==NULL){
-		ms_warning("Bug in call_proceeding()");
+	if (lc->call==NULL || (lc->call->cid!=-1 && !linphone_call_matches_event(lc->call,ev)) ) {
+		ms_warning("This call has been canceled.");
+		eXosip_lock();
+		eXosip_call_terminate(ev->cid,ev->did);
+		eXosip_unlock();
 		return;
 	}
 	lc->call->cid=ev->cid;
@@ -64,6 +72,7 @@ int linphone_call_accepted(LinphoneCore *lc, eXosip_event_t *ev)
 		return 0;
 	}
 	linphone_call_proceeding(lc,ev);
+	if (!linphone_call_matches_event(lc->call,ev)) return 0;
 	call->auth_pending=FALSE;
 	if (call->state==LCStateAVRunning){
 		return 0; /*already accepted*/
@@ -153,6 +162,15 @@ int linphone_call_failure(LinphoneCore *lc, eXosip_event_t *ev)
 	char *msg603=_("Call declined.");
 	char* tmpmsg=msg486;
 	int code;
+	LinphoneCall *call=lc->call;
+
+	if (call){
+		/*check that the faillure is related to this call, not an old one*/
+		if (!linphone_call_matches_event(call,ev)) {
+			ms_warning("Failure reported for an old call.");
+			return 0;
+		}
+	}
 
 	if (ev->response){
 		code=osip_message_get_status_code(ev->response);
@@ -219,8 +237,8 @@ int linphone_call_failure(LinphoneCore *lc, eXosip_event_t *ev)
 		lc->ringstream=NULL;
 	}
 	linphone_core_stop_media_streams(lc);
-	if (lc->call!=NULL) {
-		linphone_call_destroy(lc->call);
+	if (call!=NULL) {
+		linphone_call_destroy(call);
 		gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 		lc->call=NULL;
 	}
@@ -577,6 +595,9 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 			/* this payload is supported in our local rtp profile, so add it to the dialog rtp
 			profile */
 			rtppayload=payload_type_clone(rtppayload);
+			if (rtp_profile_get_payload(dialog_profile,payload->pt)!=NULL){
+				ms_error("Payload %s type already entered, should not happen !",rtppayload->mime_type);
+			}
 			rtp_profile_set_payload(dialog_profile,payload->pt,rtppayload);
 			/* add to the rtp payload type some other parameters (bandwidth) */
 			if (rtppayload->type==PAYLOAD_VIDEO){
@@ -593,10 +614,13 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 					payload->b_as_bandwidth, ubw);
 			}else{
 				/*limit to upload bandwidth if exist, else no limit*/
-				rtppayload->normal_bitrate=(ubw>0)? 1000*ubw : -1;
+				if (ubw>0) rtppayload->normal_bitrate=1000*ubw;
+				else {
+					if (rtppayload->type!=PAYLOAD_VIDEO){
+						rtppayload->normal_bitrate=-1; /*allow speex to use maximum bitrate*/
+					}
+				}
 			}
-			/* but anyway give our download bandwidth constraint*/
-			payload->b_as_bandwidth=(dbw>0) ? dbw : 0;
 			if (payload->a_fmtp!=NULL){
 				payload_type_set_send_fmtp(rtppayload,payload->a_fmtp);
 			}
@@ -623,10 +647,6 @@ int linphone_accept_audio_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 	PayloadType *lpt=NULL;
 
 	params=&call->audio_params;
-	if (call->profile==NULL){
-		/* create a remote user agent profile */
-		call->profile=remote_profile=rtp_profile_new("remote");
-	}
 	remote_profile=call->profile;
 	/* see if this codec is supported in our local rtp profile*/
 	supported=linphone_payload_is_supported(lc,payload,lc->local_profile,remote_profile,TRUE,&lpt);
@@ -635,6 +655,7 @@ int linphone_accept_audio_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 		return -1;
 	}
 	if (lc->sip_conf.only_one_codec && params->initialized){
+		ms_message("Only one codec has to be accepted.");
 		return -1;
 	}
 	if (supported==SupportedAndValid) {		
@@ -657,10 +678,16 @@ int linphone_accept_audio_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 			}
 			params->initialized=1;
 			/* we can now update the allocated bandwidth for audio, and then video*/
-			linphone_core_update_allocated_audio_bandwidth(lc,lpt);
+			linphone_core_update_allocated_audio_bandwidth_in_call(lc,lpt);
+			/* give our download bandwidth constraint*/
+			payload->b_as_bandwidth=(lc->dw_audio_bw>0) ? lc->dw_audio_bw : 0;
 		}else{
 			/* refuse all other audio lines*/
-			if(params->line!=payload->line) return -1;
+			if(params->line!=payload->line) {
+				ms_message("Only one audio line can be accepted.");
+				abort();
+				return -1;
+			}
 		}
 	}
 	return 0;
@@ -677,10 +704,6 @@ int linphone_accept_video_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 
 	if (!linphone_core_video_enabled(lc)) return -1;
 
-	if (call->profile==NULL){
-		/* create a remote user agent profile */
-		call->profile=rtp_profile_new("remote");
-	}
 	params=&call->video_params;
 	remote_profile=call->profile;
 	/* see if this codec is supported in our local rtp profile*/
@@ -710,6 +733,7 @@ int linphone_accept_video_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 				params->remotertcpport=params->remoteport+1;
 			}
 			params->initialized=1;
+			payload->b_as_bandwidth=(lc->dw_video_bw>0) ? lc->dw_video_bw : 0;
 		}else{
 			/* refuse all other video lines*/
 			if(params->line!=payload->line) return -1;
@@ -752,7 +776,7 @@ int linphone_read_audio_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 			}
 			params->initialized=1;
 			/* we can now update the allocated bandwidth for audio, and then video*/
-			linphone_core_update_allocated_audio_bandwidth(lc,lpt);
+			linphone_core_update_allocated_audio_bandwidth_in_call(lc,lpt);
 		}
 	}
 	return 0;
@@ -800,7 +824,7 @@ void linphone_call_ringing(LinphoneCore *lc, eXosip_event_t *ev){
 	LinphoneCall *call=lc->call;
 	
 	linphone_call_proceeding(lc,ev);
-	
+	if (call==NULL) return;
 	if (sdp==NULL){
 		if (lc->ringstream!=NULL) return;	/*already ringing !*/
 		if (lc->sound_conf.play_sndcard!=NULL){
@@ -864,7 +888,6 @@ void linphone_call_message_new(LinphoneCore *lc, eXosip_event_t *ev){
 #endif
 }
 
-
 void linphone_registration_faillure(LinphoneCore *lc, eXosip_event_t *ev){
 	int status_code=0;
 	char *msg;
@@ -883,6 +906,8 @@ void linphone_registration_faillure(LinphoneCore *lc, eXosip_event_t *ev){
 		case 407:
 			linphone_process_authentication(lc,ev);
 			break;
+		case 403:
+			linphone_proxy_config_process_authentication_failure(lc,ev);
 		default:
 			msg=ortp_strdup_printf(_("Registration on %s failed: %s"),ru,(reason!=NULL) ? reason : _("no response timeout"));
 			lc->vtable.display_status(lc,msg);
@@ -897,6 +922,7 @@ void linphone_registration_success(LinphoneCore *lc,eXosip_event_t *ev){
 	osip_uri_t *requri=osip_message_get_uri(ev->request);
 	char *msg;
 	char *ru;
+	osip_header_t *h=NULL;
 	osip_uri_to_str(requri,&ru);
 	msg=ms_strdup_printf(_("Registration on %s successful."),ru);
 	lc->vtable.display_status(lc,msg);
@@ -904,10 +930,12 @@ void linphone_registration_success(LinphoneCore *lc,eXosip_event_t *ev){
 	osip_free(ru);
 	cfg=linphone_core_get_proxy_config_from_rid(lc,ev->rid);
 	ms_return_if_fail(cfg!=NULL);
-	cfg->auth_pending=FALSE;
-	cfg->registered=TRUE;
 	gstate_new_state(lc, GSTATE_REG_OK, NULL);
-	linphone_proxy_config_register_again_with_updated_contact(cfg,ev->request,ev->response);
+	osip_message_get_expires(ev->request,0,&h);
+	if (h!=NULL && atoi(h->hvalue)!=0){
+		cfg->registered=TRUE;
+		linphone_proxy_config_register_again_with_updated_contact(cfg,ev->request,ev->response);
+	}else cfg->registered=FALSE;
 }
 
 static bool_t comes_from_local_if(osip_message_t *msg){
@@ -936,7 +964,15 @@ static void linphone_other_request(LinphoneCore *lc, eXosip_event_t *ev){
 		linphone_core_text_received(lc,ev);
 		eXosip_message_send_answer(ev->tid,200,NULL);
 	}else if (strcmp(ev->request->sip_method,"OPTIONS")==0){
-		eXosip_options_send_answer(ev->tid,200,NULL);
+#if 1
+		osip_message_t *options=NULL;
+		eXosip_options_build_answer(ev->tid,200,&options);
+		osip_message_set_allow(options,"INVITE, ACK, BYE, CANCEL, OPTIONS, MESSAGE, SUBSCRIBE, NOTIFY, INFO");
+		osip_message_set_accept(options,"application/sdp");
+		eXosip_options_send_answer(ev->tid,200,options);
+#else
+		ms_warning("Not answering to this options request.");
+#endif
 	}else if (strcmp(ev->request->sip_method,"WAKEUP")==0
 		&& comes_from_local_if(ev->request)) {
 		eXosip_message_send_answer(ev->tid,200,NULL);
@@ -944,7 +980,13 @@ static void linphone_other_request(LinphoneCore *lc, eXosip_event_t *ev){
 		if (lc->vtable.show)
 			lc->vtable.show(lc);
 	}else {
-		ms_message("Unsupported request received.");
+		char *tmp=NULL;
+		size_t msglen=0;
+		osip_message_to_str(ev->request,&tmp,&msglen);
+		if (tmp){
+			ms_message("Unsupported request received:\n%s",tmp);
+			osip_free(tmp);
+		}
 		/*answer with a 501 Not implemented*/
 		eXosip_message_send_answer(ev->tid,501,NULL);
 	}
