@@ -18,6 +18,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "linphonecore.h"
+#include "sipsetup.h"
 #include "lpconfig.h"
 #include "private.h"
 #include "mediastreamer2/mediastream.h"
@@ -50,8 +51,7 @@ static void toggle_video_preview(LinphoneCore *lc, bool_t val);
 /* relative path where is stored local ring*/
 #define LOCAL_RING "rings/oldphone.wav"
 /* same for remote ring (ringback)*/
-#define REMOTE_RING_FR "ringback.wav"
-#define REMOTE_RING_US "ringback.wav"
+#define REMOTE_RING "ringback.wav" 
 
 
 sdp_handler_t linphone_sdphandler={
@@ -81,6 +81,7 @@ static void  linphone_call_init_common(LinphoneCall *call, char *from, char *to)
 	linphone_core_notify_all_friends(call->core,LINPHONE_STATUS_ONTHEPHONE);
 	if (linphone_core_get_firewall_policy(call->core)==LINPHONE_POLICY_USE_STUN) 
 		linphone_core_run_stun_tests(call->core,call);
+	call->profile=rtp_profile_new("Call RTP profile");
 }
 
 void linphone_call_init_media_params(LinphoneCall *call){
@@ -153,7 +154,8 @@ void linphone_call_destroy(LinphoneCall *obj)
 {
 	linphone_core_notify_all_friends(obj->core,obj->core->prev_mode);
 	linphone_call_log_completed(obj->log,obj);
-	if (obj->profile!=NULL && obj->profile!=obj->core->local_profile) rtp_profile_destroy(obj->profile);
+	linphone_core_update_allocated_audio_bandwidth(obj->core);
+	if (obj->profile!=NULL) rtp_profile_destroy(obj->profile);
 	if (obj->sdpctx!=NULL) sdp_context_free(obj->sdpctx);
 	ms_free(obj);
 }
@@ -242,6 +244,12 @@ void linphone_call_log_destroy(LinphoneCallLog *cl){
 	ms_free(cl);
 }
 
+int linphone_core_get_current_call_duration(const LinphoneCore *lc){
+	LinphoneCall *call=lc->call;
+	if (call==NULL) return 0;
+	return time(NULL)-call->start_time;
+}
+
 void _osip_trace_func(char *fi, int li, osip_trace_level_t level, char *chfr, va_list ap){
 	int ortp_level=ORTP_DEBUG;
 	switch(level){
@@ -382,16 +390,16 @@ void sound_config_read(LinphoneCore *lc)
 	
 	linphone_core_set_ring(lc,tmpbuf);
 	
-	tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+	tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING;
 	tmpbuf=lp_config_get_string(lc->config,"sound","remote_ring",tmpbuf);
 	if (access(tmpbuf,F_OK)==-1){
-		tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+		tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING;
 	}
 	if (strstr(tmpbuf,".wav")==NULL){
 		/* it currently uses old sound files, so replace them */
-		tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+		tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING;
 	}
-	linphone_core_set_ringback(lc,0);
+	linphone_core_set_ringback(lc,tmpbuf);
 	check_sound_device(lc);
 	lc->sound_conf.latency=0;
 
@@ -557,11 +565,15 @@ void video_config_read(LinphoneCore *lc)
 	if (str && str[0]==0) str=NULL;
 	linphone_core_set_video_device(lc,str);
 	
+	linphone_core_set_preferred_video_size_by_name(lc,
+		lp_config_get_string(lc->config,"video","size","cif"));
+
 	enabled=lp_config_get_int(lc->config,"video","enabled",1);
 	capture=lp_config_get_int(lc->config,"video","capture",enabled);
 	display=lp_config_get_int(lc->config,"video","display",enabled);
 #ifdef VIDEO_ENABLED
 	linphone_core_enable_video(lc,capture,display);
+	linphone_core_enable_self_view(lc,TRUE);
 #endif
 }
 
@@ -592,17 +604,8 @@ void linphone_core_set_download_bandwidth(LinphoneCore *lc, int bw){
 		lc->dw_audio_bw=-1;
 		lc->dw_video_bw=-1;
 	}else {
-		if (bw>=256){
-			lc->dw_audio_bw=80;
-		}else if (bw>=128){
-			if (linphone_core_video_enabled(lc))
-				lc->dw_audio_bw=30;
-			else
-				lc->dw_audio_bw=bw;
-		}else{
-			lc->dw_audio_bw=bw;
-		}
-		lc->dw_video_bw=bw-lc->dw_audio_bw;
+		lc->dw_audio_bw=MIN(lc->audio_bw,bw);
+		lc->dw_video_bw=MAX(bw-lc->dw_audio_bw-10,0);/*-10: security margin*/
 	}
 }
 
@@ -611,16 +614,10 @@ void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw){
 	if (bw==0){ /*infinite*/
 		lc->up_audio_bw=-1;
 		lc->up_video_bw=-1;
-		return;
-	}else if (bw>=128){
-		if (linphone_core_video_enabled(lc))
-			lc->up_audio_bw=lc->audio_bw;
-		else 
-			lc->up_audio_bw=bw;
 	}else{
-		lc->up_audio_bw=bw;
+		lc->up_audio_bw=MIN(lc->audio_bw,bw);
+		lc->up_video_bw=MAX(bw-lc->up_audio_bw-10,0);/*-10: security margin*/
 	}
-	lc->up_video_bw=bw-lc->up_audio_bw-10;/*-10: security margin*/
 }
 
 int linphone_core_get_download_bandwidth(const LinphoneCore *lc){
@@ -649,7 +646,7 @@ void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, co
 	
 	memcpy(&lc->vtable,vtable,sizeof(LinphoneCoreVTable));
 
-        gstate_initialize();
+        gstate_initialize(lc);
         gstate_new_state(lc, GSTATE_POWER_STARTUP, NULL);
         
 	ortp_init();
@@ -686,6 +683,7 @@ void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, co
 	lc->rsvp_enable = 1;
 	lc->rpc_enable = 0;
 #endif
+	sip_setup_register_all();
 	sound_config_read(lc);
 	net_config_read(lc);
 	rtp_config_read(lc);
@@ -700,6 +698,7 @@ void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, co
 	ms_mutex_init(&lc->lock,NULL);
 	lc->vtable.display_status(lc,_("Ready"));
         gstate_new_state(lc, GSTATE_POWER_ON, NULL);
+	lc->ready=TRUE;
 }
 
 LinphoneCore *linphone_core_new(const LinphoneCoreVTable *vtable,
@@ -773,7 +772,7 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 		strncpy(result,lc->sip_conf.ipv6_enabled ? "::1" : "127.0.0.1",LINPHONE_IPADDR_SIZE);
 		ms_error("Could not find default routable ip address !"); 
 	}	
-	eXosip_masquerade_contact("",0);
+	eXosip_masquerade_contact(NULL,0);
 }
 
 const char *linphone_core_get_primary_contact(LinphoneCore *lc)
@@ -911,16 +910,37 @@ int linphone_core_get_sip_port(LinphoneCore *lc)
 }
 
 static bool_t exosip_running=FALSE;
+static char _ua_name[64]="Linphone";
+static char _ua_version[64]=LINPHONE_VERSION;
+
+static void set_user_agent(){
+	char ua_string[256];
+	snprintf(ua_string,sizeof(ua_string),"%s/%s (eXosip2/%s)",_ua_name,_ua_version,
+#ifdef HAVE_EXOSIP_GET_VERSION
+		 eXosip_get_version()
+#else
+		 "unknown"
+#endif
+	);
+	eXosip_set_user_agent(ua_string);
+}
+
+void linphone_core_set_user_agent(const char *name, const char *ver){
+	strncpy(_ua_name,name,sizeof(_ua_name)-1);
+	strncpy(_ua_version,ver,sizeof(_ua_version));
+}
 
 void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 {
 	const char *anyaddr;
-	char ua_string[256];
 	int err=0;
 	if (port==lc->sip_conf.sip_port) return;
 	lc->sip_conf.sip_port=port;
 	if (exosip_running) eXosip_quit();
 	eXosip_init();
+	err=0;
+	eXosip_set_option(13,&err); /*13=EXOSIP_OPT_SRV_WITH_NAPTR, as it is an enum value, we can't use it unless we are sure of the
+					version of eXosip, which is not the case*/
 	eXosip_enable_ipv6(lc->sip_conf.ipv6_enabled);
 	if (lc->sip_conf.ipv6_enabled)
 		anyaddr="::0";
@@ -940,14 +960,7 @@ void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 	eXosip_set_rsvp_mode (lc->rsvp_enable);
 	eXosip_set_rpc_mode (lc->rpc_enable);
 #endif
-	snprintf(ua_string,sizeof(ua_string),"Linphone/%s (eXosip2/%s)",LINPHONE_VERSION,
-#ifdef HAVE_EXOSIP_GET_VERSION
-		 eXosip_get_version()
-#else
-		 "unknown"
-#endif
-);
-	eXosip_set_user_agent(ua_string);
+	set_user_agent("Linphone",LINPHONE_VERSION);
 	exosip_running=TRUE;
 }
 
@@ -982,6 +995,15 @@ void linphone_core_iterate(LinphoneCore *lc)
 	eXosip_event_t *ev;
 	bool_t disconnected=FALSE;
 	int disconnect_timeout = linphone_core_get_nortp_timeout(lc); 
+	time_t curtime=time(NULL);
+	int elapsed;
+	bool_t one_second_elapsed=FALSE;
+	
+	if (curtime-lc->prevtime>=1){
+		lc->prevtime=curtime;
+		one_second_elapsed=TRUE;
+	}
+
 	if (lc->preview_finished){
 		lc->preview_finished=0;
 		ring_stop(lc->ringstream);
@@ -993,12 +1015,15 @@ void linphone_core_iterate(LinphoneCore *lc)
 		while((ev=eXosip_event_wait(0,0))!=NULL){
 			linphone_core_process_event(lc,ev);
 		}
-		if (lc->automatic_action==0) eXosip_automatic_action();
+		if (lc->automatic_action==0) {
+			eXosip_lock();
+			eXosip_automatic_action();
+			eXosip_unlock();
+		}
 	}
 	if (lc->call!=NULL){
 		LinphoneCall *call=lc->call;
-		int elapsed;
-		time_t curtime=time(NULL);
+		
 		if (call->dir==LinphoneCallIncoming && call->state==LCStateRinging){
 			elapsed=curtime-call->start_time;
 			ms_message("incoming call ringing for %i seconds",elapsed);
@@ -1006,8 +1031,7 @@ void linphone_core_iterate(LinphoneCore *lc)
 				linphone_core_terminate_call(lc,NULL);
 			}
 		}else if (call->state==LCStateAVRunning){
-			elapsed=curtime-lc->prevtime;
-			if (elapsed>=1){
+			if (one_second_elapsed){
 				RtpSession *as=NULL,*vs=NULL;
 				lc->prevtime=curtime;
 				if (lc->audiostream!=NULL)
@@ -1036,6 +1060,9 @@ void linphone_core_iterate(LinphoneCore *lc)
 	}
 	if (disconnected)
 		linphone_core_disconnected(lc);
+	if (one_second_elapsed && lp_config_needs_commit(lc->config)){
+		lp_config_sync(lc->config);
+	}
 }
 
 
@@ -1053,6 +1080,33 @@ static osip_to_t *osip_to_create(const char *to){
 	return ret;
 }
 
+static char *guess_route_if_any(LinphoneCore *lc, osip_to_t *parsed_url){
+	const MSList *elem=linphone_core_get_proxy_config_list(lc);
+	for(;elem!=NULL;elem=elem->next){
+		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
+		char prx[256];
+		if (cfg->ssctx && sip_setup_context_get_proxy(cfg->ssctx,parsed_url->url->host,prx,sizeof(prx))==0){
+			ms_message("We have a proxy for domain %s",parsed_url->url->host);
+			if (strcmp(parsed_url->url->host,prx)!=0){
+				char *route=NULL;
+				osip_route_t *rt;
+				osip_route_init(&rt);
+				if (osip_route_parse(rt,prx)==0){
+					char *rtstr;
+					osip_uri_uparam_add(rt->url,osip_strdup("lr"),NULL);
+					osip_route_to_str(rt,&rtstr);
+					route=ms_strdup(rtstr);
+					osip_free(rtstr);
+				}
+				osip_route_free(rt);
+				ms_message("Adding a route: %s",route);
+				return route;
+			}
+		}
+	}
+	return NULL;
+}
+
 bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **real_url, osip_to_t **real_parsed_url, char **route){
 	enum_lookup_res_t *enumres=NULL;
 	osip_to_t *parsed_url=NULL;
@@ -1098,6 +1152,7 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 			if (real_parsed_url!=NULL) *real_parsed_url=osip_to_create(sipaddr);
 			if (real_url!=NULL) *real_url=sipaddr;
 			else ms_free(sipaddr);
+#if 0
 			/*if the prompted uri was auto-suffixed with proxy domain,
 			then automatically set a route so that the request goes
 			through the proxy*/
@@ -1114,6 +1169,10 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 				ms_message("setting automatically a route to %s",*route);
 			}
 			else *route=ms_strdup(tmproute);
+#else
+			if (tmproute==NULL) *route=guess_route_if_any(lc,*real_parsed_url);
+			if (tmproute) *route=ms_strdup(tmproute);
+#endif
 			return TRUE;
 		}
 	}
@@ -1123,6 +1182,7 @@ bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **rea
 		if (real_parsed_url!=NULL) *real_parsed_url=parsed_url;
 		else osip_to_free(parsed_url);
 		if (tmproute) *route=ms_strdup(tmproute);
+		else *route=guess_route_if_any(lc,*real_parsed_url);
 		return TRUE;
 	}
 	/* else we could not do anything with url given by user, so display an error */
@@ -1213,7 +1273,6 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	ms_free(barmsg);
 	if (!lc->sip_conf.sdp_200_ack){
 		ctx=lc->call->sdpctx;
-		lc->call->profile=lc->local_profile;
 		sdpmesg=sdp_context_get_offer(ctx);
 		linphone_set_sdp(invite,sdpmesg);
 		linphone_core_init_media_streams(lc);
@@ -1349,12 +1408,19 @@ int linphone_core_change_qos(LinphoneCore *lc, int answer)
 void linphone_core_init_media_streams(LinphoneCore *lc){
 	lc->audiostream=audio_stream_new(linphone_core_get_audio_port(lc),linphone_core_ipv6_enabled(lc));
 #ifdef VIDEO_ENABLED
-	lc->videostream=video_stream_new(linphone_core_get_video_port(lc),linphone_core_ipv6_enabled(lc));
+	if (lc->video_conf.display || lc->video_conf.capture)
+		lc->videostream=video_stream_new(linphone_core_get_video_port(lc),linphone_core_ipv6_enabled(lc));
 #else
-  lc->videostream=NULL;
+	lc->videostream=NULL;
 #endif
 }
 
+static void linphone_core_dtmf_received(RtpSession* s, int dtmf, void* user_data){
+	LinphoneCore* lc = (LinphoneCore*)user_data;
+	if (lc->vtable.dtmf_received != NULL)
+		lc->vtable.dtmf_received(lc, dtmf);
+}
+
 void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 	osip_from_t *me=linphone_core_get_primary_contact_parsed(lc);
 	const char *tool="linphone-" LINPHONE_VERSION;
@@ -1399,6 +1465,11 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				lc->play_file,
 				lc->rec_file);
 		}
+		if (lc->vtable.dtmf_received!=NULL){
+			/* replace by our default action*/
+			audio_stream_play_received_dtmfs(lc->audiostream,FALSE);
+			rtp_session_signal_connect(lc->audiostream->session,"telephone-event",(RtpCallback)linphone_core_dtmf_received,(unsigned long)lc);
+		}
 		audio_stream_set_rtcp_information(lc->audiostream, cname, tool);
 	}
 #ifdef VIDEO_ENABLED
@@ -1414,6 +1485,8 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 			if (video_params->remoteport>0){
 				if (video_params->relay_session_id!=NULL) 
 					video_stream_set_relay_session_id(lc->videostream,video_params->relay_session_id);
+				video_stream_set_sent_video_size(lc->videostream,linphone_core_get_preferred_video_size(lc));
+				video_stream_enable_self_view(lc->videostream,lc->video_conf.selfview);
 				if (lc->video_conf.display && lc->video_conf.capture)
 					video_stream_start(lc->videostream,
 					call->profile, video_params->remoteaddr, video_params->remoteport,
@@ -1457,7 +1530,7 @@ void linphone_core_stop_media_streams(LinphoneCore *lc){
 	}
 	if (linphone_core_video_preview_enabled(lc)){
 		if (lc->previewstream==NULL){
-			lc->previewstream=video_preview_start(lc->video_conf.device);
+			lc->previewstream=video_preview_start(lc->video_conf.device, lc->video_conf.vsize);
 		}
 	}
 #endif
@@ -1493,12 +1566,10 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 		ms_error("Fail to build answer for call: err=%i",err);
 		return -1;
 	}
-	ms_message("eXosip_call_build_answer done");
 	/*if a sdp answer is computed, send it, else send an offer */
 	sdpmesg=call->sdpctx->answerstr;
 	if (sdpmesg==NULL){
 		offering=TRUE;
-		call->profile=lc->local_profile;
 		ms_message("generating sdp offer");
 		sdpmesg=sdp_context_get_offer(call->sdpctx);
 		
@@ -1667,40 +1738,64 @@ static MSSndCard *get_card_from_string_id(const char *devid, unsigned int cap){
 		if (sndcard==NULL){/*looks like a bug! take the first one !*/
 			const MSList *elem=ms_snd_card_manager_get_list(ms_snd_card_manager_get());
 			sndcard=(MSSndCard*)elem->data;
-        }
+        	}
 	}
 	if (sndcard==NULL) ms_error("Could not find a suitable soundcard !");
 	return sndcard;
 }
 
+bool_t linphone_core_sound_device_can_capture(LinphoneCore *lc, const char *devid){
+	MSSndCard *sndcard;
+	sndcard=ms_snd_card_manager_get_card(ms_snd_card_manager_get(),devid);
+	if (sndcard!=NULL && (ms_snd_card_get_capabilities(sndcard) & MS_SND_CARD_CAP_CAPTURE)) return TRUE;
+	return FALSE;
+}
+
+bool_t linphone_core_sound_device_can_playback(LinphoneCore *lc, const char *devid){
+	MSSndCard *sndcard;
+	sndcard=ms_snd_card_manager_get_card(ms_snd_card_manager_get(),devid);
+	if (sndcard!=NULL && (ms_snd_card_get_capabilities(sndcard) & MS_SND_CARD_CAP_PLAYBACK)) return TRUE;
+	return FALSE;
+}
+
 int linphone_core_set_ringer_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.ring_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
+	MSSndCard *card=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
+	lc->sound_conf.ring_sndcard=card;
+	if (card && lc->ready)
+		lp_config_set_string(lc->config,"sound","ringer_dev_id",ms_snd_card_get_string_id(card));
 	return 0;
 }
 
 int linphone_core_set_playback_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.play_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
+	MSSndCard *card=get_card_from_string_id(devid,MS_SND_CARD_CAP_PLAYBACK);
+	lc->sound_conf.play_sndcard=card;
+	if (card && lc->ready)
+		lp_config_set_string(lc->config,"sound","playback_dev_id",ms_snd_card_get_string_id(card));
 	return 0;
 }
 
 int linphone_core_set_capture_device(LinphoneCore *lc, const char * devid){
-	lc->sound_conf.capt_sndcard=get_card_from_string_id(devid,MS_SND_CARD_CAP_CAPTURE);
+	MSSndCard *card=get_card_from_string_id(devid,MS_SND_CARD_CAP_CAPTURE);
+	lc->sound_conf.capt_sndcard=card;
+	if (card && lc->ready)
+		lp_config_set_string(lc->config,"sound","capture_dev_id",ms_snd_card_get_string_id(card));
 	return 0;
 }
 
 const char * linphone_core_get_ringer_device(LinphoneCore *lc)
 {
-	return ms_snd_card_get_string_id(lc->sound_conf.ring_sndcard);
+	if (lc->sound_conf.ring_sndcard) return ms_snd_card_get_string_id(lc->sound_conf.ring_sndcard);
+	return NULL;
 }
 
 const char * linphone_core_get_playback_device(LinphoneCore *lc)
 {
-	return ms_snd_card_get_string_id(lc->sound_conf.play_sndcard);
+	return lc->sound_conf.play_sndcard ? ms_snd_card_get_string_id(lc->sound_conf.play_sndcard) : NULL;
 }
 
 const char * linphone_core_get_capture_device(LinphoneCore *lc)
 {
-	return ms_snd_card_get_string_id(lc->sound_conf.capt_sndcard);
+	return lc->sound_conf.capt_sndcard ? ms_snd_card_get_string_id(lc->sound_conf.capt_sndcard) : NULL;
 }
 
 /* returns a static array of string describing the sound devices */ 
@@ -1738,9 +1833,11 @@ void linphone_core_set_ring(LinphoneCore *lc,const char *path){
 		ms_free(lc->sound_conf.local_ring);
 	}
 	lc->sound_conf.local_ring=ms_strdup(path);
+	if (lc->ready && lc->sound_conf.local_ring)
+		lp_config_set_string(lc->config,"sound","local_ring",lc->sound_conf.local_ring);
 }
 
-const char *linphone_core_get_ring(LinphoneCore *lc){
+const char *linphone_core_get_ring(const LinphoneCore *lc){
 	return lc->sound_conf.local_ring;
 }
 
@@ -1764,20 +1861,21 @@ int linphone_core_preview_ring(LinphoneCore *lc, const char *ring,LinphoneCoreCb
 }
 
 
-void linphone_core_set_ringback(LinphoneCore *lc,RingBackType type){
-	switch(type){
-		case RINGBACK_TYPE_FR:
-			lc->sound_conf.remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
-		break;
-		case RINGBACK_TYPE_US:
-			lc->sound_conf.remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING_US;
-		break;
+void linphone_core_set_ringback(LinphoneCore *lc, const char *path){
+	if (lc->sound_conf.remote_ring!=0){
+		ms_free(lc->sound_conf.remote_ring);
 	}
+	lc->sound_conf.remote_ring=ms_strdup(path);
+}
+
+const char * linphone_core_get_ringback(const LinphoneCore *lc){
+	return lc->sound_conf.remote_ring;
 }
-RingBackType linphone_core_get_ringback(LinphoneCore *lc);
 
 void linphone_core_enable_echo_cancelation(LinphoneCore *lc, bool_t val){
 	lc->sound_conf.ec=val;
+	if (lc->ready)
+		lp_config_set_int(lc->config,"sound","echocancelation",val);
 }
 
 bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc){
@@ -1937,7 +2035,8 @@ static void toggle_video_preview(LinphoneCore *lc, bool_t val){
 	if (lc->videostream==NULL){
 		if (val){
 			if (lc->previewstream==NULL){
-				lc->previewstream=video_preview_start(lc->video_conf.device);
+				lc->previewstream=video_preview_start(lc->video_conf.device,
+							lc->video_conf.vsize);
 			}
 		}else{
 			if (lc->previewstream!=NULL){
@@ -1956,10 +2055,11 @@ void linphone_core_enable_video(LinphoneCore *lc, bool_t vcap_enabled, bool_t di
 #endif
 	lc->video_conf.capture=vcap_enabled;
 	lc->video_conf.display=display_enabled;
-	if (vcap_enabled && display_enabled)
-		lc->video_conf.show_local=1;
-	else
-		lc->video_conf.show_local=0;
+
+	if (lc->ready){
+		lp_config_set_int(lc->config,"video","display",display_enabled);
+		lp_config_set_int(lc->config,"video","capture",vcap_enabled);
+	}
 
 	/* need to re-apply network bandwidth settings*/
 	linphone_core_set_download_bandwidth(lc,
@@ -1974,14 +2074,29 @@ bool_t linphone_core_video_enabled(LinphoneCore *lc){
 
 void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val){
 	lc->video_conf.show_local=val;
+	if (lc->ready) lp_config_set_int(lc->config,"video","show_local",val);
 }
 
 bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc){
 	return lc->video_conf.show_local;
 }
 
+void linphone_core_enable_self_view(LinphoneCore *lc, bool_t val){
+	lc->video_conf.selfview=val;
+#ifdef VIDEO_ENABLED
+	if (lc->videostream){
+		video_stream_enable_self_view(lc->videostream,val);
+	}
+#endif
+}
+
+bool_t linphone_core_self_view_enabled(const LinphoneCore *lc){
+	return lc->video_conf.selfview;
+}
+
 int linphone_core_set_video_device(LinphoneCore *lc, const char *id){
 	MSWebCam *olddev=lc->video_conf.device;
+	const char *vd;
 	if (id!=NULL){
 		lc->video_conf.device=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),id);
 		if (lc->video_conf.device==NULL){
@@ -1993,6 +2108,13 @@ int linphone_core_set_video_device(LinphoneCore *lc, const char *id){
 	if (olddev!=NULL && olddev!=lc->video_conf.device){
 		toggle_video_preview(lc,FALSE);/*restart the video local preview*/
 	}
+	if (lc->ready){
+		vd=ms_web_cam_get_string_id(lc->video_conf.device);
+		if (vd && strstr(vd,"Static picture")!=NULL){
+			vd=NULL;
+		}
+		lp_config_set_string(lc->config,"video","device",vd);
+	}
 	return 0;
 }
 
@@ -2001,6 +2123,81 @@ const char *linphone_core_get_video_device(const LinphoneCore *lc){
 	return NULL;
 }
 
+unsigned long linphone_core_get_native_video_window_id(const LinphoneCore *lc){
+#ifdef VIDEO_ENABLED
+	if (lc->videostream)
+		return video_stream_get_native_window_id(lc->videostream);
+	if (lc->previewstream)
+		return video_stream_get_native_window_id(lc->previewstream);
+#endif
+	return 0;
+}
+
+static MSVideoSizeDef supported_resolutions[]={
+	{	MS_VIDEO_SIZE_SVGA	,	"svga"	},
+	{	MS_VIDEO_SIZE_4CIF	,	"4cif"	},
+	{	MS_VIDEO_SIZE_VGA	,	"vga"	},
+	{	MS_VIDEO_SIZE_CIF	,	"cif"	},
+	{	MS_VIDEO_SIZE_QVGA	,	"qvga"	},
+	{	MS_VIDEO_SIZE_QCIF	,	"qcif"	},
+	{	{0,0}			,	NULL	}
+};
+
+const MSVideoSizeDef *linphone_core_get_supported_video_sizes(LinphoneCore *lc){
+	return supported_resolutions;
+}
+
+static MSVideoSize video_size_get_by_name(const char *name){
+	MSVideoSizeDef *pdef=supported_resolutions;
+	for(;pdef->name!=NULL;pdef++){
+		if (strcasecmp(name,pdef->name)==0){
+			return pdef->vsize;
+		}
+	}
+	ms_warning("Video resolution %s is not supported in linphone.",name);
+	return (MSVideoSize){0,0};
+}
+
+const char *video_size_get_name(MSVideoSize vsize){
+	MSVideoSizeDef *pdef=supported_resolutions;
+	for(;pdef->name!=NULL;pdef++){
+		if (pdef->vsize.width==vsize.width && pdef->vsize.height==vsize.height){
+			return pdef->name;
+		}
+	}
+	return NULL;
+}
+
+static bool_t video_size_supported(MSVideoSize vsize){
+	if (video_size_get_name(vsize)) return TRUE;
+	ms_warning("Video resolution %ix%i is not supported in linphone.",vsize.width,vsize.height);
+	return FALSE;
+}
+
+
+void linphone_core_set_preferred_video_size(LinphoneCore *lc, MSVideoSize vsize){
+	if (video_size_supported(vsize)){
+		MSVideoSize oldvsize=lc->video_conf.vsize;
+		lc->video_conf.vsize=vsize;
+		if (!ms_video_size_equal(oldvsize,vsize) && lc->previewstream!=NULL){
+			toggle_video_preview(lc,FALSE);
+			toggle_video_preview(lc,TRUE);
+		}
+		if (lc->ready)
+			lp_config_set_string(lc->config,"video","size",video_size_get_name(vsize));
+	}
+}
+
+void linphone_core_set_preferred_video_size_by_name(LinphoneCore *lc, const char *name){
+	MSVideoSize vsize=video_size_get_by_name(name);
+	if (vsize.width!=0)	linphone_core_set_preferred_video_size(lc,vsize);
+	else linphone_core_set_preferred_video_size(lc,MS_VIDEO_SIZE_CIF);
+}
+
+MSVideoSize linphone_core_get_preferred_video_size(LinphoneCore *lc){
+	return lc->video_conf.vsize;
+}
+
 void linphone_core_use_files(LinphoneCore *lc, bool_t yesno){
 	lc->use_files=yesno;
 }
@@ -2121,37 +2318,19 @@ void rtp_config_uninit(LinphoneCore *lc)
 
 void sound_config_uninit(LinphoneCore *lc)
 {
-	/*char tmpbuf[2];*/
 	sound_config_t *config=&lc->sound_conf;
-	lp_config_set_string(lc->config,"sound","playback_dev_id",ms_snd_card_get_string_id(config->play_sndcard));
-	lp_config_set_string(lc->config,"sound","ringer_dev_id",ms_snd_card_get_string_id(config->ring_sndcard));
-	lp_config_set_string(lc->config,"sound","capture_dev_id",ms_snd_card_get_string_id(config->capt_sndcard));
 	ms_free(config->cards);
-	/*
-	lp_config_set_int(lc->config,"sound","rec_lev",config->rec_lev);
-	lp_config_set_int(lc->config,"sound","play_lev",config->play_lev);
-	lp_config_set_int(lc->config,"sound","ring_lev",config->ring_lev);
-	tmpbuf[0]=config->source;
-	tmpbuf[1]='\0';
-	lp_config_set_string(lc->config,"sound","source",tmpbuf);
-	*/
-	lp_config_set_string(lc->config,"sound","local_ring",config->local_ring);
+	
 	lp_config_set_string(lc->config,"sound","remote_ring",config->remote_ring);
-	lp_config_set_int(lc->config,"sound","echocancelation",config->ec);
+	
 	if (config->local_ring) ms_free(config->local_ring);
+	if (config->remote_ring) ms_free(config->remote_ring);
+	ms_snd_card_manager_destroy();
 }
 
 void video_config_uninit(LinphoneCore *lc)
 {
-	video_config_t *config=&lc->video_conf;
-	const char *vd=linphone_core_get_video_device(lc);
-	if (vd && strstr(vd,"Static picture")!=NULL){
-		vd=NULL;
-	}
-	lp_config_set_string(lc->config,"video","device",vd);
-	lp_config_set_int(lc->config,"video","display",config->display);
-	lp_config_set_int(lc->config,"video","capture",config->capture);
-	lp_config_set_int(lc->config,"video","show_local",config->show_local);
+		
 }
 
 void codecs_config_uninit(LinphoneCore *lc)
@@ -2184,15 +2363,11 @@ void codecs_config_uninit(LinphoneCore *lc)
 
 void ui_config_uninit(LinphoneCore* lc)
 {
-	MSList *elem;
-	int i;
-	for (elem=lc->friends,i=0; elem!=NULL; elem=ms_list_next(elem),i++){
-		linphone_friend_write_to_config_file(lc->config,(LinphoneFriend*)elem->data,i);
-		linphone_friend_destroy(elem->data);
+	if (lc->friends){
+		ms_list_for_each(lc->friends,(void (*)(void *))linphone_friend_destroy);
+		ms_list_free(lc->friends);
+		lc->friends=NULL;
 	}
-	linphone_friend_write_to_config_file(lc->config,NULL,i);	/* set the end */
-	ms_list_free(lc->friends);
-	lc->friends=NULL;
 }
 
 LpConfig *linphone_core_get_config(LinphoneCore *lc){
@@ -2217,9 +2392,9 @@ void linphone_core_uninit(LinphoneCore *lc)
 	video_config_uninit(lc);
 	codecs_config_uninit(lc);
 	ui_config_uninit(lc);
-	lp_config_sync(lc->config);
+	if (lp_config_needs_commit(lc->config)) lp_config_sync(lc->config);
 	lp_config_destroy(lc->config);
-
+	sip_setup_unregister_all();
 #ifdef VIDEO_ENABLED
 	if (payload_type_h264_packetization_mode_1!=NULL)
 		payload_type_destroy(payload_type_h264_packetization_mode_1);
