@@ -24,6 +24,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "ortp/ortp.h"
 #include "ortp/payloadtype.h"
 #include "mediastreamer2/mscommon.h"
+#include "mediastreamer2/msvideo.h"
+
+#ifdef IN_LINPHONE
+#include "sipsetup.h"
+#else
+#include "linphone/sipsetup.h"
+#endif
 
 #define LINPHONE_IPADDR_SIZE 64
 #define LINPHONE_HOSTNAME_SIZE 128
@@ -111,9 +118,11 @@ typedef struct codecs_config
 typedef struct video_config{
 	struct _MSWebCam *device;
 	const char **cams;
+	MSVideoSize vsize;
 	bool_t capture;
 	bool_t show_local;
 	bool_t display;
+	bool_t selfview; /*during calls*/
 }video_config_t;
 
 typedef struct ui_config
@@ -138,6 +147,7 @@ typedef struct autoreplier_config
 
 struct _LinphoneCore;
 struct _sdp_context;
+struct _SipSetupContext;
 	
 typedef struct _StreamParams
 {
@@ -265,6 +275,7 @@ char *linphone_friend_get_url(LinphoneFriend *lf);	/* name <sip address> */
 bool_t linphone_friend_get_send_subscribe(const LinphoneFriend *lf);
 LinphoneSubscribePolicy linphone_friend_get_inc_subscribe_policy(const LinphoneFriend *lf);
 LinphoneOnlineStatus linphone_friend_get_status(const LinphoneFriend *lf);
+#define linphone_friend_in_list(lf)	((lf)->lc!=NULL)
 
 #define linphone_friend_url(lf) ((lf)->url)
 
@@ -281,9 +292,11 @@ typedef struct _LinphoneProxyConfig
 	int expires;
 	int reg_time;
 	int rid;
+	char *type;
+	struct _SipSetupContext *ssctx;
+	int auth_failures;
 	bool_t frozen;
 	bool_t reg_sendregister;
-	bool_t auth_pending;
 	bool_t registered;
 	bool_t publish;
 } LinphoneProxyConfig;
@@ -298,17 +311,22 @@ void linphone_proxy_config_enable_register(LinphoneProxyConfig *obj, bool_t val)
 void linphone_proxy_config_edit(LinphoneProxyConfig *obj);
 int linphone_proxy_config_done(LinphoneProxyConfig *obj);
 void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val);
+bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj);
+const char *linphone_proxy_config_get_domain(const LinphoneProxyConfig *cfg);
 #define linphone_proxy_config_get_route(obj)  ((obj)->reg_route)
 #define linphone_proxy_config_get_identity(obj)	((obj)->reg_identity)
 #define linphone_proxy_config_publish_enabled(obj) ((obj)->publish)
 #define linphone_proxy_config_get_addr(obj) ((obj)->reg_proxy)
 #define linphone_proxy_config_get_expires(obj)	((obj)->expires)
 #define linphone_proxy_config_register_enabled(obj) ((obj)->reg_sendregister)
-
+#define linphone_proxy_config_get_core(obj) ((obj)->lc)
 /* destruction is called automatically when removing the proxy config */
 void linphone_proxy_config_destroy(LinphoneProxyConfig *cfg);
 LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(struct _LpConfig *config, int index);
 void linphone_proxy_config_write_to_config_file(struct _LpConfig* config,LinphoneProxyConfig *obj, int index);
+void linphone_proxy_config_set_sip_setup(LinphoneProxyConfig *cfg, const char *type);
+SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig *cfg);
+
 
 typedef struct _LinphoneAuthInfo
 {
@@ -368,7 +386,8 @@ typedef enum _gstate {
   GSTATE_CALL_IN_INVITE,
   GSTATE_CALL_IN_CONNECTED,
   GSTATE_CALL_END,
-  GSTATE_CALL_ERROR
+  GSTATE_CALL_ERROR,
+  GSTATE_INVALID
 } gstate_t;
 
 struct _LinphoneGeneralState {
@@ -379,14 +398,10 @@ struct _LinphoneGeneralState {
 };
 typedef struct _LinphoneGeneralState LinphoneGeneralState;
 
-/* retrieve the current state of the specified state group */
-gstate_t gstate_get_state(gstate_group_t group);
-
-/* private: set the initial states */
-void gstate_initialize(void);
-
 /* private: set a new state */
 void gstate_new_state(struct _LinphoneCore *lc, gstate_t new_state, const char *message);
+/*private*/
+void gstate_initialize(struct _LinphoneCore *lc) ;
 
 typedef void (*ShowInterfaceCb)(struct _LinphoneCore *lc);
 typedef void (*InviteReceivedCb)(struct _LinphoneCore *lc, const char *from);
@@ -402,6 +417,7 @@ typedef void (*AuthInfoRequested)(struct _LinphoneCore *lc, const char *realm, c
 typedef void (*CallLogUpdated)(struct _LinphoneCore *lc, struct _LinphoneCallLog *newcl);
 typedef void (*TextMessageReceived)(struct _LinphoneCore *lc, LinphoneChatRoom *room, const char *from, const char *message);
 typedef void (*GeneralStateChange)(struct _LinphoneCore *lc, LinphoneGeneralState *gstate);
+typedef void (*DtmfReceived)(struct _LinphoneCore* lc, int dtmf);
 
 typedef struct _LinphoneVTable
 {
@@ -423,6 +439,7 @@ typedef struct _LinphoneVTable
 	CallLogUpdated call_log_updated;
 	TextMessageReceived text_received;
 	GeneralStateChange general_state;
+	DtmfReceived dtmf_received;
 } LinphoneCoreVTable;
 
 typedef struct _LCCallbackObj
@@ -484,8 +501,12 @@ typedef struct _LinphoneCore
 	int up_video_bw;
 	int audio_bw;
 	int automatic_action;
+	gstate_t gstate_power;
+	gstate_t gstate_reg;
+	gstate_t gstate_call;
 	bool_t use_files;
 	bool_t apply_nat_settings;
+	bool_t ready;
 #ifdef VINCENT_MAURY_RSVP
 	/* QoS parameters*/
 	int rsvp_enable;
@@ -500,7 +521,8 @@ typedef struct _LinphoneCore
 void linphone_core_enable_logs(FILE *file);
 void linphone_core_enable_logs_with_cb(OrtpLogFunc logfunc);
 void linphone_core_disable_logs(void);
-
+/*sets the user-agent string in sip messages, must be set before linphone_core_new() or linphone_core_init() */
+void linphone_core_set_user_agent(const char *ua_name, const char *version);
 const char *linphone_core_get_version(void);
 
 LinphoneCore *linphone_core_new(const LinphoneCoreVTable *vtable,
@@ -580,6 +602,10 @@ void linphone_core_add_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
 
 void linphone_core_remove_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
 
+const MSList *linphone_core_get_auth_info_list(const LinphoneCore *lc);
+
+LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username);
+
 void linphone_core_abort_authentication(LinphoneCore *lc,  LinphoneAuthInfo *info);
 
 void linphone_core_clear_all_auth_info(LinphoneCore *lc);
@@ -631,6 +657,8 @@ int linphone_core_set_relay_addr(LinphoneCore *lc, const char *addr);
 /* sound functions */
 /* returns a null terminated static array of string describing the sound devices */ 
 const char**  linphone_core_get_sound_devices(LinphoneCore *lc);
+bool_t linphone_core_sound_device_can_capture(LinphoneCore *lc, const char *device);
+bool_t linphone_core_sound_device_can_playback(LinphoneCore *lc, const char *device);
 int linphone_core_get_ring_level(LinphoneCore *lc);
 int linphone_core_get_play_level(LinphoneCore *lc);
 int linphone_core_get_rec_level(LinphoneCore *lc);
@@ -646,11 +674,10 @@ int linphone_core_set_capture_device(LinphoneCore *lc, const char * devid);
 char linphone_core_get_sound_source(LinphoneCore *lc);
 void linphone_core_set_sound_source(LinphoneCore *lc, char source);
 void linphone_core_set_ring(LinphoneCore *lc, const char *path);
-const char *linphone_core_get_ring(LinphoneCore *lc);
+const char *linphone_core_get_ring(const LinphoneCore *lc);
+void linphone_core_set_ringback(LinphoneCore *lc, const char *path);
+const char * linphone_core_get_ringback(const LinphoneCore *lc);
 int linphone_core_preview_ring(LinphoneCore *lc, const char *ring,LinphoneCoreCbFunc func,void * userdata);
-typedef enum {RINGBACK_TYPE_FR,RINGBACK_TYPE_US} RingBackType;
-void linphone_core_set_ringback(LinphoneCore *lc,RingBackType type);
-RingBackType linphone_core_get_ringback(LinphoneCore *lc);
 void linphone_core_enable_echo_cancelation(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc);
 
@@ -658,6 +685,7 @@ void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const cha
 
 LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc);
 
+void linphone_core_interpret_friend_uri(LinphoneCore *lc, const char *uri, char **result);
 void linphone_core_add_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_remove_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_reject_subscriber(LinphoneCore *lc, LinphoneFriend *lf);
@@ -673,16 +701,21 @@ MSList * linphone_core_get_call_logs(LinphoneCore *lc);
 void linphone_core_enable_video(LinphoneCore *lc, bool_t vcap_enabled, bool_t display_enabled);
 bool_t linphone_core_video_enabled(LinphoneCore *lc);
 
-/*play/record support: use files instead of soundcard*/
-void linphone_core_use_files(LinphoneCore *lc, bool_t yesno);
-void linphone_core_set_play_file(LinphoneCore *lc, const char *file);
-void linphone_core_set_record_file(LinphoneCore *lc, const char *file);
-
-
+typedef struct MSVideoSizeDef{
+	MSVideoSize vsize;
+	const char *name;
+}MSVideoSizeDef;
+/* returns a zero terminated table of MSVideoSizeDef*/
+const MSVideoSizeDef *linphone_core_get_supported_video_sizes(LinphoneCore *lc);
+void linphone_core_set_preferred_video_size(LinphoneCore *lc, MSVideoSize vsize);
+MSVideoSize linphone_core_get_preferred_video_size(LinphoneCore *lc);
+void linphone_core_set_preferred_video_size_by_name(LinphoneCore *lc, const char *name);
 
 void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc);
 
+void linphone_core_enable_self_view(LinphoneCore *lc, bool_t val);
+bool_t linphone_core_self_view_enabled(const LinphoneCore *lc);
 
 
 /* returns a null terminated static array of string describing the webcams */ 
@@ -690,6 +723,18 @@ const char**  linphone_core_get_video_devices(const LinphoneCore *lc);
 int linphone_core_set_video_device(LinphoneCore *lc, const char *id);
 const char *linphone_core_get_video_device(const LinphoneCore *lc);
 
+/*function to be used for eventually setting window decorations (icons, title...)*/
+unsigned long linphone_core_get_native_video_window_id(const LinphoneCore *lc);
+
+
+/*play/record support: use files instead of soundcard*/
+void linphone_core_use_files(LinphoneCore *lc, bool_t yesno);
+void linphone_core_set_play_file(LinphoneCore *lc, const char *file);
+void linphone_core_set_record_file(LinphoneCore *lc, const char *file);
+
+gstate_t linphone_core_get_state(const LinphoneCore *lc, gstate_group_t group);
+int linphone_core_get_current_call_duration(const LinphoneCore *lc);
+
 int linphone_core_get_mtu(const LinphoneCore *lc);
 void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
 
