@@ -106,6 +106,7 @@ void linphone_authentication_ok(LinphoneCore *lc, eXosip_event_t *ev);
 void linphone_subscription_new(LinphoneCore *lc, eXosip_event_t *ev);
 void linphone_notify_recv(LinphoneCore *lc,eXosip_event_t *ev);
 LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, int rid);
+void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, eXosip_event_t *ev);
 
 void linphone_subscription_answered(LinphoneCore *lc, eXosip_event_t *ev);
 void linphone_subscription_closed(LinphoneCore *lc, eXosip_event_t *ev);
@@ -118,7 +119,10 @@ MSList *find_friend(MSList *fl, const osip_from_t *friend, LinphoneFriend **lf);
 LinphoneFriend *linphone_find_friend_by_nid(MSList *l, int nid);
 LinphoneFriend *linphone_find_friend_by_sid(MSList *l, int sid);
 
-void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc, const PayloadType *pt);
+void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc);
+void linphone_core_update_allocated_audio_bandwidth_in_call(LinphoneCore *lc, const PayloadType *pt);
 void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call);
 
+void linphone_core_write_friends_config(LinphoneCore* lc);
+
 #endif /* _PRIVATE_H */
