@@ -19,6 +19,7 @@ Copyright (C) 2000  Simon MORLAT (simon.morlat@linphone.org)
  */
  
 #include "linphonecore.h"
+#include "sipsetup.h"
 #include <eXosip2/eXosip.h>
 #include <osipparser2/osip_message.h>
 #include "lpconfig.h"
@@ -42,8 +43,16 @@ void linphone_proxy_config_destroy(LinphoneProxyConfig *obj){
 	if (obj->reg_proxy!=NULL) ms_free(obj->reg_proxy);
 	if (obj->reg_identity!=NULL) ms_free(obj->reg_identity);
 	if (obj->reg_route!=NULL) ms_free(obj->reg_route);
+	if (obj->ssctx!=NULL) sip_setup_context_free(obj->ssctx);
+	if (obj->realm!=NULL) ms_free(obj->realm);
+	if (obj->type!=NULL) ms_free(obj->type);
 }
 
+bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj){
+	return obj->registered;
+}
+
+#if 0
 static void linphone_proxy_config_register(LinphoneProxyConfig *obj){
 	osip_message_t *msg;
 	eXosip_lock();
@@ -51,6 +60,7 @@ static void linphone_proxy_config_register(LinphoneProxyConfig *obj){
 	eXosip_register_send_register(obj->rid,msg);
 	eXosip_unlock();
 }
+#endif
 
 void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer){
 	osip_message_t *msg;
@@ -73,6 +83,7 @@ void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConf
 		return;
 	}
 	eXosip_lock();
+	msg=NULL;
 	eXosip_register_build_register(obj->rid,obj->expires,&msg);
 	osip_message_get_contact(msg,0,&ctt);
 	if (ctt->url->host!=NULL){
@@ -85,7 +96,7 @@ void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConf
 	ctt->url->port=osip_strdup(rport);
 	eXosip_register_send_register(obj->rid,msg);
 	eXosip_unlock();
-	ms_message("Resending new register with updated contact %s:%i",received,rport);
+	ms_message("Resending new register with updated contact %s:%s",received,rport);
 }
 
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr){
@@ -108,7 +119,7 @@ int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *
 
 void linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity){
 	int err=0;
-	osip_from_t *url;
+	osip_from_t *url=NULL;
 	if (identity!=NULL && strlen(identity)>0){
 		osip_from_init(&url);
 		err=osip_from_parse(url,identity);
@@ -117,14 +128,23 @@ void linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *id
 			osip_from_free(url);
 			return;
 		}
-		osip_from_free(url);
 	} else err=-2;
 	if (obj->reg_identity!=NULL) {
 		ms_free(obj->reg_identity);
 		obj->reg_identity=NULL;
 	}
 	if (err==-2) obj->reg_identity=NULL;
-	else obj->reg_identity=ms_strdup(identity);
+	else {
+		obj->reg_identity=ms_strdup(identity);
+		if (obj->realm)
+			ms_free(obj->realm);
+		obj->realm=ms_strdup(url->url->host);
+	}
+	if (url) osip_from_free(url);
+}
+
+const char *linphone_proxy_config_get_domain(const LinphoneProxyConfig *cfg){
+	return cfg->realm;
 }
 
 void linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route)
@@ -191,6 +211,7 @@ void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val){
 
 void linphone_proxy_config_edit(LinphoneProxyConfig *obj){
 	obj->frozen=TRUE;
+	obj->auth_failures=0;
 	if (obj->reg_sendregister){
 		/* unregister */
 		if (obj->registered) {
@@ -440,8 +461,7 @@ void linphone_core_set_default_proxy(LinphoneCore *lc, LinphoneProxyConfig *conf
 	}
 	lc->default_proxy=config;
 	
-}
-	
+}	
 
 void linphone_core_set_default_proxy_index(LinphoneCore *lc, int index){
 	if (index<0) linphone_core_set_default_proxy(lc,NULL);
@@ -474,25 +494,23 @@ LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, i
 	else return (LinphoneProxyConfig*)elem->data;
 }
 
-void linphone_core_retry_proxy_register(LinphoneCore *lc, const char *realm)
-{
-	MSList *elem;
-	for (elem=lc->sip_conf.proxies;elem!=NULL;elem=ms_list_next(elem)){
-		LinphoneProxyConfig *cfg=(LinphoneProxyConfig*)elem->data;
-		/*ms_message("linphone_core_retry_proxy_register: cfg->auth_pending=%i, cfg->realm=%s, realm=%s",
-					cfg->auth_pending,cfg->realm,realm);*/
-		if (cfg->auth_pending && cfg->realm!=NULL && strcmp(cfg->realm,realm)==0){
-			ms_message("Restarting REGISTER request for %s.",cfg->reg_proxy);
-			linphone_proxy_config_register(cfg);
-		}
-	}
-}
-
 const MSList *linphone_core_get_proxy_config_list(const LinphoneCore *lc){
 	return lc->sip_conf.proxies;
 }
 
 
+void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, eXosip_event_t *ev){
+	LinphoneProxyConfig *cfg=linphone_core_get_proxy_config_from_rid(lc, ev->rid);
+	if (cfg){
+			cfg->auth_failures++;
+			/*restart a new register */
+			if (cfg->auth_failures==1){
+				linphone_proxy_config_done(cfg);
+			}
+	}
+}
+
+
 void linphone_proxy_config_write_to_config_file(LpConfig *config, LinphoneProxyConfig *obj, int index)
 {
 	char key[50];
@@ -502,6 +520,9 @@ void linphone_proxy_config_write_to_config_file(LpConfig *config, LinphoneProxyC
 	if (obj==NULL){
 		return;
 	}
+	if (obj->type!=NULL){
+		lp_config_set_string(config,key,"type",obj->type);
+	}
 	if (obj->reg_proxy!=NULL){
 		lp_config_set_string(config,key,"reg_proxy",obj->reg_proxy);
 	}
@@ -519,6 +540,8 @@ void linphone_proxy_config_write_to_config_file(LpConfig *config, LinphoneProxyC
 LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(LpConfig *config, int index)
 {
 	const char *tmp;
+	const char *identity;
+	const char *proxy;
 	LinphoneProxyConfig *cfg;
 	char key[50];
 	
@@ -527,21 +550,52 @@ LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(LpConfig *config
 	if (!lp_config_has_section(config,key)){
 		return NULL;
 	}
-	tmp=lp_config_get_string(config,key,"reg_proxy",NULL);
-	if (tmp==NULL) return NULL;
+
 	cfg=linphone_proxy_config_new();
-	linphone_proxy_config_set_server_addr(cfg,tmp);
-		
+
+	identity=lp_config_get_string(config,key,"reg_identity",NULL);	
+	proxy=lp_config_get_string(config,key,"reg_proxy",NULL);
+	
+	linphone_proxy_config_set_identity(cfg,identity);
+	linphone_proxy_config_set_server_addr(cfg,proxy);
+	
 	tmp=lp_config_get_string(config,key,"reg_route",NULL);
 	if (tmp!=NULL) linphone_proxy_config_set_route(cfg,tmp);
-		
-	tmp=lp_config_get_string(config,key,"reg_identity",NULL);
-	if (tmp!=NULL) linphone_proxy_config_set_identity(cfg,tmp);
-		
+
 	linphone_proxy_config_expires(cfg,lp_config_get_int(config,key,"reg_expires",600));
 	linphone_proxy_config_enableregister(cfg,lp_config_get_int(config,key,"reg_sendregister",0));
 	
 	linphone_proxy_config_enable_publish(cfg,lp_config_get_int(config,key,"publish",0));
 	
+	tmp=lp_config_get_string(config,key,"type",NULL);
+	if (tmp!=NULL && strlen(tmp)>0) 
+		linphone_proxy_config_set_sip_setup(cfg,tmp);
+
 	return cfg;
 }
+
+
+void linphone_proxy_config_set_sip_setup(LinphoneProxyConfig *cfg, const char *type){
+	SipSetup *ss=sip_setup_lookup(type);
+	SipSetupContext *ssc;
+	if (cfg->type)
+		ms_free(cfg->type);
+	cfg->type=ms_strdup(type);
+	if (!ss) return ;
+	ssc=sip_setup_context_new(ss,cfg);
+	if (sip_setup_context_login_account(ssc,cfg->reg_identity,NULL)==0){
+		if (sip_setup_context_get_capabilities(ssc) & SIP_SETUP_CAP_PROXY_PROVIDER){
+			char proxy[256];
+			if (sip_setup_context_get_proxy(ssc,NULL,proxy,sizeof(proxy))==0){
+				linphone_proxy_config_set_server_addr(cfg,proxy);
+			}
+		}
+	}
+	cfg->ssctx=ssc;
+}
+
+SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig *cfg){
+	return cfg->ssctx;
+}
+
+
