@@ -23,12 +23,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define PACKAGE_DATA_DIR "./"
 
 #ifndef LINPHONE_VERSION
-#include "private.h" /*from coreapi */
+#define LINPHONE_VERSION "3.1.0"
 #endif
 
 #endif
 
 #include <gtk/gtk.h>
+#ifdef WIN32
+// alloca is already defined by gtk
+#undef alloca
+#endif
 #include "linphonecore.h"
 
 #include <libintl.h>
@@ -42,6 +46,7 @@ GtkWidget *linphone_gtk_create_window(const char *window_name);
 GtkWidget *linphone_gtk_get_widget(GtkWidget *window, const char *name);
 LinphoneCore *linphone_gtk_get_core(void);
 GtkWidget *linphone_gtk_get_main_window();
+void linphone_gtk_display_something(GtkMessageType type,const gchar *message);
 void linphone_gtk_start_call(GtkWidget *button);
 void linphone_gtk_show_friends(void);
 void linphone_gtk_show_contact(LinphoneFriend *lf);
@@ -56,4 +61,10 @@ void linphone_gtk_log_show(void);
 void linphone_gtk_log_push(OrtpLogLevel lev, const char *fmt, va_list args);
 void linphone_gtk_destroy_log_window(void);
 gboolean linphone_gtk_check_logs();
-
+const gchar *linphone_gtk_get_ui_config(const char *key, const char *def);
+int linphone_gtk_get_ui_config_int(const char *key, int def);
+void linphone_gtk_open_browser(const char *url);
+void linphone_gtk_check_for_new_version(void);
+const char *linphone_gtk_get_lang(const char *config_file);
+SipSetupContext* linphone_gtk_get_default_sip_setup_context(void);
+void linphone_gtk_show_buddy_lookup_window(SipSetupContext *ctx);
