@@ -36,9 +36,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #ifndef PACKAGE_LOCALE_DIR
-#define PACKAGE_LOCALE_DIR "po"
+#define PACKAGE_LOCALE_DIR "share/locale/"
 #endif
 
+#define LINPHONE_ICON "linphone2.png"
+
+const char *this_program_ident_string="linphone_ident_string=" LINPHONE_VERSION;
 
 static LinphoneCore *the_core=NULL;
 static GtkWidget *the_ui=NULL;
@@ -114,6 +117,7 @@ const char *linphone_gtk_get_config_file(){
 }
 
 static void linphone_gtk_init_liblinphone(const char *file){
+	linphone_core_set_user_agent("Linphone", LINPHONE_VERSION);
 	the_core=linphone_core_new(&vtable,file,NULL);
 }
 
@@ -127,6 +131,63 @@ GtkWidget *linphone_gtk_get_main_window(){
 	return the_ui;
 }
 
+static void parse_item(const char *item, const char *window_name, GtkWidget *w){
+	char tmp[64];
+	char *dot;
+	strcpy(tmp,item);
+	dot=strchr(tmp,'.');
+	if (dot){
+		*dot='\0';
+		dot++;
+		if (strcmp(window_name,tmp)==0){
+			GtkWidget *wd=linphone_gtk_get_widget(w,dot);
+			if (wd) gtk_widget_hide(wd);
+		}
+	}
+}
+
+static void parse_hiddens(const char *hiddens, const char *window_name, GtkWidget *w){
+	char item[64];
+	const char *i;
+	const char *b;
+	int len;
+	for(b=i=hiddens;*i!='\0';++i){
+		if (*i==' '){
+			len=MIN(i-b,sizeof(item)-1);
+			strncpy(item,b,len);
+			item[len]='\0';
+			b=i+1;
+			parse_item(item,window_name,w);
+		}
+	}
+	len=MIN(i-b,sizeof(item)-1);
+	if (len>0){
+		strncpy(item,b,len);
+		item[len]='\0';
+		parse_item(item,window_name,w);
+	}
+}
+
+static void linphone_gtk_configure_window(GtkWidget *w, const char *window_name){
+	static const char *icon_path=0;
+	static const char *hiddens=0;
+	static bool_t config_loaded=FALSE;
+	if (linphone_gtk_get_core()==NULL) return;
+	if (config_loaded==FALSE){
+		hiddens=linphone_gtk_get_ui_config("hidden_widgets",NULL);
+		icon_path=linphone_gtk_get_ui_config("icon",NULL);
+		config_loaded=TRUE;
+	}
+	if (hiddens){
+		parse_hiddens(hiddens,window_name,w);
+	}
+	if (icon_path) {
+		GdkPixbuf *pbuf=create_pixbuf(icon_path);
+		gtk_window_set_icon(GTK_WINDOW(w),pbuf);
+		g_object_unref(G_OBJECT(pbuf));
+	}
+}
+
 #ifdef USE_LIBGLADE
 
 GtkWidget *linphone_gtk_create_window(const char *window_name){
@@ -146,6 +207,7 @@ GtkWidget *linphone_gtk_create_window(const char *window_name){
 	glade_xml_signal_autoconnect(gxml);
 	w=glade_xml_get_widget(gxml,window_name);
 	if (w==NULL) g_error("Could not retrieve '%s' window from xml file",window_name);
+	linphone_gtk_configure_window(w,window_name);
 	return w;
 }
 
@@ -187,6 +249,7 @@ void linphone_gtk_display_something(GtkMessageType type,const gchar *message){
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_QUESTION,
                                 GTK_BUTTONS_YES_NO,
+                                "%s",
 				(const gchar*)message);
 		/* connect to some callback : REVISIT */
 		/*
@@ -203,6 +266,7 @@ void linphone_gtk_display_something(GtkMessageType type,const gchar *message){
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   type,
                                   GTK_BUTTONS_CLOSE,
+                                  "%s",
                                   (const gchar*)message);
 		/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
 		g_signal_connect_swapped (G_OBJECT (dialog), "response",
@@ -243,14 +307,47 @@ void linphone_gtk_show_about(){
 	gtk_widget_show(about);
 }
 
+static void set_video_window_decorations(GdkWindow *w){
+	const char *title=linphone_gtk_get_ui_config("title","Linphone");
+	const char *icon_path=linphone_gtk_get_ui_config("icon","linphone2.png");
+	char video_title[256];
+	GdkPixbuf *pbuf=create_pixbuf(icon_path);
+	snprintf(video_title,sizeof(video_title),"%s video",title);
+	gdk_window_set_title(w,video_title);
+	if (pbuf){
+		GList *l=NULL;
+		l=g_list_append(l,pbuf);
+		gdk_window_set_icon_list(w,l);
+		g_list_free(l);
+		g_object_unref(G_OBJECT(pbuf));
+	}
+}
+
+
 static gboolean linphone_gtk_iterate(LinphoneCore *lc){
+	unsigned long id;
+	static unsigned long previd=0;
+
 	linphone_core_iterate(lc);
+	id=linphone_core_get_native_video_window_id(lc);
+	if (id!=previd){
+		ms_message("Updating window decorations");
+		GdkWindow *w;
+		previd=id;
+		if (id!=0){
+			w=gdk_window_foreign_new(id);
+			if (w) {
+				set_video_window_decorations(w);
+				g_object_unref(G_OBJECT(w));
+			}
+			else ms_error("gdk_window_foreign_new() failed");
+		}
+	}
 	return TRUE;
 }
 
 static void load_uri_history(){
 	GtkEntry *uribar=GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar"));
-	LpConfig *cfg=linphone_core_get_config(linphone_gtk_get_core());
 	char key[20];
 	int i;
 	GtkEntryCompletion *gep=gtk_entry_completion_new();
@@ -258,7 +355,7 @@ static void load_uri_history(){
 	for (i=0;;i++){
 		const char *uri;
 		snprintf(key,sizeof(key),"uri%i",i);
-		uri=lp_config_get_string(cfg,"GtkUi",key,NULL);
+		uri=linphone_gtk_get_ui_config(key,NULL);
 		if (uri!=NULL) {
 			GtkTreeIter iter;
 			gtk_list_store_append(model,&iter);
@@ -321,17 +418,32 @@ static void completion_add_text(GtkEntry *entry, const char *text){
 	save_uri_history();
 }
 
+static void linphone_gtk_call_terminated(GtkWidget *mw){
+	gtk_widget_hide(linphone_gtk_get_widget(mw,"terminate_call"));
+	gtk_widget_show(linphone_gtk_get_widget(mw,"start_call"));
+	g_object_set_data(G_OBJECT(mw),"incoming_call",NULL);
+}
+
+gboolean check_call_active(){
+	if (!linphone_core_in_call(linphone_gtk_get_core())){
+		linphone_gtk_call_terminated(linphone_gtk_get_main_window());
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static void linphone_gtk_call_started(GtkWidget *mw){
 	gtk_widget_hide(linphone_gtk_get_widget(mw,"start_call"));
 	gtk_widget_show(linphone_gtk_get_widget(mw,"terminate_call"));
+	g_timeout_add(250,(GSourceFunc)check_call_active,NULL);
 }
 
-void linphone_gtk_start_call(GtkWidget *button){
+void linphone_gtk_start_call(GtkWidget *w){
 	LinphoneCore *lc=linphone_gtk_get_core();
 	if (linphone_core_inc_invite_pending(lc)){
 		/*already in call */
 	}else{
-		GtkWidget *uri_bar=linphone_gtk_get_widget(gtk_widget_get_toplevel(button),"uribar");
+		GtkWidget *uri_bar=linphone_gtk_get_widget(gtk_widget_get_toplevel(w),"uribar");
 		const char *entered=gtk_entry_get_text(GTK_ENTRY(uri_bar));
 		if (linphone_core_invite(lc,entered)==0) {
 			linphone_gtk_call_started(linphone_gtk_get_main_window());
@@ -340,12 +452,11 @@ void linphone_gtk_start_call(GtkWidget *button){
 	}
 }
 
-static void linphone_gtk_call_terminated(GtkWidget *mw){
-	gtk_widget_hide(linphone_gtk_get_widget(mw,"terminate_call"));
-	gtk_widget_show(linphone_gtk_get_widget(mw,"start_call"));
-	g_object_set_data(G_OBJECT(mw),"incoming_call",NULL);
+void linphone_gtk_uri_bar_activate(GtkWidget *w){
+	linphone_gtk_start_call(w);
 }
 
+
 void linphone_gtk_terminate_call(GtkWidget *button){
 	linphone_core_terminate_call(linphone_gtk_get_core(),NULL);
 	linphone_gtk_call_terminated(gtk_widget_get_toplevel(button));
@@ -353,7 +464,7 @@ void linphone_gtk_terminate_call(GtkWidget *button){
 
 void linphone_gtk_decline_call(GtkWidget *button){
 	linphone_core_terminate_call(linphone_gtk_get_core(),NULL);
-	linphone_gtk_call_terminated(gtk_widget_get_toplevel(button));
+	linphone_gtk_call_terminated(linphone_gtk_get_main_window());
 	gtk_widget_destroy(gtk_widget_get_toplevel(button));
 }
 
@@ -366,10 +477,17 @@ void linphone_gtk_accept_call(GtkWidget *button){
 
 void linphone_gtk_set_audio_video(){
 	linphone_core_enable_video(linphone_gtk_get_core(),TRUE,TRUE);
+	linphone_core_enable_video_preview(linphone_gtk_get_core(),TRUE);
 }
 
 void linphone_gtk_set_audio_only(){
 	linphone_core_enable_video(linphone_gtk_get_core(),FALSE,FALSE);
+	linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
+}
+
+void linphone_gtk_enable_self_view(GtkWidget *w){
+	linphone_core_enable_self_view(linphone_gtk_get_core(),
+		gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w)));
 }
 
 void linphone_gtk_used_identity_changed(GtkWidget *w){
@@ -441,6 +559,7 @@ static void linphone_gtk_new_unknown_subscriber(LinphoneCore *lc, LinphoneFriend
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_QUESTION,
                                 GTK_BUTTONS_YES_NO,
+                                "%s",
 				message);
 	g_free(message);
 	g_signal_connect(G_OBJECT (dialog), "response",
@@ -549,15 +668,47 @@ static void linphone_gtk_call_log_updated(LinphoneCore *lc, LinphoneCallLog *cl)
 static void linphone_gtk_general_state(LinphoneCore *lc, LinphoneGeneralState *gstate){
 }
 
-
 static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data){
 	GtkWidget *menu=(GtkWidget*)g_object_get_data(G_OBJECT(status_icon),"menu");
 	gtk_menu_popup(GTK_MENU(menu),NULL,NULL,gtk_status_icon_position_menu,status_icon,button,activate_time);
 }
 
+void linphone_gtk_open_browser(const char *url){
+#ifdef WIN32
+	ShellExecute(0,"open",url,NULL,NULL,1);
+#else
+	char cl[255];
+	snprintf(cl,sizeof(cl),"/usr/bin/x-www-browser %s",url);
+	g_spawn_command_line_async(cl,NULL);
+#endif
+}
+
+void linphone_gtk_link_to_website(GtkWidget *item){
+	const gchar *home=(const gchar*)g_object_get_data(G_OBJECT(item),"home");
+	linphone_gtk_open_browser(home);
+}
+
 static GtkWidget *create_icon_menu(){
 	GtkWidget *menu=gtk_menu_new();
 	GtkWidget *menu_item;
+	GtkWidget *image;
+	gchar *tmp;
+	const gchar *homesite;
+	
+	homesite=linphone_gtk_get_ui_config("home","http://www.linphone.org");
+	menu_item=gtk_image_menu_item_new_with_label(homesite);
+	tmp=g_strdup(homesite);
+	g_object_set_data(G_OBJECT(menu_item),"home",tmp);
+	g_object_weak_ref(G_OBJECT(menu_item),(GWeakNotify)g_free,tmp);
+	
+	image=gtk_image_new_from_stock(GTK_STOCK_HELP,GTK_ICON_SIZE_MENU);
+	gtk_widget_show(image);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),image);
+	//g_object_unref(G_OBJECT(image));
+	gtk_widget_show(menu_item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
+	g_signal_connect(G_OBJECT(menu_item),"activate",(GCallback)linphone_gtk_link_to_website,NULL);
+	
 	menu_item=gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT,NULL);
 	gtk_widget_show(menu_item);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu),menu_item);
@@ -573,13 +724,16 @@ static GtkWidget *create_icon_menu(){
 static GtkStatusIcon *icon=NULL;
 
 static void linphone_gtk_init_status_icon(){
-	GdkPixbuf *pbuf=create_pixbuf("linphone2.png");
+	const char *icon_path=linphone_gtk_get_ui_config("icon",LINPHONE_ICON);
+	GdkPixbuf *pbuf=create_pixbuf(icon_path);
 	GtkWidget *menu=create_icon_menu();
+	const char *title;
 	icon=gtk_status_icon_new_from_pixbuf(pbuf);
 	g_object_unref(G_OBJECT(pbuf));
 	g_signal_connect_swapped(G_OBJECT(icon),"activate",(GCallback)linphone_gtk_show_main_window,linphone_gtk_get_main_window());
 	g_signal_connect(G_OBJECT(icon),"popup-menu",(GCallback)icon_popup_menu,NULL);
-	gtk_status_icon_set_tooltip(icon,_("Linphone - a video internet phone"));
+	title=linphone_gtk_get_ui_config("title",_("Linphone - a video internet phone"));
+	gtk_status_icon_set_tooltip(icon,title);
 	g_object_set_data(G_OBJECT(icon),"menu",menu);
 	g_object_weak_ref(G_OBJECT(icon),(GWeakNotify)gtk_widget_destroy,menu);
 }
@@ -633,7 +787,42 @@ static void linphone_gtk_check_menu_items(void){
 					audio_only ? "audio_only_item" : "video_item")), TRUE);
 }
 
+static void linphone_gtk_configure_main_window(){
+	static gboolean config_loaded=FALSE;
+	static const char *title;
+	static const char *home;
+	static const char *start_call_icon;
+	static const char *stop_call_icon;
+	GtkWidget *w=linphone_gtk_get_main_window();
+	if (!config_loaded){
+		title=linphone_gtk_get_ui_config("title",NULL);
+		home=linphone_gtk_get_ui_config("home","http://www.linphone.org");
+		start_call_icon=linphone_gtk_get_ui_config("start_call_icon",NULL);
+		stop_call_icon=linphone_gtk_get_ui_config("stop_call_icon",NULL);
+		config_loaded=TRUE;
+	}
+	linphone_gtk_configure_window(w,"main_window");
+	if (title) gtk_window_set_title(GTK_WINDOW(w),title);
+	if (start_call_icon){
+		GdkPixbuf *pbuf=create_pixbuf(start_call_icon);
+		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"start_call_icon")),pbuf);
+		g_object_unref(G_OBJECT(pbuf));
+	}
+	if (stop_call_icon){
+		GdkPixbuf *pbuf=create_pixbuf(stop_call_icon);
+		gtk_image_set_from_pixbuf(GTK_IMAGE(linphone_gtk_get_widget(w,"terminate_call_icon")),pbuf);
+		g_object_unref(G_OBJECT(pbuf));
+	}
+	if (home){
+		gchar *tmp;
+		GtkWidget *menu_item=linphone_gtk_get_widget(w,"home_item");
+		tmp=g_strdup(home);
+		g_object_set_data(G_OBJECT(menu_item),"home",tmp);
+	}
+}
+
 static void linphone_gtk_init_main_window(){
+	linphone_gtk_configure_main_window();
 	load_uri_history();
 	linphone_gtk_load_identities();
 	linphone_gtk_set_my_presence(linphone_core_get_presence_info(linphone_gtk_get_core()));
@@ -646,10 +835,16 @@ static void linphone_gtk_init_main_window(){
 
 void linphone_gtk_close(){
 	/* couldn't find a way to prevent closing to destroy the main window*/
-	linphone_core_enable_video_preview(linphone_gtk_get_core(),FALSE);
+	LinphoneCore *lc=linphone_gtk_get_core();
 	the_ui=NULL;
 	the_ui=linphone_gtk_create_window("main");
 	linphone_gtk_init_main_window();
+	/*shutdown call if any*/
+	if (linphone_core_in_call(lc)){
+		linphone_core_terminate_call(lc,NULL);
+		linphone_gtk_call_terminated(the_ui);
+	}
+	linphone_core_enable_video_preview(lc,FALSE);
 }
 
 void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
@@ -694,8 +889,28 @@ void linphone_gtk_log_handler(OrtpLogLevel lev, const char *fmt, va_list args){
 int main(int argc, char *argv[]){
 	void *p;
 	const char *config_file;
+	const char *lang;
+
 	g_thread_init(NULL);
 	gdk_threads_init();
+	
+	config_file=linphone_gtk_get_config_file();
+	if (linphone_core_wake_up_possible_already_running_instance(config_file)==0){
+		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
+		g_warning("This instance is going to exit now.");
+		return 0;
+	}
+	
+	if ((lang=linphone_gtk_get_lang(config_file))!=NULL && lang[0]!='\0'){
+#ifdef WIN32
+		char tmp[128];
+		snprintf(tmp,sizeof(tmp),"LANG=%s",lang);
+		_putenv(tmp);
+#else
+		setenv("LANG",lang,1);
+#endif
+	}
+
 #ifdef ENABLE_NLS
 	p=bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	if (p==NULL) perror("bindtextdomain failed");
@@ -704,12 +919,9 @@ int main(int argc, char *argv[]){
 #else
 	g_message("NLS disabled.\n");
 #endif
-	config_file=linphone_gtk_get_config_file();
-	if (linphone_core_wake_up_possible_already_running_instance(config_file)==0){
-		g_warning("Another running instance of linphone has been detected. It has been woken-up.");
-		g_warning("This instance is going to exit now.");
-		return 0;
-	}
+#ifdef WIN32
+	gtk_rc_add_default_file("./gtkrc");
+#endif
 	gdk_threads_enter();
 	if (!gtk_init_with_args(&argc,&argv,_("A free SIP video-phone"),
 				linphone_options,NULL,NULL)){
@@ -729,11 +941,13 @@ int main(int argc, char *argv[]){
 	linphone_core_enable_logs_with_cb(linphone_gtk_log_handler);
 
 	linphone_gtk_init_liblinphone(config_file);
-	gtk_timeout_add(20,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
-	gtk_timeout_add(20,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
+	/* do not lower timeouts under 30 ms because it exhibits a bug on gtk+/win32, with cpu running 20% all the time...*/
+	gtk_timeout_add(30,(GtkFunction)linphone_gtk_iterate,(gpointer)linphone_gtk_get_core());
+	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
 	linphone_gtk_init_main_window();
 	linphone_gtk_init_status_icon();
-	gtk_widget_show(the_ui);
+	linphone_gtk_show_main_window();
+	linphone_gtk_check_for_new_version();
 	gtk_main();
 	gdk_threads_leave();
 	linphone_gtk_destroy_log_window();
