@@ -73,15 +73,15 @@ typedef enum MSFilterId{
 	MS_SNOW_ENC_ID,
 	MS_CA_READ_ID,
 	MS_CA_WRITE_ID,
-#ifdef __DIRECTSOUND_ENABLED__
 	MS_WINSNDDS_READ_ID,
 	MS_WINSNDDS_WRITE_ID,
-#endif
 	MS_STATIC_IMAGE_ID,
 	MS_V4L2_CAPTURE_ID,
 	MS_H263_OLD_DEC_ID,
 	MS_H263_OLD_ENC_ID,
-	MS_MIRE_ID
+	MS_MIRE_ID,
+	MS_VFW_ID,
+	MS_ICE_ID
 } MSFilterId;
 
 
