@@ -208,10 +208,33 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 		return NULL;
 	}
 #endif
-	if (alsa_set_params(pcm_handle,0,bits,stereo,rate)<0){
-		snd_pcm_close(pcm_handle);
-		return NULL;
+
+	{
+	struct timeval tv1;
+	struct timeval tv2;
+	struct timezone tz;
+	int diff = 0;
+	err = gettimeofday(&tv1, &tz);
+	while (1) { 
+		if (!(alsa_set_params(pcm_handle,0,bits,stereo,rate)<0)){
+			ms_message("alsa_open_r: Audio params set");
+			break;
+		}
+		if (!gettimeofday(&tv2, &tz) && !err) {
+			diff = ((tv2.tv_sec - tv1.tv_sec) * 1000000) + (tv2.tv_usec - tv1.tv_usec);
+		} else {
+			diff = -1;
+		}
+		if ((diff < 0) || (diff > 3000000)) { /* 3 secondes */
+			ms_error("alsa_open_r: Error setting params for more than 3 seconds");
+			snd_pcm_close(pcm_handle);
+			return NULL;
+		}
+		ms_warning("alsa_open_r: Error setting params (for %d micros)", diff);
+		usleep(200000);
 	}
+	}
+
 	err=snd_pcm_start(pcm_handle);
 	if (err<0){
 		ms_warning("snd_pcm_start() failed: %s", snd_strerror(err));
@@ -227,20 +250,43 @@ static snd_pcm_t * alsa_open_w(const char *pcmdev,int bits,int stereo,int rate)
 		ms_warning("alsa_open_w: Error opening PCM device %s",pcmdev );
 		return NULL;
 	}
-	if (alsa_set_params(pcm_handle,1,bits,stereo,rate)<0){
-		snd_pcm_close(pcm_handle);
-		return NULL;
+	
+	{
+	struct timeval tv1;
+	struct timeval tv2;
+	struct timezone tz;
+	int diff = 0;
+	int err;
+	err = gettimeofday(&tv1, &tz);
+	while (1) { 
+		if (!(alsa_set_params(pcm_handle,1,bits,stereo,rate)<0)){
+			ms_message("alsa_open_w: Audio params set");
+			break;
+		}
+		if (!gettimeofday(&tv2, &tz) && !err) {
+			diff = ((tv2.tv_sec - tv1.tv_sec) * 1000000) + (tv2.tv_usec - tv1.tv_usec);
+		} else {
+			diff = -1;
+		}
+		if ((diff < 0) || (diff > 3000000)) { /* 3 secondes */
+			ms_error("alsa_open_w: Error setting params for more than 3 seconds");
+			snd_pcm_close(pcm_handle);
+			return NULL;
+		}
+		ms_warning("alsa_open_w: Error setting params (for %d micros)", diff);
+		usleep(200000);
 	}
+	}
+
 	return pcm_handle;
 }
 
-static int alsa_can_read(snd_pcm_t *dev, int frames)
+static int alsa_can_read(snd_pcm_t *dev)
 {
 	snd_pcm_sframes_t avail;
 	int err;
 
 	avail = snd_pcm_avail_update(dev);
-	ms_debug("*** %s %d %d", __FUNCTION__, (long)avail, frames);
 	if (avail < 0) {
 		ms_error("snd_pcm_avail_update: %s", snd_strerror(avail));	// most probably -EPIPE
 		/* overrun occured, snd_pcm_state() would return SND_PCM_STATE_XRUN
@@ -582,6 +628,7 @@ static MSSndCard * alsa_card_new(int id)
 	char *name=NULL;
 	AlsaData *ad;
 	int err;
+	snd_pcm_t *pcm_handle;
 	if (id!=-1){
 		err=snd_card_get_name(id,&name);
 		if (err<0) {
@@ -605,6 +652,20 @@ static MSSndCard * alsa_card_new(int id)
 		ad->pcmdev=ms_strdup_printf("default:%i",id);
 		ad->mixdev=ms_strdup_printf("default:%i",id);
 	}
+	/*check card capabilities: */
+	obj->capabilities=0;
+	if (snd_pcm_open(&pcm_handle,ad->pcmdev,SND_PCM_STREAM_CAPTURE,SND_PCM_NONBLOCK)==0) {
+		obj->capabilities|=MS_SND_CARD_CAP_CAPTURE;
+		snd_pcm_close(pcm_handle);
+	}
+	if (snd_pcm_open(&pcm_handle,ad->pcmdev,SND_PCM_STREAM_PLAYBACK,SND_PCM_NONBLOCK)==0) {
+		obj->capabilities|=MS_SND_CARD_CAP_PLAYBACK;
+		snd_pcm_close(pcm_handle);
+	}
+	if (obj->capabilities==0){
+		ms_warning("Strange, sound card %s does not seems to be capable of anything.",obj->name);
+		obj->capabilities=MS_SND_CARD_CAP_CAPTURE|MS_SND_CARD_CAP_PLAYBACK;
+	}
 	free(name);
 	/*ms_message("alsa device %s found",obj->name);*/
 	return obj;
@@ -680,6 +741,7 @@ static void * alsa_write_thread(void *p){
 	      {
 		ms_warning("nothing to read");
 		//ms_warning("Fail to read samples %i", count);
+		freemsg(om); /* leak fixed */
 		continue;
 	      }
 	    //ms_warning(" read %i", err);
@@ -768,7 +830,7 @@ void alsa_read_process(MSFilter *obj){
 		ad->handle=alsa_open_r(ad->pcmdev,16,ad->nchannels==2,ad->rate);
 	}
 	if (ad->handle==NULL) return;
-	while (alsa_can_read(ad->handle,samples)>=samples){
+	while (alsa_can_read(ad->handle)>=samples){
 	  
 		int size=samples*2;
 		om=allocb(size,0);
@@ -825,7 +887,7 @@ MSFilterMethod alsa_read_methods[]={
 MSFilterDesc alsa_read_desc={
 	.id=MS_ALSA_READ_ID,
 	.name="MSAlsaRead",
-	.text="Alsa sound source",
+	.text=N_("Alsa sound source"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=0,
 	.noutputs=1,
@@ -922,7 +984,7 @@ MSFilterMethod alsa_write_methods[]={
 MSFilterDesc alsa_write_desc={
 	.id=MS_ALSA_WRITE_ID,
 	.name="MSAlsaWrite",
-	.text="Alsa sound output",
+	.text=N_("Alsa sound output"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=1,
 	.noutputs=0,
