@@ -600,7 +600,7 @@ static MSFilterMethod aq_methods[]={
 MSFilterDesc aq_read_desc={
 	.id=MS_AQ_READ_ID,
 	.name="MSAQRead",
-	.text="Sound capture filter for MacOS X Audio Queue Service",
+	.text=N_("Sound capture filter for MacOS X Audio Queue Service"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=0,
 	.noutputs=1,
@@ -614,7 +614,7 @@ MSFilterDesc aq_read_desc={
 MSFilterDesc aq_write_desc={
 	.id=MS_AQ_WRITE_ID,
 	.name="MSAQWrite",
-	.text="Sound playback filter for MacOS X Audio Queue Service",
+	.text=N_("Sound playback filter for MacOS X Audio Queue Service"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=1,
 	.noutputs=0,
