@@ -62,14 +62,14 @@ static int dtmf_tab[16]={'0','1','2','3','4','5','6','7','8','9','*','#','A','B'
 
 static void on_dtmf_received(RtpSession *s, int dtmf, void * user_data)
 {
-	MSFilter *dtmfgen=(MSFilter*)user_data;
+	AudioStream *stream=(AudioStream*)user_data;
 	if (dtmf>15){
 		ms_warning("Unsupported telephone-event type.");
 		return;
 	}
 	ms_message("Receiving dtmf %c.",dtmf_tab[dtmf]);
-	if (dtmfgen!=NULL){
-		ms_filter_call_method(dtmfgen,MS_DTMF_GEN_PUT,&dtmf_tab[dtmf]);
+	if (stream->dtmfgen!=NULL && stream->play_dtmfs){
+		ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf_tab[dtmf]);
 	}
 }
 
@@ -119,6 +119,20 @@ RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
 	return rtpr;
 }
 
+#if defined(_WIN32_WCE)
+time_t
+time (time_t *t)
+{
+    DWORD timemillis = GetTickCount();
+	if (timemillis>0)
+	{
+		if (t!=NULL)
+			*t = timemillis/1000;
+	}
+	return timemillis/1000;
+}
+#endif
+
 bool_t audio_stream_alive(AudioStream * stream, int timeout){
 	RtpSession *session=stream->session;
 	const rtp_stats_t *stats=rtp_session_get_stats(session);
@@ -195,7 +209,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	stream->session=rtps;
 	
 	stream->dtmfgen=ms_filter_new(MS_DTMF_GEN_ID);
-	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,(unsigned long)stream->dtmfgen);
+	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,(unsigned long)stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)payload_type_changed,(unsigned long)stream);
 	
 	/* creates the local part */
@@ -237,6 +251,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	
 	/* give the encoder/decoder some parameters*/
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
+	ms_message("Payload's bitrate is %i",pt->normal_bitrate);
 	if (pt->normal_bitrate>0){
 		ms_message("Setting audio encoder network bitrate to %i",pt->normal_bitrate);
 		ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
@@ -264,7 +279,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	
 	/* create ticker */
 	stream->ticker=ms_ticker_new();
-
+	ms_ticker_set_name(stream->ticker,"Audio MSTicker");
 	ms_ticker_attach(stream->ticker,stream->soundread);
 	ms_ticker_attach(stream->ticker,stream->rtprecv);
 	
@@ -280,13 +295,15 @@ int audio_stream_start_with_files(AudioStream *stream, RtpProfile *prof,const ch
 
 AudioStream * audio_stream_start(RtpProfile *prof,int locport,const char *remip,int remport,int profile,int jitt_comp,bool_t use_ec)
 {
-	MSSndCard *sndcard;
+	MSSndCard *sndcard_playback;
+	MSSndCard *sndcard_capture;
 	AudioStream *stream;
-	sndcard=ms_snd_card_manager_get_default_card(ms_snd_card_manager_get());
-	if (sndcard==NULL)
+	sndcard_capture=ms_snd_card_manager_get_default_capture_card(ms_snd_card_manager_get());
+	sndcard_playback=ms_snd_card_manager_get_default_playback_card(ms_snd_card_manager_get());
+	if (sndcard_capture==NULL || sndcard_playback==NULL)
 		return NULL;
 	stream=audio_stream_new(locport, ms_is_ipv6(remip));
-	if (audio_stream_start_full(stream,prof,remip,remport,remport+1,profile,jitt_comp,NULL,NULL,sndcard,sndcard,use_ec)==0) return stream;
+	if (audio_stream_start_full(stream,prof,remip,remport,remport+1,profile,jitt_comp,NULL,NULL,sndcard_playback,sndcard_capture,use_ec)==0) return stream;
 	audio_stream_free(stream);
 	return NULL;
 }
@@ -341,9 +358,14 @@ AudioStream *audio_stream_new(int locport, bool_t ipv6){
 	AudioStream *stream=(AudioStream *)ms_new0(AudioStream,1);
 	stream->session=create_duplex_rtpsession(locport,ipv6);
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
+	stream->play_dtmfs=TRUE;
 	return stream;
 }
 
+void audio_stream_play_received_dtmfs(AudioStream *st, bool_t yesno){
+	st->play_dtmfs=yesno;
+}
+
 int audio_stream_start_now(AudioStream *stream, RtpProfile * prof,  const char *remip, int remport, int rem_rtcp_port, int payload_type, int jitt_comp, MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec){
 	return audio_stream_start_full(stream,prof,remip,remport,rem_rtcp_port,
 		payload_type,jitt_comp,NULL,NULL,playcard,captcard,use_ec);
@@ -403,6 +425,7 @@ RingStream * ring_start_with_cb(const char *file,int interval,MSSndCard *sndcard
 	ms_filter_call_method(stream->source,MS_FILTER_GET_NCHANNELS,&tmp);
 	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_NCHANNELS,&tmp);
 	stream->ticker=ms_ticker_new();
+	ms_ticker_set_name(stream->ticker,"Audio (ring) MSTicker");
 	ms_filter_link(stream->source,0,stream->sndwrite,0);
 	ms_ticker_attach(stream->ticker,stream->source);
 	return stream;
@@ -420,7 +443,9 @@ void ring_stop(RingStream *stream){
 
 int audio_stream_send_dtmf(AudioStream *stream, char dtmf)
 {
-	ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SEND_DTMF,&dtmf);
-	ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf);
+	if (stream->rtpsend)
+		ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SEND_DTMF,&dtmf);
+	if (stream->dtmfgen)
+		ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf);
 	return 0;
 }
