@@ -294,11 +294,12 @@ int ms_load_plugins(const char *dir){
         os_handle = LoadLibraryEx (szPluginFile, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
         if (os_handle==NULL)
         {
+			ms_warning("Fail to load plugin %s: error %i",szPluginFile,GetLastError());
             os_handle = LoadLibraryEx (szPluginFile, NULL, 0);
         }
         SetErrorMode (em);
         if (os_handle==NULL)
-            ms_warning("Fail to load plugin %s :", szPluginFile); 
+            ms_warning("Fail to load plugin %s", szPluginFile); 
         else
         {
             init_func_t initroutine;
@@ -310,7 +311,7 @@ int ms_load_plugins(const char *dir){
             initroutine = (init_func_t) GetProcAddress (os_handle, szMethodName);
 			if (initroutine!=NULL){
 				initroutine();
-				ms_message("Plugin loaded.");
+				ms_message("Plugin loaded (%s)", szPluginFile);
 				num++;
 			}else{
 				ms_warning("Could not locate init routine of plugin %s", szPluginFile);
@@ -346,7 +347,8 @@ int ms_load_plugins(const char *dir){
 		return -1;
 	}
 	while( (de=readdir(ds))!=NULL){
-		if (de->d_type==DT_REG && strstr(de->d_name,PLUGINS_EXT)!=NULL){
+		if ((de->d_type==DT_REG && strstr(de->d_name,PLUGINS_EXT)!=NULL)
+		    || (de->d_type==DT_UNKNOWN && strstr(de->d_name,PLUGINS_EXT)==de->d_name+strlen(de->d_name)-strlen(PLUGINS_EXT))) {
 			void *handle;
 			fullpath=ms_strdup_printf("%s/%s",dir,de->d_name);
 			ms_message("Loading plugin %s...",fullpath);
@@ -381,7 +383,7 @@ int ms_load_plugins(const char *dir){
 				if (initroutine!=NULL){
 					init_func_t func=(init_func_t)initroutine;
 					func();
-					ms_message("Plugin loaded.");
+					ms_message("Plugin loaded (%s)", fullpath);
 					num++;
 				}else{
 					ms_warning("Could not locate init routine of plugin %s",de->d_name);
@@ -471,7 +473,10 @@ extern MSWebCamDesc v4l2_desc;
 #endif
 
 #ifdef WIN32
-extern MSWebCamDesc ms_v4w_cam_desc;
+extern MSWebCamDesc ms_vfw_cam_desc;
+#endif
+#if defined(WIN32) && defined(HAVE_DIRECTSHOW)
+extern MSWebCamDesc ms_directx_cam_desc;
 #endif
 
 extern MSWebCamDesc static_image_desc;
@@ -484,8 +489,11 @@ static MSWebCamDesc * ms_web_cam_descs[]={
 #ifdef __linux
 	&v4l_desc,
 #endif
-#ifdef WIN32
-	&ms_v4w_cam_desc,
+#if defined(WIN32) && defined(HAVE_DIRECTSHOW)
+	&ms_directx_cam_desc,
+#endif
+#if defined(WIN32) && !defined(HAVE_DIRECTSHOW)
+	&ms_vfw_cam_desc,
 #endif
 	&mire_desc,
 	&static_image_desc,
