@@ -18,11 +18,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "mediastreamer2/msfilter.h"
+#include <math.h>
 
 #if defined(_WIN32_WCE)
 #define DISABLE_SPEEX
 #endif
 
+#ifndef HAVE_SPEEXDSP
+#define DISABLE_SPEEX
+#endif
+
 #ifndef DISABLE_SPEEX
 #include <speex/speex_preprocess.h>
 #endif
@@ -31,19 +36,22 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define CONF_GRAN_MAX 12 /* limit for 'too much data' */
 #endif
 
-//#ifndef CONF_GRAN
-//#define CONF_GRAN (160*4)
-//#endif
 #define CONF_NSAMPLES 160*4*4 /* (CONF_GRAN/2) */
 #ifndef CONF_MAX_PINS
 #define CONF_MAX_PINS 32
 #endif
 
+static const float max_e=(float)32767*32767;
+static const float coef=(float)0.1;
+
 typedef struct Channel{
 	MSBufferizer buff;
 	int16_t input[CONF_NSAMPLES];
 	bool_t has_contributed;
 	bool_t is_used;
+
+	int is_speaking;
+
 	int count;
 	int missed;
 
@@ -55,6 +63,8 @@ typedef struct Channel{
 	SpeexPreprocessState *speex_pp;
 #endif
 
+	float energy;
+
 } Channel;
 
 typedef struct ConfState{
@@ -62,7 +72,13 @@ typedef struct ConfState{
 	int sum[CONF_NSAMPLES];
 	int enable_directmode;
 	int enable_vad;
+
+	int enable_halfduplex;
+	int vad_prob_start;
+	int vad_prob_continue;
+
 	int agc_level;
+	int max_gain;
 	int mix_mode;
 	int samplerate;
 
@@ -73,52 +89,78 @@ typedef struct ConfState{
 
 
 static void channel_init(ConfState *s, Channel *chan, int pos){
+	float f;
+	int val;
+	memset(chan, 0, sizeof(Channel));
 	ms_bufferizer_init(&chan->buff);
 #ifndef DISABLE_SPEEX
-	//chan->speex_pp = speex_preprocess_state_init((s->conf_gran/2) *(s->samplerate/8000), s->samplerate);
 	chan->speex_pp = speex_preprocess_state_init(s->conf_gran/2, s->samplerate);
-	if (chan->speex_pp!=NULL) {
-		float f;
-		int val;
+	if (chan->speex_pp==NULL)
+		return;
+
+	/* configure sound card input on pin 0 */
+	val=0;
+	if (pos==0)
+		val=1;
+
+	if (s->enable_halfduplex>0 && pos%2==1)
+		val=1;
+
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DENOISE, &val);
+	val = -30;
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &val);
+
+	/* enable VAD only on incoming RTP stream */
+	val=0;
+	if (pos%2==1 || (pos==0 && s->enable_halfduplex>0))
+	{
 		val=1;
-		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DENOISE, &val);
-		/* enable VAD only on incoming RTP stream */
-		if (pos%2==1)
-		{
-			val=1;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_VAD, &val);
-		}
-		/* enable AGC only on local soundcard */
-		if (s->agc_level>0 && pos==0)
-		{
-			val=1;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC, &val);
-			f=s->agc_level;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC_LEVEL, &f);
-#if 0
-			val=40;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &val);
-#endif
-		}
-		else
-		{
-			val=0;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC, &val);
-			f=8000;
-			speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC_LEVEL, &f);
-		}
-		val=0;
-		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB, &val);
-		f=.4;
-		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB_DECAY, &f);
-		f=.3;
-		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB_LEVEL, &f);
 	}
+
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_VAD, &val);
+	if (s->vad_prob_start>0 && s->vad_prob_continue>0)
+	{
+		val = s->vad_prob_start; // xx%
+		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_PROB_START, &val);
+		val = s->vad_prob_continue; // xx%
+		speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_PROB_CONTINUE, &val);
+	}
+
+	/* enable AGC only on local soundcard */
+	val=0;
+	f=8000;
+	if (s->agc_level>0 && pos==0)
+		val=1;
+	else if (pos==0 && s->enable_halfduplex>0)
+		val=1;
+	else if ( pos%2==1 && s->enable_halfduplex>0)
+		val=1;
+	if (s->agc_level>0)
+		f=(float)s->agc_level;
+
+
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC, &val);
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC_LEVEL, &f);
+	val=s->max_gain;
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &val);
+
+	val=0;
+#if 0
+	val=1; // do more testing
+#endif
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB, &val);
+	f=(float).4;
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB_DECAY, &f);
+	f=(float).3;
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_SET_DEREVERB_LEVEL, &f);
+
 #endif
 }
 
 static void channel_uninit(Channel *chan){
 	ms_bufferizer_uninit(&chan->buff);
+	chan->is_speaking=0;
+	chan->energy=0;
 #ifndef DISABLE_SPEEX
 	if (chan->speex_pp!=NULL)
 	    speex_preprocess_state_destroy(chan->speex_pp);
@@ -137,6 +179,7 @@ static void conf_init(MSFilter *f){
 	s->enable_directmode=FALSE;
 	s->enable_vad=TRUE;
 	s->agc_level=0;
+	s->max_gain=30;
 	s->mix_mode=TRUE;
 	s->adaptative_msconf_buf=2;
 	f->data=s;
@@ -208,7 +251,42 @@ static bool_t should_process(MSFilter *f, ConfState *s){
 	return FALSE;
 }
 
-static void conf_sum(ConfState *s){
+#ifndef DISABLE_SPEEX
+static double powerspectrum_stat_beyond8K(struct Channel *chan)
+{
+	spx_int32_t ps_size = 0;
+	spx_int32_t *ps = NULL;
+	double mystat = 0;
+	float fftmul = 1.0 / (32768.0);
+
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_GET_PSD_SIZE, &ps_size);
+	ps = (spx_int32_t*)ortp_malloc(sizeof(spx_int32_t)*ps_size);
+	speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_GET_PSD, ps);
+
+
+	mystat = 0;
+	for (int i=ps_size/2;i < ps_size; i++) {
+		double yp;
+		yp = sqrtf(sqrtf(static_cast<float>(ps[i]))) - 1.0f;
+		yp = yp * fftmul;
+		yp = MIN(yp * 3000.0, 1.0);
+		mystat = yp + mystat;
+	}
+
+	ortp_free(ps);
+
+	/* value between 0 and 100? */
+	mystat = mystat*100*2/ps_size;
+	if (mystat<0)
+		mystat=0;
+	if (mystat>100)
+		mystat=100;
+
+	return mystat;
+}
+#endif
+
+static void conf_sum(MSFilter *f, ConfState *s){
 	int i,j;
 	Channel *chan;
 	memset(s->sum,0,s->conf_nsamples*sizeof(int));
@@ -246,24 +324,24 @@ static void conf_sum(ConfState *s){
 		chan=&s->channels[i];
 
 		/* skip soundread and short buffer entry */
-#if 0
-		if (i>0
-			&& ms_bufferizer_get_avail(&chan->buff)>=s->conf_gran*s->adaptative_msconf_buf)
-		{
-			while (ms_bufferizer_get_avail(&chan->buff)>=s->conf_gran*2)
-#endif
 		if (i>0 
 			&& ms_bufferizer_get_avail(&chan->buff)> s->conf_gran
 			&& ms_bufferizer_get_avail(&chan->buff)> (ms_bufferizer_get_avail(&s->channels[0].buff)+s->conf_gran*6) )
 		{
+#if 0
+			int loudness;
+#endif
+			int vad=0;
 			while (ms_bufferizer_get_avail(&chan->buff)> (ms_bufferizer_get_avail(&s->channels[0].buff)) )
 			{
 				ms_bufferizer_read(&chan->buff,(uint8_t*)chan->input,s->conf_gran);
+#if 0
+				speex_preprocess_ctl(chan->speex_pp, SPEEX_PREPROCESS_GET_AGC_LOUDNESS, &loudness);
+#endif
 				/* we want to remove 4 packets (40ms) in a near future: */
 #ifndef DISABLE_SPEEX
 				if (chan->speex_pp!=NULL && s->enable_vad==TRUE)
 				{
-					int vad;
 					vad = speex_preprocess(chan->speex_pp, (short*)chan->input, NULL);
 					if (vad==1)
 						break; /* voice detected: process as usual */
@@ -283,16 +361,66 @@ static void conf_sum(ConfState *s){
 				s->sum[j]+=chan->input[j];
 			}
 			chan->has_contributed=TRUE;
+
 			chan->stat_processed++;
 		}
 		else if (ms_bufferizer_get_avail(&chan->buff)>=s->conf_gran)
 		{
+			struct channel_volume {
+				float energy;
+				int channel;
+			};
+			struct channel_volume vol;
+			float en;
+
 			ms_bufferizer_read(&chan->buff,(uint8_t*)chan->input,s->conf_gran);
+
+			en=chan->energy;
+			for(j=0;j<s->conf_nsamples;++j){
+				float s=chan->input[j];
+				en=(s*s*coef) + ((float)1.0-coef)*en;
+			}
+			chan->energy=en;
+			vol.energy = 10*log10f(chan->energy/max_e);
+			vol.channel = i;
+			ms_filter_notify(f, MS_CONF_CHANNEL_VOLUME, (void*)&vol);
+
 #ifndef DISABLE_SPEEX
-			if (chan->speex_pp!=NULL && s->enable_vad==TRUE)
+			if (chan->speex_pp!=NULL && s->enable_vad==TRUE && i==0)
 			{
 				int vad;
 				vad = speex_preprocess(chan->speex_pp, (short*)chan->input, NULL);
+				ms_filter_notify(f, MS_CONF_SPEEX_PREPROCESS_MIC, (void*)chan->speex_pp);
+
+				if (s->enable_halfduplex>0)
+				{
+					double mystat = powerspectrum_stat_beyond8K(chan);
+					//ms_message("is_speaking (chan=%i) -> on/stat=%.3lf", i, mystat);
+					if (mystat>10)
+					{
+						ms_message("is_speaking (chan=%i) -> on/stat=%.3lf", i, mystat);
+						s->channels[0].is_speaking=20; /* keep RTP muted for the next few ms */
+					}
+					else
+					{
+						s->channels[0].is_speaking--;
+					}
+				}
+
+			}
+			else if (chan->speex_pp!=NULL && s->enable_vad==TRUE)
+			{
+				int vad;
+
+				if (s->enable_halfduplex>0)
+				{
+					vad = speex_preprocess(chan->speex_pp, (short*)chan->input, NULL);
+				}
+				else
+				{
+					vad = speex_preprocess(chan->speex_pp, (short*)chan->input, NULL);
+					//speex_preprocess_estimate_update(chan->speex_pp, (short*)chan->input);
+				}
 			}
 #endif
 
@@ -300,6 +428,7 @@ static void conf_sum(ConfState *s){
 				s->sum[j]+=chan->input[j];
 			}
 			chan->has_contributed=TRUE;
+
 			chan->stat_processed++;
 		} else {
 			chan->stat_missed++;
@@ -328,20 +457,20 @@ static inline int16_t saturate(int sample){
 	return (int16_t)sample;
 }
 
-static mblk_t * conf_output(ConfState *s, Channel *chan){
+static mblk_t * conf_output(ConfState *s, Channel *chan, int16_t attenuation){
 	mblk_t *m=allocb(s->conf_gran,0);
 	int i;
 	int tmp;
 	if (chan->has_contributed==TRUE){
 		for (i=0;i<s->conf_nsamples;++i){
 			tmp=s->sum[i]-(int)chan->input[i];
-			*((int16_t*)m->b_wptr)=saturate(tmp);
+			*((int16_t*)m->b_wptr)=saturate(tmp)/attenuation;
 			m->b_wptr+=2;
 		}
 	}else{
 		for (i=0;i<s->conf_nsamples;++i){
 			tmp=s->sum[i];
-			*((int16_t*)m->b_wptr)=saturate(tmp);
+			*((int16_t*)m->b_wptr)=saturate(tmp)/attenuation;
 			m->b_wptr+=2;
 		}
 	}
@@ -356,7 +485,10 @@ static void conf_dispatch(MSFilter *f, ConfState *s){
 	for (i=0;i<CONF_MAX_PINS;++i){
 		if (f->outputs[i]!=NULL){
 			chan=&s->channels[i];
-			m=conf_output(s,chan);
+			if (s->channels[0].is_speaking<0 && i%2==1) // MIC is NOT speaking -> send silence on RTP
+				m=conf_output(s,chan, 5);
+			else
+				m=conf_output(s,chan, 1);
 			ms_queue_put(f->outputs[i],m);
 		}
 	}
@@ -386,7 +518,7 @@ static void conf_process(MSFilter *f){
 
 	/*do the job */
 	while(should_process(f,s)==TRUE){
-		conf_sum(s);
+		conf_sum(f, s);
 		conf_dispatch(f,s);
 	}
 
@@ -437,16 +569,12 @@ static void conf_process(MSFilter *f){
 static void conf_postprocess(MSFilter *f){
 	int i;
 	ConfState *s=(ConfState*)f->data;
-	Channel *chan;
-	/*read from all inputs and put into bufferizers*/
-	for (i=0;i<CONF_MAX_PINS;++i){
-		if (f->inputs[i]!=NULL){
-			chan=&s->channels[i];
-			ms_bufferizer_uninit(&chan->buff);
-			ms_bufferizer_init(&chan->buff);
-		}
-	}
+	for (i=0;i<CONF_MAX_PINS;i++)
+		channel_uninit(&s->channels[i]);
+    for (i=0;i<CONF_MAX_PINS;i++)
+		channel_init(s, &s->channels[i], i);
 }
+
 static int msconf_set_sr(MSFilter *f, void *arg){
 	ConfState *s=(ConfState*)f->data;
 	int i;
@@ -479,6 +607,55 @@ static int msconf_enable_agc(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int msconf_set_max_gain(MSFilter *f, void *arg){
+	ConfState *s=(ConfState*)f->data;
+	int i;
+	s->max_gain = *(int*)arg;
+
+	for (i=0;i<CONF_MAX_PINS;i++)
+		channel_uninit(&s->channels[i]);
+    for (i=0;i<CONF_MAX_PINS;i++)
+		channel_init(s, &s->channels[i], i);
+	return 0;
+}
+
+
+static int msconf_enable_halfduplex(MSFilter *f, void *arg){
+	ConfState *s=(ConfState*)f->data;
+	int i;
+	s->enable_halfduplex = *(int*)arg;
+
+	for (i=0;i<CONF_MAX_PINS;i++)
+		channel_uninit(&s->channels[i]);
+    for (i=0;i<CONF_MAX_PINS;i++)
+		channel_init(s, &s->channels[i], i);
+	return 0;
+}
+
+static int msconf_set_vad_prob_start(MSFilter *f, void *arg){
+	ConfState *s=(ConfState*)f->data;
+	int i;
+	s->vad_prob_start = *(int*)arg;
+
+	for (i=0;i<CONF_MAX_PINS;i++)
+		channel_uninit(&s->channels[i]);
+    for (i=0;i<CONF_MAX_PINS;i++)
+		channel_init(s, &s->channels[i], i);
+	return 0;
+}
+
+static int msconf_set_vad_prob_continue(MSFilter *f, void *arg){
+	ConfState *s=(ConfState*)f->data;
+	int i;
+	s->vad_prob_continue = *(int*)arg;
+
+	for (i=0;i<CONF_MAX_PINS;i++)
+		channel_uninit(&s->channels[i]);
+    for (i=0;i<CONF_MAX_PINS;i++)
+		channel_init(s, &s->channels[i], i);
+	return 0;
+}
+
 static int msconf_enable_vad(MSFilter *f, void *arg){
 	ConfState *s=(ConfState*)f->data;
 	int i;
@@ -544,9 +721,15 @@ static MSFilterMethod msconf_methods[]={
 	{	MS_FILTER_ENABLE_DIRECTMODE, msconf_enable_directmode },
 	{	MS_FILTER_ENABLE_VAD, msconf_enable_vad },
 	{	MS_FILTER_ENABLE_AGC, msconf_enable_agc },
+	{	MS_FILTER_SET_MAX_GAIN, msconf_set_max_gain },
+	
 	{	MS_FILTER_GET_STAT_DISCARDED, msconf_get_stat_discarded },
 	{	MS_FILTER_GET_STAT_MISSED, msconf_get_stat_missed },
 	{	MS_FILTER_GET_STAT_OUTPUT, msconf_get_stat_processed },
+
+	{	MS_FILTER_ENABLE_HALFDUPLEX, msconf_enable_halfduplex },
+	{	MS_FILTER_SET_VAD_PROB_START, msconf_set_vad_prob_start },
+	{	MS_FILTER_SET_VAD_PROB_CONTINUE, msconf_set_vad_prob_continue },
 	{	0			, NULL}
 };
 
@@ -555,7 +738,7 @@ static MSFilterMethod msconf_methods[]={
 MSFilterDesc ms_conf_desc={
 	MS_CONF_ID,
 	"MSConf",
-	"A filter to make conferencing",
+	N_("A filter to make conferencing"),
 	MS_FILTER_OTHER,
 	NULL,
 	CONF_MAX_PINS,
@@ -573,7 +756,7 @@ MSFilterDesc ms_conf_desc={
 MSFilterDesc ms_conf_desc={
 	.id=MS_CONF_ID,
 	.name="MSConf",
-	.text="A filter to make conferencing",
+	.text=N_("A filter to make conferencing"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=CONF_MAX_PINS,
 	.noutputs=CONF_MAX_PINS,
@@ -588,3 +771,4 @@ MSFilterDesc ms_conf_desc={
 #endif
 
 MS_FILTER_DESC_EXPORT(ms_conf_desc)
+
