@@ -29,8 +29,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 struct SenderData {
 	RtpSession *session;
-	struct CandidatePair *cpair;	/* table of 10 cpair */
-	int round;
 	uint32_t tsoff;
 	uint32_t skip_until;
 	int rate;
@@ -49,8 +47,6 @@ static void sender_init(MSFilter * f)
 	SenderData *d = (SenderData *)ms_new(SenderData, 1);
 
 	d->session = NULL;
-	d->cpair = NULL;
-	d->round = 0;
 	d->tsoff = 0;
 	d->skip_until = 0;
 	d->skip = FALSE;
@@ -74,22 +70,9 @@ static int sender_send_dtmf(MSFilter * f, void *arg)
 	const char *dtmf = (const char *) arg;
 	SenderData *d = (SenderData *) f->data;
 
-	ms_mutex_lock(&f->lock);
+	ms_filter_lock(f);
 	d->dtmf = dtmf[0];
-	ms_mutex_unlock(&f->lock);
-	return 0;
-}
-
-static int sender_set_sdpcandidates(MSFilter * f, void *arg)
-{
-	SenderData *d = (SenderData *) f->data;
-	struct CandidatePair *scs = NULL;
-
-	if (d == NULL)
-		return -1;
-
-	scs = (struct CandidatePair *) arg;
-	d->cpair = scs;
+	ms_filter_unlock(f);
 	return 0;
 }
 
@@ -101,7 +84,9 @@ static int sender_set_session(MSFilter * f, void *arg)
 		rtp_profile_get_payload(rtp_session_get_profile(s),
 								rtp_session_get_send_payload_type(s));
 	if (pt != NULL) {
-		d->rate = pt->clock_rate;
+		if (strcasecmp("g722", pt->mime_type)==0 )
+			d->rate=8000;
+		else d->rate = pt->clock_rate;
 	} else {
 		ms_warning("Sending undefined payload type ?");
 	}
@@ -177,7 +162,6 @@ static void sender_process(MSFilter * f)
 	SenderData *d = (SenderData *) f->data;
 	RtpSession *s = d->session;
 
-	struct CandidatePair *cp = d->cpair;
 	mblk_t *im;
 	uint32_t timestamp;
 
@@ -212,27 +196,21 @@ static void sender_process(MSFilter * f)
 			}
 			freemsg(im);
 		}else{
-		  if (d->mute_mic==FALSE)
-		    {
-			int pt = mblk_get_payload_type(im);
-			header = rtp_session_create_packet(s, 12, NULL, 0);
-			if (pt>0)
-				rtp_set_payload_type(header, pt);
-			rtp_set_markbit(header, mblk_get_marker_info(im));
-			header->b_cont = im;
-			rtp_session_sendm_with_ts(s, header, timestamp);
-		    }
-		  else
-		    {
-			freemsg(im);
-		    }
+			if (d->mute_mic==FALSE){
+				int pt = mblk_get_payload_type(im);
+				header = rtp_session_create_packet(s, 12, NULL, 0);
+				if (pt>0)
+					rtp_set_payload_type(header, pt);
+				rtp_set_markbit(header, mblk_get_marker_info(im));
+				header->b_cont = im;
+				rtp_session_sendm_with_ts(s, header, timestamp);
+			}
+			else{
+				freemsg(im);
+			}
 		}
 		ms_filter_unlock(f);
 	}
-
-	/* regularly send STUN request */
-	ice_sound_send_stun_request(s, cp, d->round);
-	d->round++;
 }
 
 static MSFilterMethod sender_methods[] = {
@@ -240,7 +218,6 @@ static MSFilterMethod sender_methods[] = {
 	{MS_RTP_SEND_UNMUTE_MIC, sender_unmute_mic},
 	{MS_RTP_SEND_SET_SESSION, sender_set_session},
 	{MS_RTP_SEND_SEND_DTMF, sender_send_dtmf},
-	{MS_RTP_SEND_SET_CANDIDATEPAIRS, sender_set_sdpcandidates},
 	{MS_RTP_SEND_SET_RELAY_SESSION_ID, sender_set_relay_session_id},
 	{0, NULL}
 };
@@ -250,7 +227,7 @@ static MSFilterMethod sender_methods[] = {
 MSFilterDesc ms_rtp_send_desc = {
 	MS_RTP_SEND_ID,
 	"MSRtpSend",
-	"RTP output filter",
+	N_("RTP output filter"),
 	MS_FILTER_OTHER,
 	NULL,
 	1,
@@ -268,7 +245,7 @@ MSFilterDesc ms_rtp_send_desc = {
 MSFilterDesc ms_rtp_send_desc = {
 	.id = MS_RTP_SEND_ID,
 	.name = "MSRtpSend",
-	.text = "RTP output filter",
+	.text = N_("RTP output filter"),
 	.category = MS_FILTER_OTHER,
 	.ninputs = 1,
 	.noutputs = 0,
@@ -282,8 +259,6 @@ MSFilterDesc ms_rtp_send_desc = {
 
 struct ReceiverData {
 	RtpSession *session;
-	OrtpEvQueue *ortp_event;
-	struct CandidatePair *cpair;	/* table of 10 cpair */
 	int rate;
 };
 
@@ -293,9 +268,7 @@ static void receiver_init(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *)ms_new(ReceiverData, 1);
 
-	d->ortp_event = ortp_ev_queue_new();
 	d->session = NULL;
-	d->cpair = NULL;
 	d->rate = 8000;
 	f->data = d;
 }
@@ -303,15 +276,11 @@ static void receiver_init(MSFilter * f)
 static void receiver_postprocess(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *) f->data;
-	if (d->session!=NULL && d->ortp_event!=NULL)
-	  rtp_session_unregister_event_queue(d->session, d->ortp_event);
 }
 
 static void receiver_uninit(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *) f->data;
-	if (d->ortp_event!=NULL)
-	  ortp_ev_queue_destroy(d->ortp_event);
 	ms_free(f->data);
 }
 
@@ -323,7 +292,9 @@ static int receiver_set_session(MSFilter * f, void *arg)
 											  rtp_session_get_recv_payload_type
 											  (s));
 	if (pt != NULL) {
-		d->rate = pt->clock_rate;
+		if (strcasecmp("g722", pt->mime_type)==0 )
+			d->rate=8000;
+		else d->rate = pt->clock_rate;
 	} else {
 		ms_warning("Receiving undefined payload type ?");
 	}
@@ -332,19 +303,6 @@ static int receiver_set_session(MSFilter * f, void *arg)
 	return 0;
 }
 
-static int receiver_set_sdpcandidates(MSFilter * f, void *arg)
-{
-	ReceiverData *d = (ReceiverData *) f->data;
-	struct CandidatePair *scs = NULL;
-
-	if (d == NULL)
-		return -1;
-
-	scs = (struct CandidatePair *) arg;
-	d->cpair = scs;
-	return 0;
-}
-
 static void receiver_preprocess(MSFilter * f){
 	ReceiverData *d = (ReceiverData *) f->data;
 	if (d->session){
@@ -356,8 +314,6 @@ static void receiver_preprocess(MSFilter * f){
 				rtp_session_flush_sockets(d->session);
 		}
 	}
-	if (d->session!=NULL && d->ortp_event!=NULL)
-		rtp_session_register_event_queue(d->session, d->ortp_event);
 }
 
 static void receiver_process(MSFilter * f)
@@ -377,30 +333,10 @@ static void receiver_process(MSFilter * f)
 		rtp_get_payload(m,&m->b_rptr);
 		ms_queue_put(f->outputs[0], m);
 	}
-
-	/* check received STUN request */
-	if (d->ortp_event!=NULL)
-	{
-		OrtpEvent *evt = ortp_ev_queue_get(d->ortp_event);
-
-		while (evt != NULL) {
-			if (ortp_event_get_type(evt) ==
-				ORTP_EVENT_STUN_PACKET_RECEIVED) {
-				ice_process_stun_message(d->session, d->cpair, evt);
-			}
-			if (ortp_event_get_type(evt) ==
-				ORTP_EVENT_TELEPHONE_EVENT) {
-			}
-
-			ortp_event_destroy(evt);
-			evt = ortp_ev_queue_get(d->ortp_event);
-		}
-	}
 }
 
 static MSFilterMethod receiver_methods[] = {
 	{MS_RTP_RECV_SET_SESSION, receiver_set_session},
-	{MS_RTP_RECV_SET_CANDIDATEPAIRS, receiver_set_sdpcandidates},
 	{0, NULL}
 };
 
@@ -409,7 +345,7 @@ static MSFilterMethod receiver_methods[] = {
 MSFilterDesc ms_rtp_recv_desc = {
 	MS_RTP_RECV_ID,
 	"MSRtpRecv",
-	"RTP input filter",
+	N_("RTP input filter"),
 	MS_FILTER_OTHER,
 	NULL,
 	0,
@@ -427,7 +363,7 @@ MSFilterDesc ms_rtp_recv_desc = {
 MSFilterDesc ms_rtp_recv_desc = {
 	.id = MS_RTP_RECV_ID,
 	.name = "MSRtpRecv",
-	.text = "RTP input filter",
+	.text = N_("RTP input filter"),
 	.category = MS_FILTER_OTHER,
 	.ninputs = 0,
 	.noutputs = 1,
