@@ -45,6 +45,7 @@ void ms_ticker_init(MSTicker *ticker)
 #ifdef WIN32_TIMERS
 	ticker->TimeEvent=NULL;
 #endif
+	ticker->name=ms_strdup("MSTicker");
 	ms_ticker_start(ticker);
 }
 
@@ -61,10 +62,15 @@ void ms_ticker_stop(MSTicker *s){
 	ms_thread_join(s->thread,NULL);
 }
 
+void ms_ticker_set_name(MSTicker *s, const char *name){
+	if (s->name) ms_free(s->name);
+	s->name=ms_strdup(name);
+}
 
 void ms_ticker_uninit(MSTicker *ticker)
 {
 	ms_ticker_stop(ticker);
+	ms_free(ticker->name);
 	ms_mutex_destroy(&ticker->lock);
 }
 
@@ -184,7 +190,7 @@ static bool_t filter_can_process(MSFilter *f, int tick){
 
 static void call_process(MSFilter *f){
 	bool_t process_done=FALSE;
-	if (f->desc->ninputs==0){
+	if (f->desc->ninputs==0 || f->desc->flags & MS_FILTER_IS_PUMP){
 		ms_filter_process(f);
 	}else{
 		while (ms_filter_inputs_have_data(f)) {
@@ -260,7 +266,7 @@ static uint64_t get_cur_time(void *unused){
 	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
 #else
 	struct timespec ts;
-	if (clock_gettime(CLOCK_REALTIME,&ts)<0){
+	if (clock_gettime(CLOCK_MONOTONIC,&ts)<0){
 		ms_fatal("clock_gettime() doesn't work: %s",strerror(errno));
 	}
 	return (ts.tv_sec*1000LL) + (ts.tv_nsec/1000000LL);
@@ -349,7 +355,7 @@ void * ms_ticker_run(void *arg)
 			}else{
 				late=-diff;
 				if (late>s->interval*5 && late>lastlate){
-					ms_warning("We are late of %d miliseconds.",late);
+					ms_warning("%s: We are late of %d miliseconds.",s->name,late);
 				}
 				lastlate=late;
 				break; /*exit the while loop */
@@ -360,7 +366,7 @@ void * ms_ticker_run(void *arg)
 	}
 	ms_mutex_unlock(&s->lock);
 	unset_high_prio(precision);
-	ms_message("MSTicker thread exiting");
+	ms_message("%s thread exiting",s->name);
 
 	ms_thread_exit(NULL);
 	return NULL;
