@@ -63,6 +63,9 @@ int yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
 	}else if (size==(MS_VIDEO_SIZE_1024_W*MS_VIDEO_SIZE_1024_H*3)/2){
 		w=MS_VIDEO_SIZE_1024_W;
 		h=MS_VIDEO_SIZE_1024_H;
+	}else if (size==(MS_VIDEO_SIZE_800X600_W*MS_VIDEO_SIZE_800X600_H*3)/2){
+		w=MS_VIDEO_SIZE_800X600_W;
+		h=MS_VIDEO_SIZE_800X600_H;
 	}else if (size==(160*112*3)/2){/*format used by econf*/
 		w=160;
 		h=112;
@@ -156,12 +159,12 @@ MSPixFmt ms_fourcc_to_pix_fmt(uint32_t fourcc){
 void rgb24_revert(uint8_t *buf, int w, int h, int linesize){
 	uint8_t *p,*pe;
 	int i,j;
-	uint8_t *end=buf+(h*linesize);
+	uint8_t *end=buf+((h-1)*linesize);
 	uint8_t exch;
 	p=buf;
 	pe=end-1;
 	for(i=0;i<h/2;++i){
-		for(j=0;i<w*3;++j){
+		for(j=0;j<w*3;++j){
 			exch=p[i];
 			p[i]=pe[-i];
 			pe[-i]=exch;
@@ -177,12 +180,35 @@ void rgb24_copy_revert(uint8_t *dstbuf, int dstlsz,
 	const uint8_t *psrc;
 	uint8_t *pdst;
 	psrc=srcbuf;
-	pdst=dstbuf+(dstlsz*roi.height);
+	pdst=dstbuf+(dstlsz*(roi.height-1));
 	for(i=0;i<roi.height;++i){
-		for(j=0;j<roi.width;++j){
-			pdst[roi.width-1-j]=psrc[j];
+		for(j=0;j<roi.width*3;++j){
+			pdst[(roi.width*3)-1-j]=psrc[j];
 		}
 		pdst-=dstlsz;
 		psrc+=srclsz;
 	}
 }
+
+static MSVideoSize _ordered_vsizes[]={
+	{MS_VIDEO_SIZE_QCIF_W,MS_VIDEO_SIZE_QCIF_H},
+	{MS_VIDEO_SIZE_QVGA_W,MS_VIDEO_SIZE_QVGA_H},
+	{MS_VIDEO_SIZE_CIF_W,MS_VIDEO_SIZE_CIF_H},
+	{MS_VIDEO_SIZE_VGA_W,MS_VIDEO_SIZE_VGA_H},
+	{MS_VIDEO_SIZE_4CIF_W,MS_VIDEO_SIZE_4CIF_H},
+	{MS_VIDEO_SIZE_720P_W,MS_VIDEO_SIZE_720P_H},
+	{0,0}
+};
+
+MSVideoSize ms_video_size_get_just_lower_than(MSVideoSize vs){
+	MSVideoSize *p;
+	MSVideoSize ret;
+	ret.width=0;
+	ret.height=0;
+	for(p=_ordered_vsizes;p->width!=0;++p){
+		if (ms_video_size_greater_than(vs,*p) && !ms_video_size_equal(vs,*p)){
+			ret=*p;
+		}else return ret;
+	}
+	return ret;
+};
