@@ -36,6 +36,8 @@ int ms_pix_fmt_to_ffmpeg(MSPixFmt fmt){
 	switch(fmt){
 		case MS_RGB24:
 			return PIX_FMT_RGB24;
+		case MS_RGB24_REV:
+			return PIX_FMT_BGR24;
 		case MS_YUV420P:
 			return PIX_FMT_YUV420P;
 		case MS_YUYV:
@@ -55,6 +57,8 @@ MSPixFmt ffmpeg_pix_fmt_to_ms(int fmt){
 	switch(fmt){
 		case PIX_FMT_RGB24:
 			return MS_RGB24;
+		case PIX_FMT_BGR24:
+			return MS_RGB24_REV;
 		case PIX_FMT_YUV420P:
 			return MS_YUV420P;
 		case PIX_FMT_YUYV422:
@@ -132,6 +136,10 @@ static void pixconv_process(MSFilter *f){
 				s->out_fmt,SWS_FAST_BILINEAR,
                 		NULL, NULL, NULL);
 			}
+			if (s->in_fmt==PIX_FMT_BGR24){
+				inbuf.data[0]+=inbuf.linesize[0]*(s->size.height-1);
+				inbuf.linesize[0]=-inbuf.linesize[0];
+			}
 			if (sws_scale(s->sws_ctx,inbuf.data,inbuf.linesize, 0,
 				s->size.height, s->outbuf.planes, s->outbuf.strides)<0){
 				ms_error("MSPixConv: Error in sws_scale().");
@@ -166,7 +174,7 @@ static MSFilterMethod methods[]={
 MSFilterDesc ms_pix_conv_desc={
 	MS_PIX_CONV_ID,
 	"MSPixConv",
-	"A pixel format converter",
+	N_("A pixel format converter"),
 	MS_FILTER_OTHER,
 	NULL,
 	1,
@@ -184,7 +192,7 @@ MSFilterDesc ms_pix_conv_desc={
 MSFilterDesc ms_pix_conv_desc={
 	.id=MS_PIX_CONV_ID,
 	.name="MSPixConv",
-	.text="A pixel format converter",
+	.text=N_("A pixel format converter"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=1,
 	.noutputs=1,
