@@ -59,6 +59,7 @@ typedef struct EncState{
 	float fps;
 	int maxbr;
 	int qmin;
+	uint32_t framenum;
 	bool_t req_vfu;
 }EncState;
 
@@ -153,7 +154,7 @@ static void enc_init(MSFilter *f, enum CodecID codec)
 	f->data=s;
 	ms_ffmpeg_check_init();
 	s->profile=0;/*always default to profile 0*/
-	s->comp_buf=allocb(32000,0);
+	s->comp_buf=NULL;
 	s->fps=15;
 	s->mtu=ms_get_payload_max_size()-2;/*-2 for the H263 payload header*/
 	s->maxbr=500000;
@@ -162,6 +163,7 @@ static void enc_init(MSFilter *f, enum CodecID codec)
 	s->vsize.height=MS_VIDEO_SIZE_CIF_H;
 	s->qmin=2;
 	s->req_vfu=FALSE;
+	s->framenum=0;
 	s->av_context.codec=NULL;
 }
 
@@ -188,7 +190,7 @@ static void prepare(EncState *s){
 		/*snow does not like 1st pass rate control*/
 		/*and rate control eats too much cpu with CIF high fps pictures*/
 		c->rc_max_rate=(float)s->maxbr*0.8;
-		c->rc_min_rate=c->bit_rate;
+		c->rc_min_rate=0;
 		c->rc_buffer_size=c->rc_max_rate;
 	}else{
 		/*use qmin instead*/
@@ -202,7 +204,7 @@ static void prepare(EncState *s){
 	c->time_base.den = (int)s->fps;
 	c->gop_size=(int)s->fps*5; /*emit I frame every 5 seconds*/
 	c->pix_fmt=PIX_FMT_YUV420P;
-	
+	s->comp_buf=allocb(c->bit_rate*2,0);
 	if (s->codec==CODEC_ID_SNOW){
 		c->strict_std_compliance=-2;
 	}
@@ -245,7 +247,6 @@ static void prepare_mpeg4(EncState *s){
 
 static void enc_uninit(MSFilter  *f){
 	EncState *s=(EncState*)f->data;
-	if (s->comp_buf!=NULL)	freemsg(s->comp_buf);
 	ms_free(s);
 }
 #if 0
@@ -292,6 +293,10 @@ static void enc_postprocess(MSFilter *f){
 		avcodec_close(&s->av_context);
 		s->av_context.codec=NULL;
 	}
+	if (s->comp_buf!=NULL)	{
+		freemsg(s->comp_buf);
+		s->comp_buf=NULL;
+	}
 }
 
 static void add_rfc2190_header(mblk_t **packet, AVCodecContext *context){
@@ -455,13 +460,18 @@ static void process_frame(MSFilter *f, mblk_t *inm){
 	int error;
 	mblk_t *comp_buf=s->comp_buf;
 	int comp_buf_sz=comp_buf->b_datap->db_lim-comp_buf->b_datap->db_base;
-	
+
 	/* convert image if necessary */
 	avcodec_get_frame_defaults(&pict);
 	avpicture_fill((AVPicture*)&pict,(uint8_t*)inm->b_rptr,c->pix_fmt,c->width,c->height);
 	
 	/* timestamp used by ffmpeg, unset here */
 	pict.pts=AV_NOPTS_VALUE;
+		
+	if (s->framenum==(int)(s->fps*2.0) || s->framenum==(int)(s->fps*4.0)){
+		/*sends an I frame at 2 seconds and 4 seconds after the beginning of the call*/
+		s->req_vfu=TRUE;
+	}
 	if (s->req_vfu){
 		pict.pict_type=FF_I_TYPE;
 		s->req_vfu=FALSE;
@@ -477,6 +487,7 @@ static void process_frame(MSFilter *f, mblk_t *inm){
 	error=avcodec_encode_video(c, (uint8_t*)comp_buf->b_wptr,comp_buf_sz, &pict);
 	if (error<=0) ms_warning("ms_AVencoder_process: error %i.",error);
 	else{
+		s->framenum++;
 		if (c->coded_frame->pict_type==FF_I_TYPE){
 			ms_message("Emitting I-frame");
 		}
@@ -512,8 +523,12 @@ static int enc_set_br(MSFilter *f, void *arg){
 	bool_t snow=s->codec==CODEC_ID_SNOW;
 	s->maxbr=*(int*)arg;
 	if (s->maxbr>=1024000 && s->codec!=CODEC_ID_H263P){
-		s->vsize.width = MS_VIDEO_SIZE_4CIF_W;
-		s->vsize.height = MS_VIDEO_SIZE_4CIF_H;
+		s->vsize.width = MS_VIDEO_SIZE_SVGA_W;
+		s->vsize.height = MS_VIDEO_SIZE_SVGA_H;
+		s->fps=17;
+	}else if (s->maxbr>=800000 && s->codec!=CODEC_ID_H263P){
+		s->vsize.width = MS_VIDEO_SIZE_VGA_W;
+		s->vsize.height = MS_VIDEO_SIZE_VGA_H;
 		s->fps=17;
 	}else if (s->maxbr>=512000){
 		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
@@ -569,7 +584,7 @@ static MSFilterMethod methods[]={
 MSFilterDesc ms_h263_enc_desc={
 	MS_H263_ENC_ID,
 	"MSH263Enc",
-	"A video H.263 encoder using ffmpeg library.",
+	N_("A video H.263 encoder using ffmpeg library."),
 	MS_FILTER_ENCODER,
 	"H263-1998",
 	1, /*MS_YUV420P is assumed on this input */
@@ -585,7 +600,7 @@ MSFilterDesc ms_h263_enc_desc={
 MSFilterDesc ms_h263_old_enc_desc={
 	MS_H263_OLD_ENC_ID,
 	"MSH263OldEnc",
-	"A video H.263 encoder using ffmpeg library. It is compliant with old RFC2190 spec.",
+	N_("A video H.263 encoder using ffmpeg library. It is compliant with old RFC2190 spec."),
 	MS_FILTER_ENCODER,
 	"H263",
 	1, /*MS_YUV420P is assumed on this input */
@@ -601,7 +616,7 @@ MSFilterDesc ms_h263_old_enc_desc={
 MSFilterDesc ms_mpeg4_enc_desc={
 	MS_MPEG4_ENC_ID,
 	"MSMpeg4Enc",
-	"A video MPEG4 encoder using ffmpeg library.",
+	N_("A video MPEG4 encoder using ffmpeg library."),
 	MS_FILTER_ENCODER,
 	"MP4V-ES",
 	1, /*MS_YUV420P is assumed on this input */
@@ -617,7 +632,7 @@ MSFilterDesc ms_mpeg4_enc_desc={
 MSFilterDesc ms_snow_enc_desc={
 	MS_SNOW_ENC_ID,
 	"MSSnowEnc",
-	"A video snow encoder using ffmpeg library.",
+	N_("A video snow encoder using ffmpeg library."),
 	MS_FILTER_ENCODER,
 	"x-snow",
 	1, /*MS_YUV420P is assumed on this input */
@@ -635,7 +650,7 @@ MSFilterDesc ms_snow_enc_desc={
 MSFilterDesc ms_h263_enc_desc={
 	.id=MS_H263_ENC_ID,
 	.name="MSH263Enc",
-	.text="A video H.263 encoder using ffmpeg library.",
+	.text=N_("A video H.263 encoder using ffmpeg library."),
 	.category=MS_FILTER_ENCODER,
 	.enc_fmt="H263-1998",
 	.ninputs=1, /*MS_YUV420P is assumed on this input */
@@ -651,7 +666,7 @@ MSFilterDesc ms_h263_enc_desc={
 MSFilterDesc ms_h263_old_enc_desc={
 	.id=MS_H263_OLD_ENC_ID,
 	.name="MSH263Enc",
-	.text="A video H.263 encoder using ffmpeg library, compliant with old RFC2190 spec.",
+	.text=N_("A video H.263 encoder using ffmpeg library, compliant with old RFC2190 spec."),
 	.category=MS_FILTER_ENCODER,
 	.enc_fmt="H263",
 	.ninputs=1, /*MS_YUV420P is assumed on this input */
@@ -667,7 +682,7 @@ MSFilterDesc ms_h263_old_enc_desc={
 MSFilterDesc ms_mpeg4_enc_desc={
 	.id=MS_MPEG4_ENC_ID,
 	.name="MSMpeg4Enc",
-	.text="A video MPEG4 encoder using ffmpeg library.",
+	.text=N_("A video MPEG4 encoder using ffmpeg library."),
 	.category=MS_FILTER_ENCODER,
 	.enc_fmt="MP4V-ES",
 	.ninputs=1, /*MS_YUV420P is assumed on this input */
@@ -683,11 +698,11 @@ MSFilterDesc ms_mpeg4_enc_desc={
 MSFilterDesc ms_snow_enc_desc={
 	.id=MS_SNOW_ENC_ID,
 	.name="MSSnowEnc",
-	.text="The snow codec is royalty-free and is open-source. \n"
+	.text=N_("The snow codec is royalty-free and is open-source. \n"
 		"It uses innovative techniques that makes it one of the best video "
 		"codec. It is implemented within the ffmpeg project.\n"
 		"However it is under development and compatibility with other versions "
-		"cannot be guaranteed.",
+		"cannot be guaranteed."),
 	.category=MS_FILTER_ENCODER,
 	.enc_fmt="x-snow",
 	.ninputs=1, /*MS_YUV420P is assumed on this input */
