@@ -23,11 +23,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
+
+/*required for dllexport of win_display_desc */
+#define INVIDEOUT_C 1
 #include "mediastreamer2/msvideoout.h"
 
 #include "ffmpeg-priv.h"
 
-static int video_out_handle_resize(MSFilter *f, void *arg);
+static int video_out_set_vsize(MSFilter *f,void *arg);
 
 bool_t ms_display_poll_event(MSDisplay *d, MSDisplayEvent *ev){
 	if (d->desc->pollevent)
@@ -35,7 +38,7 @@ bool_t ms_display_poll_event(MSDisplay *d, MSDisplayEvent *ev){
 	else return FALSE;
 }
 
-#ifndef WIN32
+#ifdef HAVE_SDL
 
 #include <SDL/SDL.h>
 #include <SDL/SDL_video.h>
@@ -50,6 +53,17 @@ static SDL_Surface *sdl_screen=0;
 
 #include <SDL/SDL_syswm.h>
 
+static long sdl_get_native_window_id(){
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+	if ( SDL_GetWMInfo(&info) ) {
+		if ( info.subsystem == SDL_SYSWM_X11 ) {
+			return (long) info.info.x11.wmwindow;
+		}
+	}
+	return 0;
+}
+
 static void sdl_show_window(bool_t show){
 	SDL_SysWMinfo info;
 	SDL_VERSION(&info.version);
@@ -76,8 +90,15 @@ static void sdl_show_window(bool_t show){
 	ms_warning("SDL window show/hide not implemented");
 }
 
+static long sdl_get_native_window_id(){
+	ms_warning("sdl_get_native_window_id not implemented");
+	return 0;
+}
+
 #endif
 
+static void sdl_display_uninit(MSDisplay *obj);
+
 static SDL_Overlay * sdl_create_window(int w, int h){
 	SDL_Overlay *lay;
 	sdl_screen = SDL_SetVideoMode(w,h, 0,SDL_SWSURFACE|SDL_RESIZABLE);
@@ -87,15 +108,18 @@ static SDL_Overlay * sdl_create_window(int w, int h){
 		return NULL;
 	}
 	if (sdl_screen->flags & SDL_HWSURFACE) ms_message("SDL surface created in hardware");
-	SDL_WM_SetCaption("Linphone Video", NULL);
+	SDL_WM_SetCaption("Video window", NULL);
 	ms_message("Using yuv overlay.");
-	lay=SDL_CreateYUVOverlay(w,h,SDL_YV12_OVERLAY,sdl_screen);
+	lay=SDL_CreateYUVOverlay(w , h ,SDL_YV12_OVERLAY,sdl_screen);
 	if (lay==NULL){
 		ms_warning("Couldn't create yuv overlay: %s\n",
 						SDL_GetError());
 		return NULL;
 	}else{
-		if (lay->hw_overlay) ms_message("YUV overlay using hardware acceleration.");
+		ms_message("%i x %i YUV overlay created: hw_accel=%i, pitches=%i,%i,%i",lay->w,lay->h,lay->hw_overlay,
+			lay->pitches[0],lay->pitches[1],lay->pitches[2]);
+		ms_message("planes= %p %p %p  %i %i",lay->pixels[0],lay->pixels[1],lay->pixels[2],
+			lay->pixels[1]-lay->pixels[0],lay->pixels[2]-lay->pixels[1]);
 	}
 	return lay;
 }
@@ -113,8 +137,9 @@ static bool_t sdl_display_init(MSDisplay *obj, MSPicture *fbuf){
 		sdl_initialized=TRUE;
 		ms_mutex_init(&sdl_mutex,NULL);
 	}
-	if (obj->data!=NULL)
+	if (obj->data!=NULL){
 		SDL_FreeYUVOverlay((SDL_Overlay*)obj->data);
+	}
 	
 	lay=sdl_create_window(fbuf->w, fbuf->h);
 	if (lay){
@@ -124,8 +149,11 @@ static bool_t sdl_display_init(MSDisplay *obj, MSPicture *fbuf){
 		fbuf->strides[0]=lay->pitches[0];
 		fbuf->strides[1]=lay->pitches[2];
 		fbuf->strides[2]=lay->pitches[1];
+		fbuf->w=lay->w;
+		fbuf->h=lay->h;
 		obj->data=lay;
 		sdl_show_window(TRUE);
+		obj->window_id=sdl_get_native_window_id();
 		return TRUE;
 	}
 	return FALSE;
@@ -158,36 +186,22 @@ static void sdl_display_update(MSDisplay *obj){
 
 static bool_t sdl_poll_event(MSDisplay *obj, MSDisplayEvent *ev){
 	SDL_Event event;
-	static MSDisplayEvent last_ev;
-	static struct timeval tv;
-	static bool_t got_rs_ev=FALSE;
-	struct timeval cur;
-	int elapsed;
 	bool_t ret=FALSE;
+	if (sdl_screen==NULL) return FALSE;
 	ms_mutex_lock(&sdl_mutex);
 	if (SDL_PollEvent(&event)){
 		ms_mutex_unlock(&sdl_mutex);
 		switch(event.type){
 			case SDL_VIDEORESIZE:
-				last_ev.evtype=MS_DISPLAY_RESIZE_EVENT;
-				last_ev.w=event.resize.w;
-				last_ev.h=event.resize.h;
-				got_rs_ev=TRUE;
-				gettimeofday(&tv,NULL);
+				ev->evtype=MS_DISPLAY_RESIZE_EVENT;
+				ev->w=event.resize.w;
+				ev->h=event.resize.h;
+				return TRUE;
 			break;
 			default:
 			break;
 		}
 	}else ms_mutex_unlock(&sdl_mutex);
-	if (got_rs_ev){
-		gettimeofday(&cur,NULL);
-		elapsed=((cur.tv_sec-tv.tv_sec)*1000) + ((cur.tv_usec-tv.tv_usec)/1000);
-		if (elapsed>1000){
-			got_rs_ev=FALSE;
-			*ev=last_ev;
-			ret=TRUE;
-		}
-	}
 	return ret;
 }
 
@@ -210,10 +224,10 @@ MSDisplayDesc ms_sdl_display_desc={
 	.unlock=sdl_display_unlock,
 	.update=sdl_display_update,
 	.uninit=sdl_display_uninit,
-	.pollevent=sdl_poll_event
+	.pollevent=sdl_poll_event,
 };
 
-#else
+#elif defined(WIN32)
 
 #include <Vfw.h>
 
@@ -225,6 +239,7 @@ typedef struct _WinDisplay{
 	MSDisplayEvent last_rsz;
 	uint8_t *rgb;
 	int rgb_len;
+	struct SwsContext *sws;
 	bool_t new_ev;
 }WinDisplay;
 
@@ -291,7 +306,7 @@ static HWND create_window(int w, int h)
 		ms_error("AdjustWindowRect failed.");
 	}
 	ms_message("AdjustWindowRect: %li,%li %li,%li",rect.left,rect.top,rect.right,rect.bottom);
-	hwnd=CreateWindow("Video Window", "Video window", WS_OVERLAPPEDWINDOW|WS_VISIBLE ,
+	hwnd=CreateWindow("Video Window", "Video window", /*WS_OVERLAPPEDWINDOW*/ WS_THICKFRAME|WS_VISIBLE ,
 						CW_USEDEFAULT, CW_USEDEFAULT, rect.right-rect.left,rect.bottom-rect.top,
 													NULL, NULL, hInstance, NULL);
 	if (hwnd==NULL){
@@ -315,6 +330,8 @@ static bool_t win_display_init(MSDisplay *obj, MSPicture *fbuf){
 		if (wd->rgb) ms_free(wd->rgb);
 		wd->rgb=NULL;
 		wd->rgb_len=0;
+		sws_freeContext(wd->sws);
+		wd->sws=NULL;
 	}
 	else
 		wd=(WinDisplay*)ms_new0(WinDisplay,1);
@@ -325,7 +342,7 @@ static bool_t win_display_init(MSDisplay *obj, MSPicture *fbuf){
 	wd->fb.h=fbuf->h;
 	
 	if (wd->window==NULL){
-		if (obj->window_id!=0){
+		if (obj->use_external_window && obj->window_id!=0){
 			void *p;
 			wd->window=(HWND)obj->window_id;
 			p=(void*)GetWindowLongPtr(wd->window,GWLP_USERDATA);
@@ -335,9 +352,15 @@ static bool_t win_display_init(MSDisplay *obj, MSPicture *fbuf){
 			}else SetWindowLongPtr(wd->window,GWLP_USERDATA,(LONG_PTR)obj);
 		}else{
 			wd->window=create_window(wd->fb.w,wd->fb.h);
+			obj->window_id=(long)wd->window;
 			if (wd->window!=NULL) SetWindowLongPtr(wd->window,GWLP_USERDATA,(LONG_PTR)obj);
 			else return FALSE;
 		}
+	}else if (!obj->use_external_window){
+		/* the window might need to be resized*/
+		RECT cur;
+		GetWindowRect(wd->window,&cur);
+		MoveWindow(wd->window,cur.left, cur.top, wd->fb.w, wd->fb.h,TRUE);
 	}
 	
 	if (wd->ddh==NULL) wd->ddh=DrawDibOpen();
@@ -372,39 +395,20 @@ typedef struct yuv{
 
 
 
-static void yuv420p_to_rgb(MSPicture *src, uint8_t *rgb){
-	struct SwsContext *sws;
+static void yuv420p_to_rgb(WinDisplay *wd, MSPicture *src, uint8_t *rgb){
 	int rgb_stride=-src->w*3;
 	uint8_t *p;
 
 	p=rgb+(src->w*3*(src->h-1));
-	sws=sws_getContext(src->w,src->h,PIX_FMT_YUV420P,
-			src->w,src->h,PIX_FMT_RGB24,
+	if (wd->sws==NULL){
+		wd->sws=sws_getContext(src->w,src->h,PIX_FMT_YUV420P,
+			src->w,src->h,PIX_FMT_BGR24,
 			0, NULL, NULL, NULL);
-	if (sws_scale(sws,src->planes,src->strides, 0,
-           			0, &p, &rgb_stride)<0){
-			ms_error("Error in 420->rgb sws_scale().");
 	}
-	sws_freeContext(sws);
-
-	/*revert colors*/	
-	{
-		int i,j,stride;
-		rgb_t pix;
-		stride=src->w*3;
-		p=rgb;
-		for(i=0;i<src->h;++i){
-			for(j=0;j<stride;j+=3){
-				pix.r=p[j];
-				pix.g=p[j+1];
-				pix.b=p[j+2];
-				p[j]=pix.b;
-				p[j+2]=pix.r;
-			}
-			p+=stride;
-		}
+	if (sws_scale(wd->sws,src->planes,src->strides, 0,
+           			src->h, &p, &rgb_stride)<0){
+		ms_error("Error in 420->rgb sws_scale().");
 	}
-
 }
 
 static void win_display_update(MSDisplay *obj){
@@ -419,7 +423,7 @@ static void win_display_update(MSDisplay *obj){
 		ms_error("Could not get window dc");
 		return;
 	}
-	yuv420p_to_rgb(&wd->fb, wd->rgb);
+	yuv420p_to_rgb(wd, &wd->fb, wd->rgb);
 	memset(&bi,0,sizeof(bi));
 	bi.biSize=sizeof(bi);
 	GetClientRect(wd->window,&rect);
@@ -457,20 +461,15 @@ static void win_display_uninit(MSDisplay *obj){
 	WinDisplay *wd=(WinDisplay*)obj->data;
 	if (wd==NULL)
 		return;
-	if (wd->window && !obj->window_id) DestroyWindow(wd->window);
+	if (wd->window && !obj->use_external_window) DestroyWindow(wd->window);
 	if (wd->ddh) DrawDibClose(wd->ddh);
 	if (wd->fb.planes[0]) ms_free(wd->fb.planes[0]);
 	if (wd->rgb) ms_free(wd->rgb);
+	if (wd->sws) sws_freeContext(wd->sws);
 	ms_free(wd);
 }
 
 bool_t win_display_pollevent(MSDisplay *d, MSDisplayEvent *ev){
-	WinDisplay *wd=(WinDisplay*)d->data;
-	if (wd->new_ev){
-		wd->new_ev=FALSE;
-		*ev=wd->last_rsz;
-		return TRUE;
-	}
 	return FALSE;
 }
 
@@ -507,6 +506,7 @@ MSDisplay *ms_display_new(MSDisplayDesc *desc){
 
 void ms_display_set_window_id(MSDisplay *d, long id){
 	d->window_id=id;
+	d->use_external_window=TRUE;
 }
 
 void ms_display_destroy(MSDisplay *obj){
@@ -514,6 +514,26 @@ void ms_display_destroy(MSDisplay *obj){
 	ms_free(obj);
 }
 
+#ifdef HAVE_SDL
+static MSDisplayDesc *default_display_desc=&ms_sdl_display_desc;
+#elif defined(WIN32)
+static MSDisplayDesc *default_display_desc=&ms_win_display_desc;
+#else
+static MSDisplayDesc *default_display_desc=NULL;
+#endif
+
+void ms_display_desc_set_default(MSDisplayDesc *desc){
+	default_display_desc=desc;
+}
+
+MSDisplayDesc * ms_display_desc_get_default(void){
+	return default_display_desc;
+}
+
+void ms_display_desc_set_default_window_id(MSDisplayDesc *desc, long id){
+	desc->default_window_id=id;
+}
+
 typedef struct VideoOut
 {
 	AVRational ratio;
@@ -521,12 +541,15 @@ typedef struct VideoOut
 	MSPicture local_pic;
 	MSRect local_rect;
 	mblk_t *local_msg;
+	MSVideoSize prevsize;
 	int corner;
 	struct SwsContext *sws1;
 	struct SwsContext *sws2;
 	MSDisplay *display;
 	bool_t own_display;
 	bool_t ready;
+	bool_t autofit;
+	bool_t mirror;
 } VideoOut;
 
 
@@ -535,8 +558,8 @@ typedef struct VideoOut
 static void set_corner(VideoOut *s, int corner)
 {
 	s->corner=corner;
-	s->local_pic.w=s->fbuf.w/SCALE_FACTOR;
-	s->local_pic.h=s->fbuf.h/SCALE_FACTOR;
+	s->local_pic.w=(s->fbuf.w/SCALE_FACTOR) & ~0x1;
+	s->local_pic.h=(s->fbuf.h/SCALE_FACTOR) & ~0x1;
 	if (corner==1)
 	{
 	/* top left corner */
@@ -573,13 +596,10 @@ static void set_corner(VideoOut *s, int corner)
 }
 
 static void set_vsize(VideoOut *s, MSVideoSize *sz){
-	if (s->ratio.num!=0){
-		sz->width=sz->width & (~0x1);
-		sz->height=sz->width*s->ratio.den/s->ratio.num;
-	}
-	s->fbuf.w=sz->width;
-	s->fbuf.h=sz->height;
-	set_corner(s, s->corner);
+	s->fbuf.w=sz->width & ~0x1;
+	s->fbuf.h=sz->height & ~0x1;
+	set_corner(s,s->corner);
+	ms_message("Video size set to %ix%i",s->fbuf.w,s->fbuf.h);
 }
 
 static void video_out_init(MSFilter  *f){
@@ -589,6 +609,8 @@ static void video_out_init(MSFilter  *f){
 	obj->ratio.den=9;
 	def_size.width=MS_VIDEO_SIZE_CIF_W;
 	def_size.height=MS_VIDEO_SIZE_CIF_H;
+	obj->prevsize.width=0;
+	obj->prevsize.height=0;
 	obj->local_msg=NULL;
 	obj->corner=0;
 	obj->sws1=NULL;
@@ -596,6 +618,8 @@ static void video_out_init(MSFilter  *f){
 	obj->display=NULL;
 	obj->own_display=FALSE;
 	obj->ready=FALSE;
+	obj->autofit=FALSE;
+	obj->mirror=FALSE;
 	set_vsize(obj,&def_size);
 	f->data=obj;
 }
@@ -620,15 +644,10 @@ static void video_out_uninit(MSFilter *f){
 	ms_free(obj);
 }
 
-
-static void video_out_preprocess(MSFilter *f){
+static void video_out_prepare(MSFilter *f){
 	VideoOut *obj=(VideoOut*)f->data;
 	if (obj->display==NULL){
-#ifndef WIN32
-		obj->display=ms_display_new(&ms_sdl_display_desc);
-#else
-		obj->display=ms_display_new(&ms_win_display_desc);
-#endif
+		obj->display=ms_display_new(default_display_desc);
 		obj->own_display=TRUE;
 	}
 	if (!ms_display_init(obj->display,&obj->fbuf)){
@@ -647,10 +666,32 @@ static void video_out_preprocess(MSFilter *f){
 		freemsg(obj->local_msg);
 		obj->local_msg=NULL;
 	}
+	set_corner(obj,obj->corner);
 	obj->ready=TRUE;
 }
 
-static void video_out_postprocess(MSFilter *f){
+static int video_out_handle_resizing(MSFilter *f, void *data){
+	VideoOut *s=(VideoOut*)f->data;
+	MSDisplay *disp=s->display;
+	if (disp!=NULL){
+		MSDisplayEvent ev;
+		if (ms_display_poll_event(disp,&ev)){
+			if (ev.evtype==MS_DISPLAY_RESIZE_EVENT){
+				MSVideoSize sz;
+				sz.width=ev.w;
+				sz.height=ev.h;
+				ms_filter_lock(f);
+				set_vsize(s,&sz);
+				s->ready=FALSE;
+				ms_filter_unlock(f);
+			}
+		}
+	}
+	return 0;
+}
+
+static void video_out_preprocess(MSFilter *f){
+	video_out_prepare(f);
 }
 
 
@@ -658,11 +699,8 @@ static void video_out_process(MSFilter *f){
 	VideoOut *obj=(VideoOut*)f->data;
 	mblk_t *inm;
 
-#ifdef WIN32
-	video_out_handle_resize(f, NULL);
-#endif
-
 	ms_filter_lock(f);
+	if (!obj->ready) video_out_prepare(f);
 	if (obj->display==NULL){
 		ms_filter_unlock(f);
 		if (f->inputs[0]!=NULL)
@@ -704,6 +742,27 @@ static void video_out_process(MSFilter *f){
 	if (f->inputs[0]!=NULL && (inm=ms_queue_peek_last(f->inputs[0]))!=0) {
 		MSPicture src;
 		if (yuv_buf_init_from_mblk(&src,inm)==0){
+			MSVideoSize cur,newsize;
+			cur.width=obj->fbuf.w;
+			cur.height=obj->fbuf.h;
+			newsize.width=src.w;
+			newsize.height=src.h;
+			if (obj->autofit && !ms_video_size_equal(newsize,obj->prevsize) ) {
+				MSVideoSize qvga_size;
+				qvga_size.width=MS_VIDEO_SIZE_QVGA_W;
+				qvga_size.height=MS_VIDEO_SIZE_QVGA_H;
+				obj->prevsize=newsize;
+				/*don't resize less than QVGA, it is too small*/
+				if (ms_video_size_greater_than(qvga_size,newsize)){
+					newsize.width=MS_VIDEO_SIZE_QVGA_W;
+					newsize.height=MS_VIDEO_SIZE_QVGA_H;
+				}
+				if (!ms_video_size_equal(newsize,cur)){
+					set_vsize(obj,&newsize);
+					ms_message("autofit: new size is %ix%i",newsize.width,newsize.height);
+					video_out_prepare(f);
+				}
+			}
 			if (obj->sws1==NULL){
 				obj->sws1=sws_getContext(src.w,src.h,PIX_FMT_YUV420P,
 				obj->fbuf.w,obj->fbuf.h,PIX_FMT_YUV420P,
@@ -714,6 +773,7 @@ static void video_out_process(MSFilter *f){
             			src.h, obj->fbuf.planes, obj->fbuf.strides)<0){
 				ms_error("Error in sws_scale().");
 			}
+			if (obj->mirror && !mblk_get_precious_flag(inm)) yuv_buf_mirror(&obj->fbuf);
 			ms_display_unlock(obj->display);
 		}
 		ms_queue_flush(f->inputs[0]);
@@ -734,18 +794,14 @@ static void video_out_process(MSFilter *f){
 				corner.planes,corner.strides,roi);
 		ms_display_unlock(obj->display);
 	}
-	
 	ms_display_update(obj->display);
 	ms_filter_unlock(f);
 }
 
 static int video_out_set_vsize(MSFilter *f,void *arg){
 	VideoOut *s=(VideoOut*)f->data;
-	bool_t reconfigure;
 	ms_filter_lock(f);
-	reconfigure=s->ready;
 	set_vsize(s,(MSVideoSize*)arg);
-	if (reconfigure) video_out_preprocess(f);
 	ms_filter_unlock(f);
 	return 0;
 }
@@ -756,52 +812,59 @@ static int video_out_set_display(MSFilter *f,void *arg){
 	return 0;
 }
 
-static int video_out_handle_resize(MSFilter *f, void *arg){
+static int video_out_auto_fit(MSFilter *f, void *arg){
 	VideoOut *s=(VideoOut*)f->data;
-	MSDisplay *disp=s->display;
-	if (disp!=NULL){
-		MSDisplayEvent ev;
-		if (ms_display_poll_event(disp,&ev)){
-			if (ev.evtype==MS_DISPLAY_RESIZE_EVENT){
-				MSVideoSize sz;
-				sz.width=ev.w;
-				sz.height=ev.h;
-#ifndef WIN32
-				video_out_set_vsize(f,&sz);
-#endif
-			}
-		}
-	}
+	s->autofit=*(int*)arg;
 	return 0;
 }
 
 static int video_out_set_corner(MSFilter *f,void *arg){
 	VideoOut *s=(VideoOut*)f->data;
-	set_corner(s, *(int*)arg);
-#if 1
 	ms_filter_lock(f);
-	ms_display_lock(s->display);
-	{
-	  int w=s->fbuf.w;
-	  int h=s->fbuf.h;
-	  int ysize=w*h;
-	  int usize=ysize/4;
-	  
-	  memset(s->fbuf.planes[0], 0, ysize);
-	  memset(s->fbuf.planes[1], 0, usize);
-	  memset(s->fbuf.planes[2], 0, usize);
+	set_corner(s, *(int*)arg);
+	if (s->display){
+		ms_display_lock(s->display);
+		{
+		int w=s->fbuf.w;
+		int h=s->fbuf.h;
+		int ysize=w*h;
+		int usize=ysize/4;
+		
+		memset(s->fbuf.planes[0], 0, ysize);
+		memset(s->fbuf.planes[1], 0, usize);
+		memset(s->fbuf.planes[2], 0, usize);
+		}
+		ms_display_unlock(s->display);
 	}
-	ms_display_unlock(s->display);
 	ms_filter_unlock(f);
-#endif
 	return 0;
 }
 
+static int video_out_enable_mirroring(MSFilter *f,void *arg){
+	VideoOut *s=(VideoOut*)f->data;
+	s->mirror=*(int*)arg;
+	return 0;
+}
+
+static int video_out_get_native_window_id(MSFilter *f, void*arg){
+	VideoOut *s=(VideoOut*)f->data;
+	unsigned long *id=(unsigned long*)arg;
+	*id=0;
+	if (s->display){
+		*id=s->display->window_id;
+		return 0;
+	}
+	return -1;
+}
+
 static MSFilterMethod methods[]={
 	{	MS_FILTER_SET_VIDEO_SIZE	,	video_out_set_vsize },
 	{	MS_VIDEO_OUT_SET_DISPLAY	,	video_out_set_display},
-	{ 	MS_VIDEO_OUT_HANDLE_RESIZING	,	video_out_handle_resize},
 	{	MS_VIDEO_OUT_SET_CORNER 	,	video_out_set_corner},
+	{	MS_VIDEO_OUT_AUTO_FIT		,	video_out_auto_fit},
+	{	MS_VIDEO_OUT_HANDLE_RESIZING	,	video_out_handle_resizing},
+	{	MS_VIDEO_OUT_ENABLE_MIRRORING	,	video_out_enable_mirroring},
+	{	MS_VIDEO_OUT_GET_NATIVE_WINDOW_ID,	video_out_get_native_window_id},
 	{	0	,NULL}
 };
 
@@ -810,7 +873,7 @@ static MSFilterMethod methods[]={
 MSFilterDesc ms_video_out_desc={
 	MS_VIDEO_OUT_ID,
 	"MSVideoOut",
-	"A generic video display",
+	N_("A generic video display"),
 	MS_FILTER_OTHER,
 	NULL,
 	2,
@@ -818,7 +881,7 @@ MSFilterDesc ms_video_out_desc={
 	video_out_init,
 	video_out_preprocess,
 	video_out_process,
-	video_out_postprocess,
+	NULL,
 	video_out_uninit,
 	methods
 };
@@ -828,14 +891,13 @@ MSFilterDesc ms_video_out_desc={
 MSFilterDesc ms_video_out_desc={
 	.id=MS_VIDEO_OUT_ID,
 	.name="MSVideoOut",
-	.text="A generic video display",
+	.text=N_("A generic video display"),
 	.category=MS_FILTER_OTHER,
 	.ninputs=2,
 	.noutputs=0,
 	.init=video_out_init,
 	.preprocess=video_out_preprocess,
 	.process=video_out_process,
-	.postprocess=video_out_postprocess,
 	.uninit=video_out_uninit,
 	.methods=methods
 };
