@@ -21,14 +21,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msrtp.h"
-
-
-#if (defined(WIN32) || defined(_WIN32_WCE))
-/* avoid double declaration of ms_win_display_desc */
-#define MS_VIDEO_OUT_HANDLE_RESIZING MS_FILTER_METHOD_NO_ARG(MS_VIDEO_OUT_ID,1)
-#else
 #include "mediastreamer2/msvideoout.h"
-#endif
+
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
@@ -143,9 +137,11 @@ static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
 }
 
 void video_stream_iterate(VideoStream *stream){
+	
 	if (stream->output!=NULL)
 		ms_filter_call_method_noarg(stream->output,
 			MS_VIDEO_OUT_HANDLE_RESIZING);
+	
 	if (stream->evq){
 		OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
 		if (ev!=NULL){
@@ -170,29 +166,41 @@ VideoStream *video_stream_new(int locport, bool_t use_ipv6){
 	stream->evq=ortp_ev_queue_new();
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
 	rtp_session_register_event_queue(stream->session,stream->evq);
+	stream->sent_vsize.width=MS_VIDEO_SIZE_CIF_W;
+	stream->sent_vsize.height=MS_VIDEO_SIZE_CIF_H;
 	return stream;
 }
 
+void video_stream_set_sent_video_size(VideoStream *stream, MSVideoSize vsize){
+	stream->sent_vsize=vsize;
+}
+
 void video_stream_set_relay_session_id(VideoStream *stream, const char *id){
 	ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
 
+void video_stream_enable_self_view(VideoStream *stream, bool_t val){
+	MSFilter *out=stream->output;
+	stream->corner=val ? 0 : -1;
+	if (out){
+		ms_filter_call_method(out,MS_VIDEO_OUT_SET_CORNER,&stream->corner);
+	}
+}
 
 void video_stream_enable_adaptive_bitrate_control(VideoStream *s, bool_t yesno){
 	s->adapt_bitrate=yesno;
 }
 
 int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,
-	int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *cam)
-{
+	int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *cam){
 	PayloadType *pt;
 	RtpSession *rtps=stream->session;
 	MSPixFmt format;
-	MSVideoSize vsize;
+	MSVideoSize vsize,cam_vsize,disp_size;
 	float fps=15;
-
-	vsize.height=MS_VIDEO_SIZE_CIF_H;
-	vsize.width=MS_VIDEO_SIZE_CIF_W;
+	int tmp;
+	JBParameters jbp;
+	const int socket_buf_size=2000000;
 
 	pt=rtp_profile_get_payload(profile,payload);
 	if (pt==NULL){
@@ -217,6 +225,13 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 
 	rtp_session_set_recv_buf_size(stream->session,MAX_RTP_SIZE);
 
+	rtp_session_get_jitter_buffer_params(stream->session,&jbp);
+	jbp.max_packets=1000;//needed for high resolution video
+	rtp_session_set_jitter_buffer_params(stream->session,&jbp);
+
+	rtp_session_set_rtp_socket_recv_buffer_size(stream->session,socket_buf_size);
+	rtp_session_set_rtp_socket_send_buffer_size(stream->session,socket_buf_size);
+
 	/* creates two rtp filters to recv send streams (remote part) */
 	if (remport>0) ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SET_SESSION,stream->session);
 	
@@ -239,6 +254,8 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,pt->send_fmtp);
 	}
 	ms_filter_call_method(stream->encoder,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+	vsize=ms_video_size_min(vsize,stream->sent_vsize);
+	ms_filter_call_method(stream->encoder,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	ms_filter_call_method(stream->encoder,MS_FILTER_GET_FPS,&fps);
 	ms_message("Setting vsize=%ix%i, fps=%f",vsize.width,vsize.height,fps);
 	/* configure the filters */
@@ -254,26 +271,24 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		/*set it to the pixconv */
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
 
-		ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+		ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&cam_vsize);
 	
-		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
-		  
+		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&cam_vsize);
 	}
 
-	ms_filter_call_method(stream->encoder,MS_FILTER_GET_VIDEO_SIZE,&vsize);
 	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 
 	/*force the decoder to output YUV420P */
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->decoder,MS_FILTER_SET_PIX_FMT,&format);
 
-
-	/*ask the video display to always output CIF */
-	vsize.height=MS_VIDEO_SIZE_CIF_H;
-	vsize.width=MS_VIDEO_SIZE_CIF_W;
-
-	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	disp_size.width=MS_VIDEO_SIZE_CIF_W;
+	disp_size.height=MS_VIDEO_SIZE_CIF_H;
+	tmp=1;
+	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&disp_size);
+	ms_filter_call_method(stream->output,MS_VIDEO_OUT_AUTO_FIT,&tmp);
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
+	ms_filter_call_method(stream->output,MS_VIDEO_OUT_SET_CORNER,&stream->corner);
 
 	if (pt->recv_fmtp!=NULL)
 		ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
@@ -291,7 +306,8 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	ms_filter_link(stream->tee,1,stream->output,1);
 
 	/* create the ticker */
-	stream->ticker = ms_ticker_new(); 
+	stream->ticker = ms_ticker_new();
+	ms_ticker_set_name(stream->ticker,"Video MSTicker");
 	/* attach it the graph */
 	ms_ticker_attach (stream->ticker, stream->source);
 	return 0;
@@ -330,14 +346,22 @@ void video_stream_set_rtcp_information(VideoStream *st, const char *cname, const
 	}
 }
 
+unsigned long video_stream_get_native_window_id(VideoStream *stream){
+	unsigned long id;
+	if (stream->output){
+		if (ms_filter_call_method(stream->output,MS_VIDEO_OUT_GET_NATIVE_WINDOW_ID,&id)==0)
+			return id;
+	}
+	return 0;
+}
 
 
-VideoStream * video_preview_start(MSWebCam *device){
+VideoStream * video_preview_start(MSWebCam *device, MSVideoSize disp_size){
 	VideoStream *stream = (VideoStream *)ms_new0 (VideoStream, 1);
+	MSVideoSize vsize=disp_size;
 	MSPixFmt format;
-	MSVideoSize vsize;
-	vsize.width=MS_VIDEO_SIZE_CIF_W;
-	vsize.height=MS_VIDEO_SIZE_CIF_H;
+	float fps=29.97;
+	int mirroring=1;
 
 	/* creates the filters */
 	stream->source = ms_web_cam_create_reader(device);
@@ -345,9 +369,10 @@ VideoStream * video_preview_start(MSWebCam *device){
 
 
 	/* configure the filters */
+	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
-	
 	if (format==MS_MJPEG){
 		stream->pixconv=ms_filter_new(MS_MJPEG_DEC_ID);
 	}else{
@@ -358,14 +383,16 @@ VideoStream * video_preview_start(MSWebCam *device){
 
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
-	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&disp_size);
+	ms_filter_call_method(stream->output,MS_VIDEO_OUT_ENABLE_MIRRORING,&mirroring);
 	/* and then connect all */
 
 	ms_filter_link(stream->source,0, stream->pixconv,0);
 	ms_filter_link(stream->pixconv, 0, stream->output, 0);
 
 	/* create the ticker */
-	stream->ticker = ms_ticker_new(); 
+	stream->ticker = ms_ticker_new();
+	ms_ticker_set_name(stream->ticker,"Video MSTicker");
 	ms_ticker_attach (stream->ticker, stream->source);
 	return stream;
 }
