@@ -48,6 +48,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	int err;
 	int optval = 1;
 	ortp_socket_t sock=-1;
+
 #ifdef ORTP_INET6
 	char num[8];
 	struct addrinfo hints, *res0, *res;
@@ -69,8 +70,9 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	
 	for (res = res0; res; res = res->ai_next) {
 		sock = socket(res->ai_family, res->ai_socktype, 0);
-		if (sock < 0)
-		  continue;
+		if (sock==-1)
+			continue;
+
 		if (reuse_addr){
 			err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
 					(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
@@ -82,13 +84,12 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 
 		*sock_family=res->ai_family;
 		err = bind (sock, res->ai_addr, res->ai_addrlen);
-		if (err != 0)
-		  {
-		    ortp_warning ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr,port, getSocketError());
-		    close_socket (sock);
+		if (err != 0){
+			ortp_warning ("Fail to bind rtp socket to (addr=%s port=%i) : %s.", addr,port, getSocketError());
+			close_socket (sock);
 			sock=-1;
-		    continue;
-		  }
+			continue;
+		}
 #ifndef __hpux
 		switch (res->ai_family)
 		  {
@@ -99,12 +100,11 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 			  mreq.imr_multiaddr.s_addr = ((struct sockaddr_in *) res->ai_addr)->sin_addr.s_addr;
 			  mreq.imr_interface.s_addr = INADDR_ANY;
 			  err = setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (SOCKET_OPTION_VALUE) &mreq, sizeof(mreq));
-			  if (err < 0)
-			    {
-			      ortp_warning ("Fail to join address group: %s.", getSocketError());
-			      close_socket (sock);
-					sock=-1;
-			      continue;
+			  if (err < 0){
+				ortp_warning ("Fail to join address group: %s.", getSocketError());
+				close_socket (sock);
+				sock=-1;
+				continue;
 			    }
 			}
 		      break;
@@ -119,7 +119,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 			    {
 			      ortp_warning ("Fail to join address group: %s.", getSocketError());
 			      close_socket (sock);
-					sock=-1;
+			      sock=-1;
 			      continue;
 			    }
 			}
@@ -136,13 +136,13 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	if (err < 0)
 	{
 		ortp_warning ("Error in socket address:%s.", getSocketError());
-		return err;
+		return -1;
 	}
 	saddr.sin_port = htons (port);
 
 	sock = socket (PF_INET, SOCK_DGRAM, 0);
-	
-	if (sock<0) return -1;
+	if (sock==-1) return -1;
+
 	if (reuse_addr){
 		err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
 				(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
@@ -163,7 +163,7 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 		return -1;
 	}
 #endif
-	if (sock>=0){
+	if (sock!=-1){
 		set_non_blocking_socket (sock);
 	}
 	return sock;
@@ -217,7 +217,7 @@ static ortp_socket_t create_and_bind_random(const char *localip, int *sock_famil
 		while ((localport < 5000) || (localport > 0xffff));
 		/*do not set REUSEADDR in case of random allocation */
 		sock = create_and_bind(localip, localport, sock_family,FALSE);
-		if (sock>=0) {
+		if (sock!=-1) {
 			*port=localport;
 			return sock;
 		}
@@ -255,14 +255,14 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 		sock=create_and_bind(addr,port,&sockfamily,TRUE);
 	else
 		sock=create_and_bind_random(addr,&sockfamily,&port);
-	if (sock>=0){
+	if (sock!=-1){
 		set_socket_sizes(sock,session->rtp.snd_socket_size,session->rtp.rcv_socket_size);
 		session->rtp.sockfamily=sockfamily;
 		session->rtp.socket=sock;
 		session->rtp.loc_port=port;
 		/*try to bind rtcp port */
 		sock=create_and_bind(addr,port+1,&sockfamily,TRUE);
-		if (sock>=0){
+		if (sock!=-1){
 			session->rtcp.sockfamily=sockfamily;
 			session->rtcp.socket=sock;
 		}else{
@@ -901,10 +901,10 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 				  &addrlen);
 		if (error > 0){
 			if (session->symmetric_rtp && !sock_connected){
-				/* store the sender rtp address to do symmetric RTP */
-				memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
-				session->rtp.rem_addrlen=addrlen;
 				if (session->use_connect){
+					/* store the sender rtp address to do symmetric RTP */
+					memcpy(&session->rtp.rem_addr,&remaddr,addrlen);
+					session->rtp.rem_addrlen=addrlen;
 					if (try_connect(sockfd,(struct sockaddr*)&remaddr,addrlen))
 						session->flags|=RTP_SOCKET_CONNECTED;
 				}
